@@ -1,12 +1,5 @@
 #include <config.h>
 #include <gtk/gtk.h>
-#ifdef GDK_WINDOWING_X11
-#include <gdk/x11/gdkx.h>
-#elif defined (GDK_WINDOWING_WIN32)
-#include <gdk/win32/gdkwin32.h>
-#elif defined (GDK_WINDOWING_FB)
-#include <gdk/linux-fb/gdkfb.h>
-#endif
 
 int close_app(GtkWidget *widget, gpointer data)
 {
@@ -62,7 +55,7 @@ int configure_cb(GtkWidget *drawing_area, GdkEventConfigure *evt, gpointer data)
       GdkWindow *root;
       GdkPixbuf *new_pixbuf;
 
-      root = GDK_ROOT_PARENT();
+      root = gdk_get_default_root_window ();
       new_pixbuf = gdk_pixbuf_get_from_drawable(NULL, root, NULL,
 						0, 0, 0, 0, evt->width, evt->height);
       gtk_object_set_data(GTK_OBJECT(drawing_area), "pixbuf", new_pixbuf);
@@ -86,11 +79,10 @@ int main(int argc, char **argv)
 
    gtk_init(&argc, &argv);   
    gdk_rgb_set_verbose(TRUE);
-   gdk_rgb_init();
 
-   gtk_widget_set_default_colormap(gdk_rgb_get_cmap());
+   gtk_widget_set_default_colormap(gdk_rgb_get_colormap());
 
-   root = GDK_ROOT_PARENT();
+   root = gdk_get_default_root_window ();
    pixbuf = gdk_pixbuf_get_from_drawable(NULL, root, NULL,
 					 0, 0, 0, 0, 150, 160);
    
@@ -104,9 +96,9 @@ int main(int argc, char **argv)
    gtk_container_add(GTK_CONTAINER(window), vbox);  
    
    drawing_area = gtk_drawing_area_new();
-   gtk_widget_set_usize(GTK_WIDGET(drawing_area),
-                        gdk_pixbuf_get_width (pixbuf),
-                        gdk_pixbuf_get_height (pixbuf));
+   gtk_widget_set_size_request (GTK_WIDGET(drawing_area),
+                                gdk_pixbuf_get_width (pixbuf),
+                                gdk_pixbuf_get_height (pixbuf));
    gtk_signal_connect(GTK_OBJECT(drawing_area), "expose_event",
                       GTK_SIGNAL_FUNC(expose_cb), NULL);
 
