@@ -393,7 +393,7 @@ new_testrgb_window (GdkPixbuf *pixbuf, gchar *title)
 	drawing_area = gtk_drawing_area_new ();
 
 	temp_box = gtk_hbox_new (FALSE, 0);
-	gtk_widget_set_usize (GTK_WIDGET(drawing_area), w, h);
+	gtk_widget_set_size_request (GTK_WIDGET(drawing_area), w, h);
 	gtk_box_pack_start (GTK_BOX (temp_box), drawing_area, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), temp_box, FALSE, FALSE, 0);
 	
@@ -449,19 +449,15 @@ update_timeout(gpointer data)
                 }
                         
                         
-        } else { /* Really done */ 
-
-                GdkPixbuf *pixbuf = gdk_pixbuf_loader_get_pixbuf (status->loader); 
-                new_testrgb_window (pixbuf, "After progressive load"); 
-                done = TRUE; 
-
-        }
-
+        } 
+	else 
+	  done = TRUE;
+	  
 	if (done) {
-                gtk_widget_queue_draw(*status->rgbwin);
                 /* ignoring errors, we should not do that. */
 		gdk_pixbuf_loader_close (GDK_PIXBUF_LOADER (status->loader), NULL);
-		g_object_unref (G_OBJECT(status->loader));
+		gtk_widget_queue_draw (*status->rgbwin);
+		g_object_unref (G_OBJECT (status->loader));
 		fclose (status->imagefile);
 		g_free (status->buf);
 	}
@@ -520,9 +516,7 @@ main (int argc, char **argv)
 
 	gdk_rgb_set_verbose (TRUE);
 
-	gdk_rgb_init ();
-
-	gtk_widget_set_default_colormap (gdk_rgb_get_cmap ());
+	gtk_widget_set_default_colormap (gdk_rgb_get_colormap ());
 
 	{
 		char *tbf_readlen = getenv("TBF_READLEN");
@@ -543,7 +537,8 @@ main (int argc, char **argv)
 	i = 1;
 	if (argc == 1) {
                 const gchar*** xpmp;
-                
+                GError *error = NULL;
+		
 		pixbuf = gdk_pixbuf_new_from_data (default_image, GDK_COLORSPACE_RGB, FALSE, 8,
 						   DEFAULT_WIDTH, DEFAULT_HEIGHT, DEFAULT_WIDTH * 3,
 						   NULL, NULL);
@@ -557,11 +552,18 @@ main (int argc, char **argv)
                 }
 
                 /* Test loading from inline data. */
-                pixbuf = gdk_pixbuf_new_from_inline (apple_red, FALSE, -1, NULL);
-                new_testrgb_window (pixbuf, "Red apple from inline data");
+                pixbuf = gdk_pixbuf_new_from_stream (-1, apple_red, FALSE, &error);
+		if (!pixbuf)
+		  {
+		    fprintf (stderr, "failed to construct \"red apple\" pixbuf: %s\n",
+			     error->message);
+		    g_error_free (error);
+		  }
+		else
+		  new_testrgb_window (pixbuf, "Red apple from inlined RLE data");
 
-                pixbuf = gdk_pixbuf_new_from_inline (gnome_foot, TRUE, sizeof (gnome_foot), NULL);
-                new_testrgb_window (pixbuf, "Foot from inline data");
+                pixbuf = gdk_pixbuf_new_from_stream (sizeof (gnome_foot), gnome_foot, TRUE, NULL);
+                new_testrgb_window (pixbuf, "GNOME Foot from inlined RLE data");
                 
 		found_valid = TRUE;
 	} else {
@@ -597,18 +599,14 @@ main (int argc, char **argv)
 			status.rgbwin = &rgb_window;
 
 			status.buf = g_malloc (readlen);
-                        g_signal_connect_data (G_OBJECT(pixbuf_loader),
-                                               "area_prepared",
-                                               GTK_SIGNAL_FUNC(progressive_prepared_callback),
-                                               &rgb_window,
-                                               NULL, FALSE, FALSE);
-
-                        g_signal_connect_data (G_OBJECT(pixbuf_loader),
-                                               "area_updated",
-                                               GTK_SIGNAL_FUNC(progressive_updated_callback),
-                                               &rgb_window,
-                                               NULL, FALSE, FALSE);
-
+                        g_signal_connect (G_OBJECT(pixbuf_loader),
+					  "area_prepared",
+					  GTK_SIGNAL_FUNC(progressive_prepared_callback),
+					  &rgb_window);
+                        g_signal_connect (G_OBJECT(pixbuf_loader),
+					  "area_updated",
+					  GTK_SIGNAL_FUNC(progressive_updated_callback),
+					  &rgb_window);
 			
                         status.imagefile = fopen (argv[1], "r");
                         g_assert (status.imagefile != NULL);
