@@ -337,7 +337,16 @@ gdk_pixbuf__jpeg_image_begin_load (ModulePreparedNotifyFunc prepared_func,
 	/* create libjpeg structures */
 	jpeg_create_decompress (&context->cinfo);
 
-	context->cinfo.src = (struct jpeg_source_mgr *) g_new0 (my_source_mgr, 1);
+	context->cinfo.src = (struct jpeg_source_mgr *) g_try_malloc (sizeof (my_source_mgr));
+	if (!context->cinfo.src) {
+	  g_set_error (error,
+		       GDK_PIXBUF_ERROR,
+		       GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
+		       _("Couldn't allocate memory for loading JPEG file"));
+	  return NULL;
+	}
+	memset (context->cinfo.src, 0, sizeof (my_source_mgr));
+       
 	src = (my_src_ptr) context->cinfo.src;
 
 	context->cinfo.err = jpeg_std_error (&context->jerr.pub);
@@ -693,7 +702,14 @@ gdk_pixbuf__jpeg_image_save (FILE          *f,
        g_return_val_if_fail (pixels != NULL, FALSE);
 
        /* allocate a small buffer to convert image data */
-       buf = g_malloc (w * 3 * sizeof (guchar));
+       buf = g_try_malloc (w * 3 * sizeof (guchar));
+       if (!buf) {
+	       g_set_error (error,
+			    GDK_PIXBUF_ERROR,
+			    GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
+			    _("Couldn't allocate memory for loading JPEG file"));
+	       return FALSE;
+       }
 
        /* set up error handling */
        jerr.pub.error_exit = fatal_error_handler;
