@@ -51,15 +51,6 @@ struct _GdkErrorTrap
   gint error_code;
 };
 
-/* 
- * Private function declarations
- */
-static void	    gdk_exit_func		 (void);
-
-GdkFilterReturn gdk_wm_protocols_filter (GdkXEvent *xev,
-					 GdkEvent  *event,
-					 gpointer   data);
-
 /* Private variable declarations
  */
 static int gdk_initialized = 0;			    /* 1 if the library is initialized,
@@ -69,12 +60,13 @@ static int gdk_initialized = 0;			    /* 1 if the library is initialized,
 static GSList *gdk_error_traps = NULL;               /* List of error traps */
 static GSList *gdk_error_trap_free_list = NULL;      /* Free list */
 
+static gchar  *gdk_progclass = NULL;
+
 #ifdef G_ENABLE_DEBUG
 static const GDebugKey gdk_debug_keys[] = {
   {"events",	    GDK_DEBUG_EVENTS},
   {"misc",	    GDK_DEBUG_MISC},
   {"dnd",	    GDK_DEBUG_DND},
-  {"color-context", GDK_DEBUG_COLOR_CONTEXT},
   {"xim",	    GDK_DEBUG_XIM}
 };
 
@@ -216,20 +208,26 @@ gdk_arg_context_parse (GdkArgContext *context, gint *argc, gchar ***argv)
 static void
 gdk_arg_debug_cb (const char *key, const char *value, gpointer user_data)
 {
-  gdk_debug_flags |= g_parse_debug_string (value,
-					   (GDebugKey *) gdk_debug_keys,
-					   gdk_ndebug_keys);
+  _gdk_debug_flags |= g_parse_debug_string (value,
+					    (GDebugKey *) gdk_debug_keys,
+					    gdk_ndebug_keys);
 }
 
 static void
 gdk_arg_no_debug_cb (const char *key, const char *value, gpointer user_data)
 {
-  gdk_debug_flags &= ~g_parse_debug_string (value,
-					    (GDebugKey *) gdk_debug_keys,
-					    gdk_ndebug_keys);
+  _gdk_debug_flags &= ~g_parse_debug_string (value,
+					     (GDebugKey *) gdk_debug_keys,
+					     gdk_ndebug_keys);
 }
 #endif /* G_ENABLE_DEBUG */
 
+static void
+gdk_arg_class_cb (const char *key, const char *value, gpointer user_data)
+{
+  gdk_set_program_class (value);
+}
+
 static void
 gdk_arg_name_cb (const char *key, const char *value, gpointer user_data)
 {
@@ -237,6 +235,7 @@ gdk_arg_name_cb (const char *key, const char *value, gpointer user_data)
 }
 
 static GdkArgDesc gdk_args[] = {
+  { "class" ,       GDK_ARG_STRING,   NULL, gdk_arg_class_cb    },
   { "name",         GDK_ARG_STRING,   NULL, gdk_arg_name_cb     },
 #ifdef G_ENABLE_DEBUG
   { "gdk-debug",    GDK_ARG_CALLBACK, NULL, gdk_arg_debug_cb    },
@@ -309,7 +308,7 @@ gdk_init_check (int    *argc,
   {
     gchar *debug_string = getenv("GDK_DEBUG");
     if (debug_string != NULL)
-      gdk_debug_flags = g_parse_debug_string (debug_string,
+      _gdk_debug_flags = g_parse_debug_string (debug_string,
 					      (GDebugKey *) gdk_debug_keys,
 					      gdk_ndebug_keys);
   }
@@ -334,19 +333,13 @@ gdk_init_check (int    *argc,
   if (!result)
     return FALSE;
   
-  g_atexit (gdk_exit_func);
-  
-  gdk_events_init ();
-  gdk_visual_init ();
+  _gdk_visual_init ();
   _gdk_windowing_window_init ();
   _gdk_windowing_image_init ();
-  gdk_input_init ();
-  gdk_dnd_init ();
+  _gdk_events_init ();
+  _gdk_input_init ();
+  _gdk_dnd_init ();
 
-#ifdef USE_XIM
-  gdk_im_open ();
-#endif
-  
   gdk_initialized = 1;
 
   return TRUE;
@@ -389,6 +382,13 @@ gdk_exit (gint errorcode)
   exit (errorcode);
 }
 
+#if 0
+
+/* This is disabled, but the code isn't removed, because we might
+ * want to have some sort of explicit way to shut down GDK cleanly
+ * at some point in the future.
+ */
+
 /*
  *--------------------------------------------------------------
  * gdk_exit_func
@@ -420,22 +420,17 @@ gdk_exit_func (void)
   
   if (gdk_initialized)
     {
-#ifdef USE_XIM
-      /* cleanup IC */
-      gdk_ic_cleanup ();
-      /* close IM */
-      gdk_im_close ();
-#endif
-      gdk_image_exit ();
-      gdk_input_exit ();
-      gdk_key_repeat_restore ();
+      _gdk_image_exit ();
+      _gdk_input_exit ();
 
-      gdk_windowing_exit ();
+      _gdk_windowing_exit ();
       
       gdk_initialized = 0;
     }
 }
 
+#endif
+
 /*************************************************************
  * gdk_error_trap_push:
  *     Push an error trap. X errors will be trapped until
@@ -467,11 +462,11 @@ gdk_error_trap_push (void)
   gdk_error_traps = node;
   
   trap = node->data;
-  trap->error_code = gdk_error_code;
-  trap->error_warnings = gdk_error_warnings;
+  trap->error_code = _gdk_error_code;
+  trap->error_warnings = _gdk_error_warnings;
 
-  gdk_error_code = 0;
-  gdk_error_warnings = 0;
+  _gdk_error_code = 0;
+  _gdk_error_warnings = 0;
 }
 
 /*************************************************************
@@ -498,193 +493,15 @@ gdk_error_trap_pop (void)
   node->next = gdk_error_trap_free_list;
   gdk_error_trap_free_list = node;
   
-  result = gdk_error_code;
+  result = _gdk_error_code;
   
   trap = node->data;
-  gdk_error_code = trap->error_code;
-  gdk_error_warnings = trap->error_warnings;
+  _gdk_error_code = trap->error_code;
+  _gdk_error_warnings = trap->error_warnings;
   
   return result;
 }
 
-#ifndef HAVE_XCONVERTCASE
-/* compatibility function from X11R6.3, since XConvertCase is not
- * supplied by X11R5.
- */
-void
-gdk_keyval_convert_case (guint symbol,
-			 guint *lower,
-			 guint *upper)
-{
-  guint xlower = symbol;
-  guint xupper = symbol;
-
-  switch (symbol >> 8)
-    {
-#if	defined (GDK_A) && defined (GDK_Ooblique)
-    case 0: /* Latin 1 */
-      if ((symbol >= GDK_A) && (symbol <= GDK_Z))
-	xlower += (GDK_a - GDK_A);
-      else if ((symbol >= GDK_a) && (symbol <= GDK_z))
-	xupper -= (GDK_a - GDK_A);
-      else if ((symbol >= GDK_Agrave) && (symbol <= GDK_Odiaeresis))
-	xlower += (GDK_agrave - GDK_Agrave);
-      else if ((symbol >= GDK_agrave) && (symbol <= GDK_odiaeresis))
-	xupper -= (GDK_agrave - GDK_Agrave);
-      else if ((symbol >= GDK_Ooblique) && (symbol <= GDK_Thorn))
-	xlower += (GDK_oslash - GDK_Ooblique);
-      else if ((symbol >= GDK_oslash) && (symbol <= GDK_thorn))
-	xupper -= (GDK_oslash - GDK_Ooblique);
-      break;
-#endif	/* LATIN1 */
-      
-#if	defined (GDK_Aogonek) && defined (GDK_tcedilla)
-    case 1: /* Latin 2 */
-      /* Assume the KeySym is a legal value (ignore discontinuities) */
-      if (symbol == GDK_Aogonek)
-	xlower = GDK_aogonek;
-      else if (symbol >= GDK_Lstroke && symbol <= GDK_Sacute)
-	xlower += (GDK_lstroke - GDK_Lstroke);
-      else if (symbol >= GDK_Scaron && symbol <= GDK_Zacute)
-	xlower += (GDK_scaron - GDK_Scaron);
-      else if (symbol >= GDK_Zcaron && symbol <= GDK_Zabovedot)
-	xlower += (GDK_zcaron - GDK_Zcaron);
-      else if (symbol == GDK_aogonek)
-	xupper = GDK_Aogonek;
-      else if (symbol >= GDK_lstroke && symbol <= GDK_sacute)
-	xupper -= (GDK_lstroke - GDK_Lstroke);
-      else if (symbol >= GDK_scaron && symbol <= GDK_zacute)
-	xupper -= (GDK_scaron - GDK_Scaron);
-      else if (symbol >= GDK_zcaron && symbol <= GDK_zabovedot)
-	xupper -= (GDK_zcaron - GDK_Zcaron);
-      else if (symbol >= GDK_Racute && symbol <= GDK_Tcedilla)
-	xlower += (GDK_racute - GDK_Racute);
-      else if (symbol >= GDK_racute && symbol <= GDK_tcedilla)
-	xupper -= (GDK_racute - GDK_Racute);
-      break;
-#endif	/* LATIN2 */
-      
-#if	defined (GDK_Hstroke) && defined (GDK_Cabovedot)
-    case 2: /* Latin 3 */
-      /* Assume the KeySym is a legal value (ignore discontinuities) */
-      if (symbol >= GDK_Hstroke && symbol <= GDK_Hcircumflex)
-	xlower += (GDK_hstroke - GDK_Hstroke);
-      else if (symbol >= GDK_Gbreve && symbol <= GDK_Jcircumflex)
-	xlower += (GDK_gbreve - GDK_Gbreve);
-      else if (symbol >= GDK_hstroke && symbol <= GDK_hcircumflex)
-	xupper -= (GDK_hstroke - GDK_Hstroke);
-      else if (symbol >= GDK_gbreve && symbol <= GDK_jcircumflex)
-	xupper -= (GDK_gbreve - GDK_Gbreve);
-      else if (symbol >= GDK_Cabovedot && symbol <= GDK_Scircumflex)
-	xlower += (GDK_cabovedot - GDK_Cabovedot);
-      else if (symbol >= GDK_cabovedot && symbol <= GDK_scircumflex)
-	xupper -= (GDK_cabovedot - GDK_Cabovedot);
-      break;
-#endif	/* LATIN3 */
-      
-#if	defined (GDK_Rcedilla) && defined (GDK_Amacron)
-    case 3: /* Latin 4 */
-      /* Assume the KeySym is a legal value (ignore discontinuities) */
-      if (symbol >= GDK_Rcedilla && symbol <= GDK_Tslash)
-	xlower += (GDK_rcedilla - GDK_Rcedilla);
-      else if (symbol >= GDK_rcedilla && symbol <= GDK_tslash)
-	xupper -= (GDK_rcedilla - GDK_Rcedilla);
-      else if (symbol == GDK_ENG)
-	xlower = GDK_eng;
-      else if (symbol == GDK_eng)
-	xupper = GDK_ENG;
-      else if (symbol >= GDK_Amacron && symbol <= GDK_Umacron)
-	xlower += (GDK_amacron - GDK_Amacron);
-      else if (symbol >= GDK_amacron && symbol <= GDK_umacron)
-	xupper -= (GDK_amacron - GDK_Amacron);
-      break;
-#endif	/* LATIN4 */
-      
-#if	defined (GDK_Serbian_DJE) && defined (GDK_Cyrillic_yu)
-    case 6: /* Cyrillic */
-      /* Assume the KeySym is a legal value (ignore discontinuities) */
-      if (symbol >= GDK_Serbian_DJE && symbol <= GDK_Serbian_DZE)
-	xlower -= (GDK_Serbian_DJE - GDK_Serbian_dje);
-      else if (symbol >= GDK_Serbian_dje && symbol <= GDK_Serbian_dze)
-	xupper += (GDK_Serbian_DJE - GDK_Serbian_dje);
-      else if (symbol >= GDK_Cyrillic_YU && symbol <= GDK_Cyrillic_HARDSIGN)
-	xlower -= (GDK_Cyrillic_YU - GDK_Cyrillic_yu);
-      else if (symbol >= GDK_Cyrillic_yu && symbol <= GDK_Cyrillic_hardsign)
-	xupper += (GDK_Cyrillic_YU - GDK_Cyrillic_yu);
-      break;
-#endif	/* CYRILLIC */
-      
-#if	defined (GDK_Greek_ALPHAaccent) && defined (GDK_Greek_finalsmallsigma)
-    case 7: /* Greek */
-      /* Assume the KeySym is a legal value (ignore discontinuities) */
-      if (symbol >= GDK_Greek_ALPHAaccent && symbol <= GDK_Greek_OMEGAaccent)
-	xlower += (GDK_Greek_alphaaccent - GDK_Greek_ALPHAaccent);
-      else if (symbol >= GDK_Greek_alphaaccent && symbol <= GDK_Greek_omegaaccent &&
-	       symbol != GDK_Greek_iotaaccentdieresis &&
-	       symbol != GDK_Greek_upsilonaccentdieresis)
-	xupper -= (GDK_Greek_alphaaccent - GDK_Greek_ALPHAaccent);
-      else if (symbol >= GDK_Greek_ALPHA && symbol <= GDK_Greek_OMEGA)
-	xlower += (GDK_Greek_alpha - GDK_Greek_ALPHA);
-      else if (symbol >= GDK_Greek_alpha && symbol <= GDK_Greek_omega &&
-	       symbol != GDK_Greek_finalsmallsigma)
-	xupper -= (GDK_Greek_alpha - GDK_Greek_ALPHA);
-      break;
-#endif	/* GREEK */
-    }
-
-  if (lower)
-    *lower = xlower;
-  if (upper)
-    *upper = xupper;
-}
-#endif
-
-guint
-gdk_keyval_to_upper (guint keyval)
-{
-  guint result;
-  
-  gdk_keyval_convert_case (keyval, NULL, &result);
-
-  return result;
-}
-
-guint
-gdk_keyval_to_lower (guint keyval)
-{
-  guint result;
-  
-  gdk_keyval_convert_case (keyval, &result, NULL);
-
-  return result;
-}
-
-gboolean
-gdk_keyval_is_upper (guint keyval)
-{
-  if (keyval)
-    {
-      guint upper_val = 0;
-      
-      gdk_keyval_convert_case (keyval, NULL, &upper_val);
-      return upper_val == keyval;
-    }
-  return FALSE;
-}
-
-gboolean
-gdk_keyval_is_lower (guint keyval)
-{
-  if (keyval)
-    {
-      guint lower_val = 0;
-      
-      gdk_keyval_convert_case (keyval, &lower_val, NULL);
-      return lower_val == keyval;
-    }
-  return FALSE;
-}
-
 void
 gdk_threads_enter ()
 {
@@ -697,3 +514,23 @@ gdk_threads_leave ()
   GDK_THREADS_LEAVE ();
 }
 
+G_CONST_RETURN char *
+gdk_get_program_class (void)
+{
+  if (gdk_progclass == NULL)
+    {
+      gdk_progclass = g_strdup (g_get_prgname ());
+      gdk_progclass[0] = g_ascii_toupper (gdk_progclass[0]);
+    }
+  
+  return gdk_progclass;
+}
+
+void
+gdk_set_program_class (const char *program_class)
+{
+  if (gdk_progclass)
+    g_free (gdk_progclass);
+
+  gdk_progclass = g_strdup (program_class);
+}
