@@ -30,11 +30,6 @@
 typedef struct _GdkIOClosure GdkIOClosure;
 typedef struct _GdkEventPrivate GdkEventPrivate;
 
-#define DOUBLE_CLICK_TIME      250
-#define TRIPLE_CLICK_TIME      500
-#define DOUBLE_CLICK_DIST      5
-#define TRIPLE_CLICK_DIST      5
-
 typedef enum
 {
   /* Following flag is set for events on the event queue during
@@ -57,14 +52,6 @@ struct _GdkEventPrivate
   guint    flags;
 };
 
-/* 
- * Private function declarations
- */
-
-GdkFilterReturn gdk_wm_protocols_filter (GdkXEvent *xev,
-					 GdkEvent  *event,
-					 gpointer   data);
-
 /* Private variable declarations
  */
 
@@ -78,18 +65,21 @@ static GdkWindow *button_window[2] = { NULL, NULL}; /* The last 2 windows to rec
 					     */
 static guint button_number[2] = { -1, -1 }; /* The last 2 buttons to be pressed.
 					     */
-GdkEventFunc   gdk_event_func = NULL;    /* Callback for events */
-gpointer       gdk_event_data = NULL;
-GDestroyNotify gdk_event_notify = NULL;
+GdkEventFunc   _gdk_event_func = NULL;    /* Callback for events */
+gpointer       _gdk_event_data = NULL;
+GDestroyNotify _gdk_event_notify = NULL;
 
-GPollFD event_poll_fd;
+static guint double_click_time = 250;
+#define TRIPLE_CLICK_TIME      (2*double_click_time)
+#define DOUBLE_CLICK_DIST      5
+#define TRIPLE_CLICK_DIST      5
 
 /*********************************************
  * Functions for maintaining the event queue *
  *********************************************/
 
 /*************************************************************
- * gdk_event_queue_find_first:
+ * _gdk_event_queue_find_first:
  *     Find the first event on the queue that is not still
  *     being filled in.
  *   arguments:
@@ -99,9 +89,9 @@ GPollFD event_poll_fd;
  *************************************************************/
 
 GList*
-gdk_event_queue_find_first (void)
+_gdk_event_queue_find_first (void)
 {
-  GList *tmp_list = gdk_queued_events;
+  GList *tmp_list = _gdk_queued_events;
 
   while (tmp_list)
     {
@@ -116,7 +106,7 @@ gdk_event_queue_find_first (void)
 }
 
 /*************************************************************
- * gdk_event_queue_remove_link:
+ * _gdk_event_queue_remove_link:
  *     Remove a specified list node from the event queue.
  *   arguments:
  *     node: Node to remove.
@@ -124,21 +114,21 @@ gdk_event_queue_find_first (void)
  *************************************************************/
 
 void
-gdk_event_queue_remove_link (GList *node)
+_gdk_event_queue_remove_link (GList *node)
 {
   if (node->prev)
     node->prev->next = node->next;
   else
-    gdk_queued_events = node->next;
+    _gdk_queued_events = node->next;
   
   if (node->next)
     node->next->prev = node->prev;
   else
-    gdk_queued_tail = node->prev;
+    _gdk_queued_tail = node->prev;
 }
 
 /*************************************************************
- * gdk_event_queue_append:
+ * _gdk_event_queue_append:
  *     Append an event onto the tail of the event queue.
  *   arguments:
  *     event: Event to append.
@@ -146,14 +136,14 @@ gdk_event_queue_remove_link (GList *node)
  *************************************************************/
 
 void
-gdk_event_queue_append (GdkEvent *event)
+_gdk_event_queue_append (GdkEvent *event)
 {
-  gdk_queued_tail = g_list_append (gdk_queued_tail, event);
+  _gdk_queued_tail = g_list_append (_gdk_queued_tail, event);
   
-  if (!gdk_queued_events)
-    gdk_queued_events = gdk_queued_tail;
+  if (!_gdk_queued_events)
+    _gdk_queued_events = _gdk_queued_tail;
   else
-    gdk_queued_tail = gdk_queued_tail->next;
+    _gdk_queued_tail = _gdk_queued_tail->next;
 }
 
 /*************************************************************
@@ -172,12 +162,12 @@ gdk_event_handler_set (GdkEventFunc   func,
 		       gpointer       data,
 		       GDestroyNotify notify)
 {
-  if (gdk_event_notify)
-    (*gdk_event_notify) (gdk_event_data);
+  if (_gdk_event_notify)
+    (*_gdk_event_notify) (_gdk_event_data);
 
-  gdk_event_func = func;
-  gdk_event_data = data;
-  gdk_event_notify = notify;
+  _gdk_event_func = func;
+  _gdk_event_data = data;
+  _gdk_event_notify = notify;
 }
 
 /*
@@ -201,9 +191,9 @@ gdk_event_handler_set (GdkEventFunc   func,
 GdkEvent*
 gdk_event_get (void)
 {
-  gdk_events_queue ();
+  _gdk_events_queue ();
 
-  return gdk_event_unqueue ();
+  return _gdk_event_unqueue ();
 }
 
 /*
@@ -230,7 +220,7 @@ gdk_event_peek (void)
 {
   GList *tmp_list;
 
-  tmp_list = gdk_event_queue_find_first ();
+  tmp_list = _gdk_event_queue_find_first ();
   
   if (tmp_list)
     return gdk_event_copy (tmp_list->data);
@@ -247,7 +237,7 @@ gdk_event_put (GdkEvent *event)
   
   new_event = gdk_event_copy (event);
 
-  gdk_event_queue_append (new_event);
+  _gdk_event_queue_append (new_event);
 }
 
 /*
@@ -271,7 +261,7 @@ gdk_event_put (GdkEvent *event)
 static GMemChunk *event_chunk = NULL;
 
 GdkEvent*
-gdk_event_new (void)
+_gdk_event_new (void)
 {
   GdkEventPrivate *new_event;
   
@@ -294,7 +284,7 @@ gdk_event_copy (GdkEvent *event)
   
   g_return_val_if_fail (event != NULL, NULL);
   
-  new_event = gdk_event_new ();
+  new_event = _gdk_event_new ();
   
   *new_event = *event;
   if (new_event->any.window)
@@ -322,6 +312,15 @@ gdk_event_copy (GdkEvent *event)
       gdk_drag_context_ref (event->dnd.context);
       break;
       
+    case GDK_EXPOSE:
+      if (event->expose.region)
+	new_event->expose.region = gdk_region_copy (event->expose.region);
+      break;
+      
+    case GDK_SETTING:
+      new_event->setting.name = g_strdup (new_event->setting.name);
+      break;
+      
     default:
       break;
     }
@@ -384,12 +383,21 @@ gdk_event_free (GdkEvent *event)
       if (event->button.axes)
 	g_free (event->button.axes);
       break;
-
+      
+    case GDK_EXPOSE:
+      if (event->expose.region)
+	gdk_region_destroy (event->expose.region);
+      break;
+      
     case GDK_MOTION_NOTIFY:
       if (event->motion.axes)
 	g_free (event->motion.axes);
       break;
       
+    case GDK_SETTING:
+      g_free (event->setting.name);
+      break;
+      
     default:
       break;
     }
@@ -397,18 +405,15 @@ gdk_event_free (GdkEvent *event)
   g_mem_chunk_free (event_chunk, event);
 }
 
-/*
- *--------------------------------------------------------------
+/**
  * gdk_event_get_time:
- *    Get the timestamp from an event.
- *   arguments:
- *     event:
- *   results:
- *    The event's time stamp, if it has one, otherwise
- *    GDK_CURRENT_TIME.
- *--------------------------------------------------------------
- */
-
+ * @event: a #GdkEvent
+ * 
+ * Returns the time stamp from @event, if there is one; otherwise
+ * returns #GDK_CURRENT_TIME. If @event is %NULL, returns #GDK_CURRENT_TIME.
+ * 
+ * Return value: time stamp field from @event
+ **/
 guint32
 gdk_event_get_time (GdkEvent *event)
 {
@@ -421,8 +426,9 @@ gdk_event_get_time (GdkEvent *event)
       case GDK_2BUTTON_PRESS:
       case GDK_3BUTTON_PRESS:
       case GDK_BUTTON_RELEASE:
-      case GDK_SCROLL:
 	return event->button.time;
+      case GDK_SCROLL:
+        return event->scroll.time;
       case GDK_KEY_PRESS:
       case GDK_KEY_RELEASE:
 	return event->key.time;
@@ -445,13 +451,223 @@ gdk_event_get_time (GdkEvent *event)
       case GDK_DROP_START:
       case GDK_DROP_FINISHED:
 	return event->dnd.time;
-      default:			/* use current time */
-	break;
+      case GDK_CLIENT_EVENT:
+      case GDK_VISIBILITY_NOTIFY:
+      case GDK_NO_EXPOSE:
+      case GDK_CONFIGURE:
+      case GDK_FOCUS_CHANGE:
+      case GDK_NOTHING:
+      case GDK_DELETE:
+      case GDK_DESTROY:
+      case GDK_EXPOSE:
+      case GDK_MAP:
+      case GDK_UNMAP:
+      case GDK_WINDOW_STATE:
+      case GDK_SETTING:
+        /* return current time */
+        break;
       }
   
   return GDK_CURRENT_TIME;
 }
 
+/**
+ * gdk_event_get_state:
+ * @event: a #GdkEvent or NULL
+ * @state: return location for state
+ * 
+ * If the event contains a "state" field, puts that field in @state. Otherwise
+ * stores an empty state (0). Returns %TRUE if there was a state field
+ * in the event. @event may be %NULL, in which case it's treated
+ * as if the event had no state field.
+ * 
+ * Return value: %TRUE if there was a state field in the event 
+ **/
+gboolean
+gdk_event_get_state (GdkEvent        *event,
+                     GdkModifierType *state)
+{
+  g_return_val_if_fail (state != NULL, FALSE);
+  
+  if (event)
+    switch (event->type)
+      {
+      case GDK_MOTION_NOTIFY:
+	*state = event->motion.state;
+        return TRUE;
+      case GDK_BUTTON_PRESS:
+      case GDK_2BUTTON_PRESS:
+      case GDK_3BUTTON_PRESS:
+      case GDK_BUTTON_RELEASE:
+        *state =  event->button.state;
+        return TRUE;
+      case GDK_SCROLL:
+	*state =  event->scroll.state;
+        return TRUE;
+      case GDK_KEY_PRESS:
+      case GDK_KEY_RELEASE:
+	*state =  event->key.state;
+        return TRUE;
+      case GDK_ENTER_NOTIFY:
+      case GDK_LEAVE_NOTIFY:
+	*state =  event->crossing.state;
+        return TRUE;
+      case GDK_PROPERTY_NOTIFY:
+	*state =  event->property.state;
+        return TRUE;
+      case GDK_VISIBILITY_NOTIFY:
+        *state =  event->visibility.state;
+        return TRUE;
+      case GDK_CLIENT_EVENT:
+      case GDK_NO_EXPOSE:
+      case GDK_CONFIGURE:
+      case GDK_FOCUS_CHANGE:
+      case GDK_SELECTION_CLEAR:
+      case GDK_SELECTION_REQUEST:
+      case GDK_SELECTION_NOTIFY:
+      case GDK_PROXIMITY_IN:
+      case GDK_PROXIMITY_OUT:
+      case GDK_DRAG_ENTER:
+      case GDK_DRAG_LEAVE:
+      case GDK_DRAG_MOTION:
+      case GDK_DRAG_STATUS:
+      case GDK_DROP_START:
+      case GDK_DROP_FINISHED:
+      case GDK_NOTHING:
+      case GDK_DELETE:
+      case GDK_DESTROY:
+      case GDK_EXPOSE:
+      case GDK_MAP:
+      case GDK_UNMAP:
+      case GDK_WINDOW_STATE:
+      case GDK_SETTING:
+        /* no state field */
+        break;
+      }
+
+  *state = 0;
+  return FALSE;
+}
+
+/**
+ * gdk_event_get_coords:
+ * @event: a #GdkEvent
+ * @x_root: location to put event window x coordinate
+ * @y_root: location to put event window y coordinate
+ * 
+ * Extract the event window relative x/y coordinates from an event.
+ * 
+ * Return value: %TRUE if the event delivered event window coordinates
+ **/
+gboolean
+gdk_event_get_coords (GdkEvent *event,
+		      gdouble  *x_win,
+		      gdouble  *y_win)
+{
+  gdouble x = 0, y = 0;
+  gboolean fetched = TRUE;
+  
+  g_return_val_if_fail (event != NULL, FALSE);
+
+  switch (event->type)
+    {
+    case GDK_CONFIGURE:
+      x = event->configure.x;
+      y = event->configure.y;
+      break;
+    case GDK_ENTER_NOTIFY:
+    case GDK_LEAVE_NOTIFY:
+      x = event->crossing.x;
+      y = event->crossing.y;
+      break;
+    case GDK_SCROLL:
+      x = event->scroll.x;
+      y = event->scroll.y;
+      break;
+    case GDK_BUTTON_PRESS:
+    case GDK_2BUTTON_PRESS:
+    case GDK_3BUTTON_PRESS:
+    case GDK_BUTTON_RELEASE:
+      x = event->button.x;
+      y = event->button.y;
+      break;
+    case GDK_MOTION_NOTIFY:
+      x = event->motion.x;
+      y = event->motion.y;
+      break;
+    default:
+      fetched = FALSE;
+      break;
+    }
+
+  if (x_win)
+    *x_win = x;
+  if (y_win)
+    *y_win = x;
+
+  return fetched;
+}
+
+/**
+ * gdk_event_get_root_coords:
+ * @event: a #GdkEvent
+ * @x_root: location to put root window x coordinate
+ * @y_root: location to put root window y coordinate
+ * 
+ * Extract the root window relative x/y coordinates from an event.
+ * 
+ * Return value: %TRUE if the event delivered root window coordinates
+ **/
+gboolean
+gdk_event_get_root_coords (GdkEvent *event,
+			   gdouble  *x_root,
+			   gdouble  *y_root)
+{
+  gdouble x = 0, y = 0;
+  gboolean fetched = TRUE;
+  
+  g_return_val_if_fail (event != NULL, FALSE);
+
+  switch (event->type)
+    {
+    case GDK_MOTION_NOTIFY:
+      x = event->motion.x_root;
+      y = event->motion.y_root;
+      break;
+    case GDK_BUTTON_PRESS:
+    case GDK_2BUTTON_PRESS:
+    case GDK_3BUTTON_PRESS:
+    case GDK_BUTTON_RELEASE:
+      x = event->button.x_root;
+      y = event->button.y_root;
+      break;
+    case GDK_ENTER_NOTIFY:
+    case GDK_LEAVE_NOTIFY:
+      x = event->crossing.x_root;
+      y = event->crossing.y_root;
+      break;
+    case GDK_DRAG_ENTER:
+    case GDK_DRAG_LEAVE:
+    case GDK_DRAG_MOTION:
+    case GDK_DRAG_STATUS:
+    case GDK_DROP_START:
+    case GDK_DROP_FINISHED:
+      x = event->dnd.x_root;
+      y = event->dnd.y_root;
+      break;
+    default:
+      fetched = FALSE;
+      break;
+    }
+
+  if (x_root)
+    *x_root = x;
+  if (y_root)
+    *y_root = x;
+
+  return fetched;
+}
+
 /**
  * gdk_event_get_axis:
  * @event: a #GdkEvent
@@ -550,15 +766,15 @@ void
 gdk_set_show_events (gboolean show_events)
 {
   if (show_events)
-    gdk_debug_flags |= GDK_DEBUG_EVENTS;
+    _gdk_debug_flags |= GDK_DEBUG_EVENTS;
   else
-    gdk_debug_flags &= ~GDK_DEBUG_EVENTS;
+    _gdk_debug_flags &= ~GDK_DEBUG_EVENTS;
 }
 
 gboolean
 gdk_get_show_events (void)
 {
-  return (gdk_debug_flags & GDK_DEBUG_EVENTS) != 0;
+  return (_gdk_debug_flags & GDK_DEBUG_EVENTS) != 0;
 }
 
 static void
@@ -648,17 +864,17 @@ gdk_input_remove (gint tag)
 }
 
 GdkEvent*
-gdk_event_unqueue (void)
+_gdk_event_unqueue (void)
 {
   GdkEvent *event = NULL;
   GList *tmp_list;
 
-  tmp_list = gdk_event_queue_find_first ();
+  tmp_list = _gdk_event_queue_find_first ();
 
   if (tmp_list)
     {
       event = tmp_list->data;
-      gdk_event_queue_remove_link (tmp_list);
+      _gdk_event_queue_remove_link (tmp_list);
       g_list_free_1 (tmp_list);
     }
 
@@ -680,7 +896,7 @@ gdk_synthesize_click (GdkEvent *event,
 }
 
 void
-gdk_event_button_generate (GdkEvent *event)
+_gdk_event_button_generate (GdkEvent *event)
 {
   if ((event->button.time < (button_click_time[1] + TRIPLE_CLICK_TIME)) &&
       (event->button.window == button_window[1]) &&
@@ -695,7 +911,7 @@ gdk_event_button_generate (GdkEvent *event)
       button_number[1] = -1;
       button_number[0] = -1;
     }
-  else if ((event->button.time < (button_click_time[0] + DOUBLE_CLICK_TIME)) &&
+  else if ((event->button.time < (button_click_time[0] + double_click_time)) &&
 	   (event->button.window == button_window[0]) &&
 	   (event->button.button == button_number[0]))
     {
@@ -718,3 +934,79 @@ gdk_event_button_generate (GdkEvent *event)
       button_number[0] = event->button.button;
     }
 }
+
+
+void
+gdk_synthesize_window_state (GdkWindow     *window,
+                             GdkWindowState unset_flags,
+                             GdkWindowState set_flags)
+{
+  GdkEventWindowState temp_event;
+  GdkWindowState old;
+  
+  g_return_if_fail (window != NULL);
+  
+  temp_event.window = window;
+  temp_event.type = GDK_WINDOW_STATE;
+  temp_event.send_event = FALSE;
+  
+  old = ((GdkWindowObject*) temp_event.window)->state;
+  
+  temp_event.changed_mask = (unset_flags | set_flags) ^ old;
+  temp_event.new_window_state = old;
+  temp_event.new_window_state |= set_flags;
+  temp_event.new_window_state &= ~unset_flags;
+
+  if (temp_event.new_window_state == old)
+    return; /* No actual work to do, nothing changed. */
+
+  /* Actually update the field in GdkWindow, this is sort of an odd
+   * place to do it, but seems like the safest since it ensures we expose no
+   * inconsistent state to the user.
+   */
+  
+  ((GdkWindowObject*) window)->state = temp_event.new_window_state;
+
+  /* We only really send the event to toplevels, since
+   * all the window states don't apply to non-toplevels.
+   * Non-toplevels do use the GDK_WINDOW_STATE_WITHDRAWN flag
+   * internally so we needed to update window->state.
+   */
+  switch (((GdkWindowObject*) window)->window_type)
+    {
+    case GDK_WINDOW_TOPLEVEL:
+    case GDK_WINDOW_DIALOG:
+    case GDK_WINDOW_TEMP: /* ? */
+      gdk_event_put ((GdkEvent*) &temp_event);
+      break;
+      
+    case GDK_WINDOW_FOREIGN:
+    case GDK_WINDOW_ROOT:
+    case GDK_WINDOW_CHILD:
+      break;
+    }
+}
+
+void
+gdk_set_double_click_time (guint msec)
+{
+  double_click_time = msec;
+}
+
+GType
+gdk_event_get_type (void)
+{
+  static GType our_type = 0;
+  
+  if (our_type == 0)
+    our_type = g_boxed_type_register_static ("GdkEvent",
+					     (GBoxedCopyFunc)gdk_event_copy,
+					     (GBoxedFreeFunc)gdk_event_free);
+  return our_type;
+}
+
+GdkDevice *
+gdk_device_get_core_pointer (void)
+{
+  return _gdk_core_pointer;
+}
