@@ -16,7 +16,7 @@ typedef struct _GdkPixmapObjectClass GdkPixmapObjectClass;
 #define GDK_PIXMAP_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), GDK_TYPE_PIXMAP, GdkPixmapObjectClass))
 #define GDK_IS_PIXMAP(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), GDK_TYPE_PIXMAP))
 #define GDK_IS_PIXMAP_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), GDK_TYPE_PIXMAP))
-#define GDK_PIXMAP_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), GDK_TYPE_PIXMAP, GdkPixmapClass))
+#define GDK_PIXMAP_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), GDK_TYPE_PIXMAP, GdkPixmapObjectClass))
 #define GDK_PIXMAP_OBJECT(object)    ((GdkPixmapObject *) GDK_PIXMAP (object))
 
 struct _GdkPixmapObject
@@ -53,26 +53,32 @@ GdkPixmap* gdk_pixmap_create_from_data	(GdkWindow   *window,
 					 gint	      depth,
 					 GdkColor    *fg,
 					 GdkColor    *bg);
-GdkPixmap* gdk_pixmap_create_from_xpm	(GdkWindow  *window,
-					 GdkBitmap **mask,
-					 GdkColor   *transparent_color,
-					 const gchar *filename);
-GdkPixmap* gdk_pixmap_colormap_create_from_xpm 
-                                        (GdkWindow   *window,
-					 GdkColormap *colormap,
-					 GdkBitmap  **mask,
-					 GdkColor    *transparent_color,
-					 const gchar *filename);
-GdkPixmap* gdk_pixmap_create_from_xpm_d (GdkWindow  *window,
-					 GdkBitmap **mask,
-					 GdkColor   *transparent_color,
-					 gchar	   **data);
-GdkPixmap* gdk_pixmap_colormap_create_from_xpm_d 
-                                        (GdkWindow   *window,
-					 GdkColormap *colormap,
-					 GdkBitmap  **mask,
-					 GdkColor    *transparent_color,
-					 gchar     **data);
+
+GdkPixmap* gdk_pixmap_create_from_xpm            (GdkWindow    *window,
+						  GdkBitmap   **mask,
+						  GdkColor     *transparent_color,
+						  const gchar  *filename);
+GdkPixmap* gdk_pixmap_colormap_create_from_xpm   (GdkWindow    *window,
+						  GdkColormap  *colormap,
+						  GdkBitmap   **mask,
+						  GdkColor     *transparent_color,
+						  const gchar  *filename);
+GdkPixmap* gdk_pixmap_create_from_xpm_d          (GdkWindow    *window,
+						  GdkBitmap   **mask,
+						  GdkColor     *transparent_color,
+						  gchar       **data);
+GdkPixmap* gdk_pixmap_colormap_create_from_xpm_d (GdkWindow    *window,
+						  GdkColormap  *colormap,
+						  GdkBitmap   **mask,
+						  GdkColor     *transparent_color,
+						  gchar       **data);
+
+#ifndef GDK_DISABLE_DEPRECATED
+#define gdk_bitmap_ref                 gdk_drawable_ref
+#define gdk_bitmap_unref               gdk_drawable_unref
+#define gdk_pixmap_ref                 gdk_drawable_ref
+#define gdk_pixmap_unref               gdk_drawable_unref
+#endif /* GDK_DISABLE_DEPRECATED */
 
 #ifdef __cplusplus
 }
