@@ -1517,6 +1517,11 @@ GdkFont*
 gdk_font_load (const gchar *font_name)
 {
   GdkFont *font;
+  GdkFontPrivateWin32 *private;
+  GdkWin32SingleFont *singlefont;
+  HGDIOBJ oldfont;
+  HANDLE *f;
+  TEXTMETRIC textmetric;
 
   g_return_val_if_fail (font_name != NULL, NULL);
 
@@ -1524,6 +1529,29 @@ gdk_font_load (const gchar *font_name)
   if (font)
     return font;
 
+  private = g_new (GdkFontPrivateWin32, 1);
+  font = (GdkFont*) private;
+
+  singlefont = gdk_font_load_internal (font_name);
+
+  private->base.ref_count = 1;
+  private->names = NULL;
+  private->fonts = g_slist_append (NULL, singlefont);
+
+  /* Pretend all fonts are fontsets... Gtktext and gtkentry work better
+   * that way, they use wide chars, which is necessary for non-ASCII
+   * chars to work. (Yes, even Latin-1, as we use Unicode internally.)
+   */
+  font->type = GDK_FONT_FONTSET;
+  oldfont = SelectObject (gdk_display_hdc, singlefont->hfont);
+  GetTextMetrics (gdk_display_hdc, &textmetric);
+  SelectObject (gdk_display_hdc, oldfont);
+  font->ascent = textmetric.tmAscent;
+  font->descent = textmetric.tmDescent;
+
+  GDK_NOTE (MISC, g_print ("... asc %d desc %d\n",
+			   font->ascent, font->descent));
+
   gdk_font_hash_insert (GDK_FONT_FONTSET, font, font_name);
 
   return gdk_font_from_one_singlefont (gdk_font_load_internal (font_name));
@@ -1556,20 +1584,10 @@ gdk_font_from_description (PangoFontDescription *font_desc)
 
   if (font)
     {
-      gint n_subfonts;
-      PangoWin32Subfont *subfont_ids;
-
-      n_subfonts = pango_win32_list_subfonts (font, PANGO_WIN32_U_BASIC_LATIN,
-					      &subfont_ids);
-      if (n_subfonts > 0)
-	{
-	  LOGFONT *lfp =
-	    pango_win32_font_subfont_logfont (font, subfont_ids[0]);
-	  result = gdk_font_from_one_singlefont (gdk_font_load_logfont (lfp));
-	  g_free (lfp);
-	}
-
-      g_free (subfont_ids);
+      LOGFONT *lfp =
+	pango_win32_font_logfont (font);
+      result = gdk_font_from_one_singlefont (gdk_font_load_logfont (lfp));
+      g_free (lfp);
 
       g_object_unref (G_OBJECT (font));
     }
