@@ -170,7 +170,7 @@ gdk_pixmap_new (GdkWindow *window,
 #endif
 
   if (!window)
-    window = gdk_parent_root;
+    window = _gdk_parent_root;
 
   if (GDK_WINDOW_DESTROYED (window))
     return NULL;
@@ -231,12 +231,14 @@ gdk_pixmap_new (GdkWindow *window,
 	bmi.u.bmiColors[1].rgbRed = 0xFF;
       bmi.u.bmiColors[1].rgbReserved = 0x00;
       draw_impl->colormap = NULL;
+      GDK_NOTE (MISC, g_print ("... colormap NULL\n"));
     }
   else
     {
       draw_impl->colormap = GDK_DRAWABLE_IMPL_WIN32 (GDK_WINDOW_OBJECT (window)->impl)->colormap;
       if (draw_impl->colormap == NULL)
 	draw_impl->colormap = gdk_colormap_get_system ();
+      GDK_NOTE (MISC, g_print ("... colormap %p\n", draw_impl->colormap));
 
       if (depth == 8)
 	{
@@ -328,7 +330,7 @@ gdk_bitmap_create_from_data (GdkWindow   *window,
   g_return_val_if_fail (window == NULL || GDK_IS_WINDOW (window), NULL);
 
   if (!window)
-    window = gdk_parent_root;
+    window = _gdk_parent_root;
 
   if (GDK_WINDOW_DESTROYED (window))
     return NULL;
