@@ -32,6 +32,19 @@
 #include "gdkfont.h"
 #include "gdkprivate-x11.h"
 
+typedef struct _GdkFontPrivateX        GdkFontPrivateX;
+
+struct _GdkFontPrivateX
+{
+  GdkFontPrivate base;
+  /* XFontStruct *xfont; */
+  /* generic pointer point to XFontStruct or XFontSet */
+  gpointer xfont;
+  Display *xdisplay;
+
+  GSList *names;
+};
+
 static GHashTable *font_name_hash = NULL;
 static GHashTable *fontset_name_hash = NULL;
 
@@ -137,7 +150,7 @@ gdk_font_load (const gchar *font_name)
 }
 
 static char *
-gdk_font_charset_for_locale ()
+gdk_font_charset_for_locale (void)
 {
   static char *charset_map[][2] = {
     { "ANSI_X3.4-1968", "iso8859-1" },
@@ -154,7 +167,7 @@ gdk_font_charset_for_locale ()
     { "UTF-8",      "iso8859-1" }
   };
 
-  char *codeset;
+  const char *codeset;
   char *result = NULL;
   int i;
 
@@ -648,3 +661,44 @@ gdk_text_extents_wc (GdkFont        *font,
     }
 
 }
+
+Display *
+gdk_x11_font_get_xdisplay (GdkFont *font)
+{
+  g_return_val_if_fail (font != NULL, NULL);
+
+  return ((GdkFontPrivateX *)font)->xdisplay;
+}
+
+gpointer
+gdk_x11_font_get_xfont (GdkFont *font)
+{
+  g_return_val_if_fail (font != NULL, NULL);
+
+  return ((GdkFontPrivateX *)font)->xfont;
+}
+
+/**
+ * gdk_x11_font_get_name:
+ * @font: a #GdkFont.
+ * 
+ * Return the X Logical Font Description (for font->type == GDK_FONT_FONT)
+ * or comma separated list of XLFDs (for font->type == GDK_FONT_FONTSET)
+ * that was used to load the font. If the same font was loaded
+ * via multiple names, which name is returned is undefined.
+ * 
+ * Return value: the name of the font. This string is owned
+ *   by GDK and must not be modified or freed.
+ **/
+G_CONST_RETURN char *
+gdk_x11_font_get_name (GdkFont *font)
+{
+  GdkFontPrivateX *private = (GdkFontPrivateX *)font;
+
+  g_return_val_if_fail (font != NULL, NULL);
+
+  g_assert (private->names);
+
+  return private->names->data;
+}
+     
