@@ -62,7 +62,7 @@ static void gdk_gc_x11_finalize   (GObject           *object);
 static gpointer parent_class = NULL;
 
 GType
-gdk_gc_x11_get_type (void)
+_gdk_gc_x11_get_type (void)
 {
   static GType object_type = 0;
 
@@ -139,7 +139,7 @@ _gdk_x11_gc_new (GdkDrawable      *drawable,
    */
   g_return_val_if_fail (GDK_IS_DRAWABLE_IMPL_X11 (drawable), NULL);
 
-  gc = g_object_new (gdk_gc_x11_get_type (), NULL);
+  gc = g_object_new (_gdk_gc_x11_get_type (), NULL);
   private = GDK_GC_X11 (gc);
 
   private->dirty_mask = 0;
@@ -734,3 +734,26 @@ gdk_gc_copy (GdkGC *dst_gc, GdkGC *src_gc)
   x11_dst_gc->dirty_mask = x11_src_gc->dirty_mask;
   x11_dst_gc->fg_pixel = x11_src_gc->fg_pixel;
 }
+
+Display *
+gdk_x11_gc_get_xdisplay (GdkGC *gc)
+{
+  g_return_val_if_fail (GDK_IS_GC_X11 (gc), NULL);
+
+  return GDK_GC_X11(gc)->xdisplay;
+}
+
+GC
+gdk_x11_gc_get_xgc (GdkGC *gc)
+{
+  GdkGCX11 *gc_x11;
+  
+  g_return_val_if_fail (GDK_IS_GC_X11 (gc), NULL);
+
+  gc_x11 = GDK_GC_X11 (gc);
+
+  if (gc_x11->dirty_mask)
+    _gdk_x11_gc_flush (gc);
+
+  return gc_x11->xgc;
+}
