@@ -28,6 +28,7 @@
 #include "gdkprivate-x11.h"
 #include "gdkx.h"
 #include "gdkregion.h"
+#include "gdkinternals.h"
 
 typedef struct _GdkWindowQueueItem GdkWindowQueueItem;
 typedef struct _GdkWindowParentPos GdkWindowParentPos;
@@ -142,7 +143,11 @@ gdk_window_scroll (GdkWindow *window,
   
   if (GDK_WINDOW_DESTROYED (window))
     return;
-
+  
+  /* Move the current invalid region */
+  if (obj->update_area)
+    gdk_region_offset (obj->update_area, dx, dy);
+  
   /* We can guffaw scroll if we are a child window, and the parent
    * does not extend beyond our edges.
    */
@@ -328,7 +333,7 @@ _gdk_window_move_resize_child (GdkWindow *window,
       if (impl->position_info.no_bg)
 	gdk_window_tmp_reset_bg (window);
 
-      if (!impl->position_info.mapped && new_info.mapped && obj->mapped)
+      if (!impl->position_info.mapped && new_info.mapped && GDK_WINDOW_IS_MAPPED (obj))
 	XMapWindow (GDK_WINDOW_XDISPLAY (window), GDK_WINDOW_XID (window));
       
       impl->position_info = new_info;
@@ -374,7 +379,7 @@ _gdk_window_move_resize_child (GdkWindow *window,
       if (impl->position_info.no_bg)
 	gdk_window_tmp_reset_bg (window);
 
-      if (!impl->position_info.mapped && new_info.mapped && obj->mapped)
+      if (!impl->position_info.mapped && new_info.mapped && GDK_WINDOW_IS_MAPPED (obj))
 	XMapWindow (GDK_WINDOW_XDISPLAY (window), GDK_WINDOW_XID (window));
 
       impl->position_info = new_info;
@@ -382,7 +387,7 @@ _gdk_window_move_resize_child (GdkWindow *window,
 }
 
 static void
-gdk_window_compute_position (GdkWindowImplX11      *window,
+gdk_window_compute_position (GdkWindowImplX11   *window,
 			     GdkWindowParentPos *parent_pos,
 			     GdkXPositionInfo   *info)
 {
@@ -647,7 +652,7 @@ gdk_window_postmove (GdkWindow          *window,
 			 new_info.x, new_info.y, new_info.width, new_info.height);
     }
 
-  if (!impl->position_info.mapped && new_info.mapped && obj->mapped)
+  if (!impl->position_info.mapped && new_info.mapped && GDK_WINDOW_IS_MAPPED (obj))
     XMapWindow (GDK_DRAWABLE_XDISPLAY (window), GDK_DRAWABLE_XID (window));
 
   if (impl->position_info.no_bg)
@@ -704,7 +709,7 @@ _gdk_window_process_expose (GdkWindow    *window,
   GdkRegion *invalidate_region = gdk_region_rectangle (area);
   GdkRegion *clip_region;
   GSList *tmp_list = translate_queue;
-
+  
   impl = GDK_WINDOW_IMPL_X11 (GDK_WINDOW_OBJECT (window)->impl);
   
   while (tmp_list)
@@ -717,7 +722,7 @@ _gdk_window_process_expose (GdkWindow    *window,
 	  if (item->window == window)
 	    {
 	      if (item->type == GDK_WINDOW_QUEUE_TRANSLATE)
-		gdk_region_offset (invalidate_region, - item->u.translate.dx, - item->u.translate.dy);
+		gdk_region_offset (invalidate_region, item->u.translate.dx, item->u.translate.dy);
 	      else		/* anti-expose */
 		gdk_region_subtract (invalidate_region, item->u.antiexpose.area);
 	    }
@@ -742,7 +747,7 @@ _gdk_window_process_expose (GdkWindow    *window,
 
   if (!gdk_region_empty (invalidate_region))
     gdk_window_invalidate_region (window, invalidate_region, FALSE);
-
+  
   gdk_region_destroy (invalidate_region);
   gdk_region_destroy (clip_region);
 }
