@@ -32,7 +32,8 @@
 #include <unistd.h>
 #include <X11/Xlib.h>
 
-#include <gdk/gdkpixmap.h>
+#include "gdkx.h"
+
 #include "gdkpixmap-x11.h"
 #include "gdkprivate-x11.h"
 
@@ -60,7 +61,7 @@ static void gdk_pixmap_impl_x11_finalize   (GObject            *object);
 
 static gpointer parent_class = NULL;
 
-GType
+static GType
 gdk_pixmap_impl_x11_get_type (void)
 {
   static GType object_type = 0;
@@ -150,19 +151,21 @@ gdk_pixmap_new (GdkWindow *window,
   GdkDrawableImplX11 *draw_impl;
   GdkPixmapImplX11 *pix_impl;
   GdkColormap *cmap;
+  gint window_depth;
   
   g_return_val_if_fail (window == NULL || GDK_IS_WINDOW (window), NULL);
   g_return_val_if_fail ((window != NULL) || (depth != -1), NULL);
   g_return_val_if_fail ((width != 0) && (height != 0), NULL);
   
   if (!window)
-    window = gdk_parent_root;
+    window = _gdk_parent_root;
 
   if (GDK_WINDOW_DESTROYED (window))
     return NULL;
 
+  window_depth = gdk_drawable_get_depth (GDK_DRAWABLE (window));
   if (depth == -1)
-    depth = gdk_drawable_get_depth (GDK_DRAWABLE (window));
+    depth = window_depth;
 
   pixmap = g_object_new (gdk_pixmap_get_type (), NULL);
   draw_impl = GDK_DRAWABLE_IMPL_X11 (GDK_PIXMAP_OBJECT (pixmap)->impl);
@@ -179,14 +182,14 @@ gdk_pixmap_new (GdkWindow *window,
   pix_impl->height = height;
   GDK_PIXMAP_OBJECT (pixmap)->depth = depth;
 
-  if (window)
+  if (depth == window_depth)
     {
       cmap = gdk_drawable_get_colormap (window);
       if (cmap)
         gdk_drawable_set_colormap (pixmap, cmap);
     }
   
-  gdk_xid_table_insert (&GDK_PIXMAP_XID (pixmap), pixmap);
+  gdk_xid_table_insert (&draw_impl->xid, pixmap);
   
   return pixmap;
 }
@@ -206,7 +209,7 @@ gdk_bitmap_create_from_data (GdkWindow   *window,
   g_return_val_if_fail (window == NULL || GDK_IS_WINDOW (window), NULL);
 
   if (!window)
-    window = gdk_parent_root;
+    window = _gdk_parent_root;
 
   if (GDK_WINDOW_DESTROYED (window))
     return NULL;
@@ -226,7 +229,7 @@ gdk_bitmap_create_from_data (GdkWindow   *window,
                                           GDK_WINDOW_XID (window),
                                           (char *)data, width, height);
 
-  gdk_xid_table_insert (&GDK_PIXMAP_XID (pixmap), pixmap);
+  gdk_xid_table_insert (&draw_impl->xid, pixmap);
   
   return pixmap;
 }
@@ -252,7 +255,7 @@ gdk_pixmap_create_from_data (GdkWindow   *window,
   g_return_val_if_fail ((width != 0) && (height != 0), NULL);
 
   if (!window)
-    window = gdk_parent_root;
+    window = _gdk_parent_root;
 
   if (GDK_WINDOW_DESTROYED (window))
     return NULL;
@@ -276,7 +279,7 @@ gdk_pixmap_create_from_data (GdkWindow   *window,
                                                 (char *)data, width, height,
                                                 fg->pixel, bg->pixel, depth);
 
-  gdk_xid_table_insert (&GDK_PIXMAP_XID (pixmap), pixmap);
+  gdk_xid_table_insert (&draw_impl->xid, pixmap);
 
   return pixmap;
 }
@@ -319,7 +322,7 @@ gdk_pixmap_foreign_new (GdkNativeWindow anid)
   pix_impl->height = h_ret;
   GDK_PIXMAP_OBJECT (pixmap)->depth = depth_ret;
   
-  gdk_xid_table_insert(&GDK_PIXMAP_XID (pixmap), pixmap);
+  gdk_xid_table_insert(&draw_impl->xid, pixmap);
 
   return pixmap;
 }
