@@ -21,7 +21,7 @@
  */
 
 /*
- * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
+ * Modified by the GTK+ Team and others 1997-2001.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
@@ -32,28 +32,30 @@
 #include "gtkmain.h"
 #include "gtksignal.h"
 #include "gtkaccellabel.h"
-
+#include "gtkintl.h"
 
 enum {
-  ARG_0,
-  ARG_ACCEL_WIDGET
+  PROP_0,
+  PROP_ACCEL_OBJECT
 };
 
-static void gtk_accel_label_class_init	     (GtkAccelLabelClass *klass);
-static void gtk_accel_label_init	     (GtkAccelLabel  *accel_label);
-static void gtk_accel_label_set_arg	     (GtkObject	     *object,
-					      GtkArg	     *arg,
-					      guint	      arg_id);
-static void gtk_accel_label_get_arg	     (GtkObject      *object,
-					      GtkArg	     *arg,
-					      guint	      arg_id);
-static void gtk_accel_label_destroy	     (GtkObject	     *object);
-static void gtk_accel_label_finalize	     (GObject	     *object);
-static void gtk_accel_label_size_request     (GtkWidget	     *widget,
-					      GtkRequisition *requisition);
-static gboolean gtk_accel_label_expose_event (GtkWidget	     *widget,
-					      GdkEventExpose *event);
-static gboolean gtk_accel_label_refetch_idle (GtkAccelLabel  *accel_label);
+static void     gtk_accel_label_class_init   (GtkAccelLabelClass *klass);
+static void     gtk_accel_label_init         (GtkAccelLabel      *accel_label);
+static void     gtk_accel_label_set_property (GObject            *object,
+					      guint               prop_id,
+					      const GValue       *value,
+					      GParamSpec         *pspec);
+static void     gtk_accel_label_get_property (GObject            *object,
+					      guint               prop_id,
+					      GValue             *value,
+					      GParamSpec         *pspec);
+static void     gtk_accel_label_destroy      (GtkObject          *object);
+static void     gtk_accel_label_finalize     (GObject            *object);
+static void     gtk_accel_label_size_request (GtkWidget          *widget,
+					      GtkRequisition     *requisition);
+static gboolean gtk_accel_label_expose_event (GtkWidget          *widget,
+					      GdkEventExpose     *event);
+static gboolean gtk_accel_label_refetch_idle (GtkAccelLabel      *accel_label);
 
 static GtkAccelLabelClass *accel_label_class = NULL;
 static GtkLabelClass *parent_class = NULL;
@@ -101,21 +103,28 @@ gtk_accel_label_class_init (GtkAccelLabelClass *class)
   
   parent_class = gtk_type_class (GTK_TYPE_LABEL);
   
-  gtk_object_add_arg_type ("GtkAccelLabel::accel_widget", GTK_TYPE_WIDGET, GTK_ARG_READWRITE, ARG_ACCEL_WIDGET);
-
   gobject_class->finalize = gtk_accel_label_finalize;
-
-  object_class->set_arg = gtk_accel_label_set_arg;
-  object_class->get_arg = gtk_accel_label_get_arg;
+  gobject_class->set_property = gtk_accel_label_set_property;
+  gobject_class->get_property = gtk_accel_label_get_property;
+  
   object_class->destroy = gtk_accel_label_destroy;
   
+  g_object_class_install_property (G_OBJECT_CLASS(object_class),
+                                   PROP_ACCEL_OBJECT,
+                                   g_param_spec_object ("accel_object",
+                                                        _("Accelerator object"),
+                                                        _("The object monitored by this accelerator label"),
+                                                        G_TYPE_OBJECT,
+                                                        G_PARAM_READABLE | G_PARAM_WRITABLE));
+
+   
   widget_class->size_request = gtk_accel_label_size_request;
   widget_class->expose_event = gtk_accel_label_expose_event;
 
   class->signal_quote1 = g_strdup ("<:");
   class->signal_quote2 = g_strdup (":>");
-  class->mod_name_shift = g_strdup ("Shft");
-  class->mod_name_control = g_strdup ("Ctl");
+  class->mod_name_shift = g_strdup ("Shift");
+  class->mod_name_control = g_strdup ("Ctrl");
   class->mod_name_alt = g_strdup ("Alt");
   class->mod_separator = g_strdup ("+");
   class->accel_seperator = g_strdup (" / ");
@@ -123,40 +132,42 @@ gtk_accel_label_class_init (GtkAccelLabelClass *class)
 }
 
 static void
-gtk_accel_label_set_arg (GtkObject      *object,
-			 GtkArg		*arg,
-			 guint		 arg_id)
+gtk_accel_label_set_property (GObject      *object,
+			      guint         prop_id,
+			      const GValue *value,
+			      GParamSpec   *pspec)
 {
   GtkAccelLabel  *accel_label;
 
   accel_label = GTK_ACCEL_LABEL (object);
 
-  switch (arg_id)
+  switch (prop_id)
     {
-    case ARG_ACCEL_WIDGET:
-      gtk_accel_label_set_accel_widget (accel_label, (GtkWidget*) GTK_VALUE_OBJECT (*arg));
+    case PROP_ACCEL_OBJECT:
+      gtk_accel_label_set_accel_object (accel_label, g_value_get_object (value));
       break;
     default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
     }
 }
 
-static void
-gtk_accel_label_get_arg (GtkObject      *object,
-			 GtkArg		*arg,
-			 guint		 arg_id)
+static void gtk_accel_label_get_property (GObject    *object,
+					  guint       prop_id,
+					  GValue     *value,
+					  GParamSpec *pspec)
 {
   GtkAccelLabel  *accel_label;
 
   accel_label = GTK_ACCEL_LABEL (object);
 
-  switch (arg_id)
+  switch (prop_id)
     {
-    case ARG_ACCEL_WIDGET:
-      GTK_VALUE_OBJECT (*arg) = (GtkObject*) accel_label->accel_widget;
+    case PROP_ACCEL_OBJECT:
+       g_value_set_object (value, accel_label->accel_object);
       break;
     default:
-      arg->type = GTK_TYPE_INVALID;
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
     }
 }
@@ -166,7 +177,7 @@ gtk_accel_label_init (GtkAccelLabel *accel_label)
 {
   accel_label->queue_id = 0;
   accel_label->accel_padding = 3;
-  accel_label->accel_widget = NULL;
+  accel_label->accel_object = NULL;
   accel_label->accel_string = NULL;
   
   gtk_accel_label_refetch (accel_label);
@@ -191,12 +202,11 @@ gtk_accel_label_destroy (GtkObject *object)
 {
   GtkAccelLabel *accel_label;
   
-  g_return_if_fail (object != NULL);
   g_return_if_fail (GTK_IS_ACCEL_LABEL (object));
   
   accel_label = GTK_ACCEL_LABEL (object);
 
-  gtk_accel_label_set_accel_widget (accel_label, NULL);
+  gtk_accel_label_set_accel_object (accel_label, NULL);
   
   GTK_OBJECT_CLASS (parent_class)->destroy (object);
 }
@@ -215,10 +225,27 @@ gtk_accel_label_finalize (GObject *object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+/**
+ * gtk_accel_label_get_accel_object:
+ * @accel_label: a #GtkAccelLabel
+ *
+ * Fetches the widget monitored by this accelerator label. See
+ * gtk_accel_label_set_accel_object().
+ *
+ * Return value: the object monitored by the accelerator label,
+ *               or %NULL.
+ **/
+GObject *
+gtk_accel_label_get_accel_object (GtkAccelLabel *accel_label)
+{
+  g_return_val_if_fail (GTK_IS_ACCEL_LABEL (accel_label), NULL);
+
+  return accel_label->accel_object;
+}
+
 guint
 gtk_accel_label_get_accel_width (GtkAccelLabel *accel_label)
 {
-  g_return_val_if_fail (accel_label != NULL, 0);
   g_return_val_if_fail (GTK_IS_ACCEL_LABEL (accel_label), 0);
   
   return (accel_label->accel_string_width +
@@ -233,7 +260,6 @@ gtk_accel_label_size_request (GtkWidget	     *widget,
   PangoLayout *layout;
   gint width;
   
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_ACCEL_LABEL (widget));
   g_return_if_fail (requisition != NULL);
   
@@ -257,7 +283,6 @@ gtk_accel_label_expose_event (GtkWidget      *widget,
   GtkAccelLabel *accel_label;
   PangoLayout *layout;
   
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_ACCEL_LABEL (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
   
@@ -314,7 +339,6 @@ gtk_accel_label_expose_event (GtkWidget      *widget,
 static void
 gtk_accel_label_queue_refetch (GtkAccelLabel *accel_label)
 {
-  g_return_if_fail (accel_label != NULL);
   g_return_if_fail (GTK_IS_ACCEL_LABEL (accel_label));
 
   if (accel_label->queue_id == 0)
@@ -324,41 +348,44 @@ gtk_accel_label_queue_refetch (GtkAccelLabel *accel_label)
 }
 
 void
-gtk_accel_label_set_accel_widget (GtkAccelLabel *accel_label,
-				  GtkWidget	*accel_widget)
+gtk_accel_label_set_accel_object (GtkAccelLabel *accel_label,
+				  GObject	*accel_object)
 {
-  g_return_if_fail (accel_label != NULL);
   g_return_if_fail (GTK_IS_ACCEL_LABEL (accel_label));
-  if (accel_widget != NULL)
-    g_return_if_fail (GTK_IS_WIDGET (accel_widget));
-  
-  if (accel_widget != accel_label->accel_widget)
+  g_return_if_fail (accel_object == NULL || G_IS_OBJECT (accel_object));
+    
+  if (accel_object != accel_label->accel_object)
     {
-      if (accel_label->accel_widget)
+      if (accel_label->accel_object)
 	{
-	  gtk_signal_disconnect_by_func (GTK_OBJECT (accel_label->accel_widget),
-					 GTK_SIGNAL_FUNC (gtk_accel_label_queue_refetch),
-					 accel_label);
-	  gtk_widget_unref (accel_label->accel_widget);
+	  g_signal_handlers_disconnect_by_func (accel_label->accel_object,
+						G_CALLBACK (gtk_accel_label_queue_refetch),
+						accel_object);
+	  g_object_unref (accel_label->accel_object);
 	}
       if (accel_label->queue_id)
 	{
 	  gtk_idle_remove (accel_label->queue_id);
 	  accel_label->queue_id = 0;
 	}
-      accel_label->accel_widget = accel_widget;
-      if (accel_label->accel_widget)
+      accel_label->accel_object = accel_object;
+      if (accel_label->accel_object)
 	{
-	  gtk_widget_ref (accel_label->accel_widget);
-	  gtk_signal_connect_object_after (GTK_OBJECT (accel_label->accel_widget),
-					   "add-accelerator",
-					   GTK_SIGNAL_FUNC (gtk_accel_label_queue_refetch),
-					   GTK_OBJECT (accel_label));
-	  gtk_signal_connect_object_after (GTK_OBJECT (accel_label->accel_widget),
-					   "remove-accelerator",
-					   GTK_SIGNAL_FUNC (gtk_accel_label_queue_refetch),
-					   GTK_OBJECT (accel_label));
+	  g_object_ref (accel_label->accel_object);
+	  g_signal_connect_object (accel_label->accel_object,
+				   "add-accelerator",
+				   G_CALLBACK (gtk_accel_label_queue_refetch),
+				   accel_label,
+				   G_CONNECT_AFTER | G_CONNECT_SWAPPED);
+	  g_signal_connect_object (accel_label->accel_object,
+				   "remove-accelerator",
+				   G_CALLBACK (gtk_accel_label_queue_refetch),
+				   accel_label,
+				   G_CONNECT_AFTER | G_CONNECT_SWAPPED);
 	}
+       gtk_accel_label_refetch (accel_label);
+
+       g_object_notify (G_OBJECT (accel_label), "accel_object");
     }
 }
 
@@ -379,7 +406,6 @@ gtk_accel_label_refetch (GtkAccelLabel *accel_label)
 {
   GtkAccelLabelClass *class;
 
-  g_return_val_if_fail (accel_label != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_ACCEL_LABEL (accel_label), FALSE);
 
   class = GTK_ACCEL_LABEL_GET_CLASS (accel_label);
@@ -387,12 +413,12 @@ gtk_accel_label_refetch (GtkAccelLabel *accel_label)
   g_free (accel_label->accel_string);
   accel_label->accel_string = NULL;
   
-  if (accel_label->accel_widget)
+  if (accel_label->accel_object)
     {
       GtkAccelEntry *entry = NULL;
       GSList *slist;
       
-      slist = gtk_accel_group_entries_from_object (GTK_OBJECT (accel_label->accel_widget));
+      slist = gtk_accel_group_entries_from_object (accel_label->accel_object);
       for (; slist; slist = slist->next)
 	{
 	  entry = slist->data;
