@@ -32,6 +32,7 @@ extern "C" {
 #define GTK_CELL_RENDERER_TOGGLE_CLASS(klass)		(GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_CELL_RENDERER_TOGGLE, GtkCellRendererToggleClass))
 #define GTK_IS_CELL_RENDERER_TOGGLE(obj)		(GTK_CHECK_TYPE ((obj), GTK_TYPE_CELL_RENDERER_TOGGLE))
 #define GTK_IS_CELL_RENDERER_TOGGLE_CLASS(klass)	(GTK_CHECK_CLASS_TYPE ((obj), GTK_TYPE_CELL_RENDERER_TOGGLE))
+#define GTK_CELL_RENDERER_TOGGLE_GET_CLASS(obj)         (GTK_CHECK_GET_CLASS ((obj), GTK_TYPE_CELL_RENDERER_TOGGLE, GtkCellRendererToggleClass))
 
 typedef struct _GtkCellRendererToggle GtkCellRendererToggle;
 typedef struct _GtkCellRendererToggleClass GtkCellRendererToggleClass;
@@ -42,6 +43,7 @@ struct _GtkCellRendererToggle
 
   /*< private >*/
   guint active : 1;
+  guint activatable : 1;
   guint radio : 1;
 };
 
@@ -49,13 +51,14 @@ struct _GtkCellRendererToggleClass
 {
   GtkCellRendererClass parent_class;
 
-  void (* toggled) (GtkCellRendererToggle *celltoggle,
+  void (* toggled) (GtkCellRendererToggle *cell_renderer_toggle,
 		    gchar                 *path);
 };
 
 GtkType          gtk_cell_renderer_toggle_get_type  (void);
 GtkCellRenderer *gtk_cell_renderer_toggle_new       (void);
 
+gboolean         gtk_cell_renderer_toggle_get_radio (GtkCellRendererToggle *toggle);
 void             gtk_cell_renderer_toggle_set_radio (GtkCellRendererToggle *toggle,
 						     gboolean               radio);
 
