@@ -82,7 +82,11 @@ struct _GtkEntry
   guint        activates_default : 1;
 
   guint        cursor_visible : 1;
-  
+
+  guint        in_click : 1;	/* Flag so we don't select all when clicking in entry to focus in */
+
+  guint        is_cell_renderer : 1;
+
   guint   button;
   guint   blink_timeout;
   guint   recompute_idle;
@@ -149,8 +153,10 @@ GtkType    gtk_entry_get_type       		(void) G_GNUC_CONST;
 GtkWidget* gtk_entry_new            		(void);
 void       gtk_entry_set_visibility 		(GtkEntry      *entry,
 						 gboolean       visible);
+gboolean   gtk_entry_get_visibility             (GtkEntry      *entry);
 void       gtk_entry_set_invisible_char         (GtkEntry      *entry,
                                                  gunichar       ch);
+gunichar   gtk_entry_get_invisible_char         (GtkEntry      *entry);
 void       gtk_entry_set_editable   		(GtkEntry      *entry,
 						 gboolean       editable);
 void       gtk_entry_set_has_frame              (GtkEntry      *entry,
@@ -159,6 +165,7 @@ gboolean   gtk_entry_get_has_frame              (GtkEntry      *entry);
 /* text is truncated if needed */
 void       gtk_entry_set_max_length 		(GtkEntry      *entry,
 						 gint           max);
+gint       gtk_entry_get_max_length             (GtkEntry      *entry);
 void       gtk_entry_set_activates_default      (GtkEntry      *entry,
                                                  gboolean       setting);
 gboolean   gtk_entry_get_activates_default      (GtkEntry      *entry);
@@ -181,6 +188,8 @@ void         gtk_entry_get_layout_offsets       (GtkEntry      *entry,
 
 /* Deprecated compatibility functions
  */
+
+#ifndef GTK_DISABLE_DEPRECATED
 GtkWidget* gtk_entry_new_with_max_length	(gint           max);
 void       gtk_entry_append_text    		(GtkEntry      *entry,
 						 const gchar   *text);
@@ -191,6 +200,7 @@ void       gtk_entry_set_position   		(GtkEntry      *entry,
 void       gtk_entry_select_region  		(GtkEntry      *entry,
 						 gint           start,
 						 gint           end);
+#endif /* GTK_DISABLE_DEPRECATED */
 
 #ifdef __cplusplus
 }
