@@ -78,7 +78,14 @@ struct _GtkFontSelection
   GtkWidget *filter_button;
   GtkWidget *preview_entry;
 
-  PangoFontDescription *font_desc;
+  PangoFontFamily **families;
+  PangoFontFamily *family;	/* Current family */
+  
+  PangoFontFace **faces;
+  PangoFontFace *face;		/* Current face */
+  
+  gint size;
+  
   GdkFont *font;		/* Cache for gdk_font_selection_get_font, so we can preserve
 				 * refcounting behavior
 				 */
@@ -123,7 +130,11 @@ struct _GtkFontSelectionDialogClass
 GtkType	   gtk_font_selection_get_type		(void) G_GNUC_CONST;
 GtkWidget* gtk_font_selection_new		(void);
 gchar*	   gtk_font_selection_get_font_name	(GtkFontSelection *fontsel);
+
+#ifndef GTK_DISABLE_DEPRECATED
 GdkFont*   gtk_font_selection_get_font		(GtkFontSelection *fontsel);
+#endif /* GTK_DISABLE_DEPRECATED */
+
 gboolean   gtk_font_selection_set_font_name	(GtkFontSelection *fontsel,
 						 const gchar	  *fontname);
 gchar*	   gtk_font_selection_get_preview_text	(GtkFontSelection *fontsel);
@@ -148,10 +159,12 @@ GtkWidget* gtk_font_selection_dialog_new	(const gchar	  *title);
    You should g_free() the returned font name after you're done with it. */
 gchar*	 gtk_font_selection_dialog_get_font_name    (GtkFontSelectionDialog *fsd);
 
+#ifndef GTK_DISABLE_DEPRECATED
 /* This will return the current GdkFont, or NULL if none is selected or there
    was a problem loading it. Remember to use gdk_font_ref/unref() if you want
    to use the font (in a style, for example). */
 GdkFont* gtk_font_selection_dialog_get_font	    (GtkFontSelectionDialog *fsd);
+#endif /* GTK_DISABLE_DEPRECATED */
 
 /* This sets the currently displayed font. It should be a valid X Logical
    Font Description font name (anything else will be ignored), e.g.
