@@ -43,8 +43,6 @@ static void gtk_image_menu_item_remove               (GtkContainer          *con
 static void gtk_image_menu_item_toggle_size_request  (GtkMenuItem           *menu_item,
 						      gint                  *requisition);
 
-static void gtk_image_menu_item_map        (GtkWidget      *widget);
-static void gtk_image_menu_item_unmap      (GtkWidget      *widget);
 static void gtk_image_menu_item_forall     (GtkContainer   *container,
                                             gboolean	    include_internals,
                                             GtkCallback     callback,
@@ -111,8 +109,6 @@ gtk_image_menu_item_class_init (GtkImageMenuItemClass *klass)
   
   widget_class->size_request = gtk_image_menu_item_size_request;
   widget_class->size_allocate = gtk_image_menu_item_size_allocate;
-  widget_class->map = gtk_image_menu_item_map;
-  widget_class->unmap = gtk_image_menu_item_unmap;
 
   container_class->forall = gtk_image_menu_item_forall;
   container_class->remove = gtk_image_menu_item_remove;
@@ -149,22 +145,11 @@ gtk_image_menu_item_set_property (GObject         *object,
     {
     case PROP_IMAGE:
       {
-        GtkWidget *child;
+        GtkWidget *image;
 
-        child = (GtkWidget*) g_value_get_object (value);
+        image = (GtkWidget*) g_value_get_object (value);
 
-        if (child != image_menu_item->image)
-          {
-            if (image_menu_item->image)
-              gtk_container_remove (GTK_CONTAINER (image_menu_item),
-                                    image_menu_item->image);
-            
-            if (child)
-              {
-                gtk_image_menu_item_add_image (image_menu_item,
-                                               child);
-              }
-          }
+	gtk_image_menu_item_set_image (image_menu_item, image);
       }
       break;
     default:
@@ -199,7 +184,6 @@ gtk_image_menu_item_toggle_size_request (GtkMenuItem *menu_item,
 {
   GtkImageMenuItem *image_menu_item;
   
-  g_return_if_fail (menu_item != NULL);
   g_return_if_fail (GTK_IS_IMAGE_MENU_ITEM (menu_item));
 
   image_menu_item = GTK_IMAGE_MENU_ITEM (menu_item);
@@ -279,44 +263,6 @@ gtk_image_menu_item_size_allocate (GtkWidget     *widget,
     }
 }
 
-static void
-gtk_image_menu_item_map (GtkWidget *widget)
-{
-  GtkImageMenuItem *image_menu_item;
-
-  g_return_if_fail (GTK_IS_IMAGE_MENU_ITEM (widget));
-
-  image_menu_item = GTK_IMAGE_MENU_ITEM (widget);
-  
-  (* GTK_WIDGET_CLASS (parent_class)->map) (widget);
-
-  if (image_menu_item->image &&
-      GTK_WIDGET_VISIBLE (image_menu_item->image) &&
-      !GTK_WIDGET_MAPPED (image_menu_item->image))
-    gtk_widget_map (image_menu_item->image);
-
-  if (!GTK_WIDGET_NO_WINDOW (widget))
-    gdk_window_show (widget->window);  
-}
-
-static void
-gtk_image_menu_item_unmap (GtkWidget *widget)
-{
-  GtkImageMenuItem *image_menu_item;
-
-  g_return_if_fail (GTK_IS_IMAGE_MENU_ITEM (widget));
-
-  image_menu_item = GTK_IMAGE_MENU_ITEM (widget);
-  
-  (* GTK_WIDGET_CLASS (parent_class)->unmap) (widget);
-  
-  if (!GTK_WIDGET_NO_WINDOW (widget))
-    gdk_window_hide (widget->window);
-  
-  if (image_menu_item->image && GTK_WIDGET_MAPPED (image_menu_item->image))
-    gtk_widget_unmap (image_menu_item->image);  
-}
-
 static void
 gtk_image_menu_item_forall (GtkContainer   *container,
                             gboolean	    include_internals,
@@ -339,8 +285,13 @@ gtk_image_menu_item_forall (GtkContainer   *container,
 }
 
 GtkWidget*
-gtk_image_menu_item_new (GtkWidget   *widget,
-                         const gchar *label)
+gtk_image_menu_item_new (void)
+{
+  return g_object_new (GTK_TYPE_IMAGE_MENU_ITEM, NULL);
+}
+
+GtkWidget*
+gtk_image_menu_item_new_with_label (const gchar *label)
 {
   GtkImageMenuItem *image_menu_item;
   GtkWidget *accel_label;
@@ -356,9 +307,38 @@ gtk_image_menu_item_new (GtkWidget   *widget,
                                     GTK_WIDGET (image_menu_item));
   gtk_widget_show (accel_label);
 
-  if (widget)
-    gtk_image_menu_item_add_image (image_menu_item, widget);
+  return GTK_WIDGET(image_menu_item);
+}
+
+
+/**
+ * gtk_image_menu_item_new_with_mnemonic:
+ * @label: The text of the button, with an underscore in front of the
+ *         mnemonic character
+ * @returns: a new #GtkImageMenuItem
+ *
+ * Creates a new #GtkImageMenuItem containing a label. The label
+ * will be created using gtk_label_new_with_mnemonic(), so underscores
+ * in @label indicate the mnemonic for the menu item.
+ **/
+GtkWidget*
+gtk_image_menu_item_new_with_mnemonic (const gchar *label)
+{
+  GtkImageMenuItem *image_menu_item;
+  GtkWidget *accel_label;
   
+  image_menu_item = GTK_IMAGE_MENU_ITEM (g_object_new (GTK_TYPE_IMAGE_MENU_ITEM,
+                                                       NULL));
+
+  accel_label = gtk_type_new (GTK_TYPE_ACCEL_LABEL);
+  gtk_label_set_text_with_mnemonic (GTK_LABEL (accel_label), label);
+  gtk_misc_set_alignment (GTK_MISC (accel_label), 0.0, 0.5);
+
+  gtk_container_add (GTK_CONTAINER (image_menu_item), accel_label);
+  gtk_accel_label_set_accel_widget (GTK_ACCEL_LABEL (accel_label),
+                                    GTK_WIDGET (image_menu_item));
+  gtk_widget_show (accel_label);
+
   return GTK_WIDGET(image_menu_item);
 }
 
@@ -376,7 +356,9 @@ gtk_image_menu_item_new_from_stock (const gchar      *stock_id,
 
   if (gtk_stock_lookup (stock_id, &stock_item))
     {
-      item = gtk_image_menu_item_new (image, stock_item.label);
+      item = gtk_image_menu_item_new_with_mnemonic (stock_item.label);
+
+      gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
       
       if (stock_item.keyval && accel_group)
 	gtk_widget_add_accelerator (item,
@@ -387,34 +369,37 @@ gtk_image_menu_item_new_from_stock (const gchar      *stock_id,
 				    GTK_ACCEL_VISIBLE);
     }
   else
-    item = gtk_image_menu_item_new (image, stock_id);
-  
+    {
+      item = gtk_image_menu_item_new_with_label (stock_id);
+
+      gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+    }
+
   gtk_widget_show (image);
   return item;
 }
 
 void
-gtk_image_menu_item_add_image (GtkImageMenuItem *image_menu_item,
-                               GtkWidget        *child)
+gtk_image_menu_item_set_image (GtkImageMenuItem *image_menu_item,
+                               GtkWidget        *image)
 {
   g_return_if_fail (GTK_IS_IMAGE_MENU_ITEM (image_menu_item));
-  g_return_if_fail (image_menu_item->image == NULL);
-  
-  gtk_widget_set_parent (child, GTK_WIDGET (image_menu_item));
-  image_menu_item->image = child;
+
+  if (image == image_menu_item->image)
+    return;
+
+  if (image_menu_item->image)
+    gtk_container_remove (GTK_CONTAINER (image_menu_item),
+			  image_menu_item->image);
+
+  image_menu_item->image = image;
+
+  if (image == NULL)
+    return;
+
+  gtk_widget_set_parent (image, GTK_WIDGET (image_menu_item));
 
   g_object_notify (G_OBJECT (image_menu_item), "image");
-  
-  if (GTK_WIDGET_REALIZED (child->parent))
-    gtk_widget_realize (child);
-  
-  if (GTK_WIDGET_VISIBLE (child->parent) && GTK_WIDGET_VISIBLE (child))
-    {
-      if (GTK_WIDGET_MAPPED (child->parent))
-	gtk_widget_map (child);
-
-      gtk_widget_queue_resize (child);
-    }
 }
 
 GtkWidget*
