@@ -199,7 +199,7 @@ gtk_input_dialog_init (GtkInputDialog *inputd)
 
       for (tmp_list = device_info; tmp_list; tmp_list = tmp_list->next) {
 	GdkDevice *info = (GdkDevice *)(tmp_list->data);
-	if (info != gdk_core_pointer)
+	if (info != gdk_device_get_core_pointer ())
 	  {
 	    menuitem = gtk_menu_item_new_with_label(info->name);
 
@@ -561,9 +561,9 @@ gtk_input_dialog_set_key (GtkInputKeyInfo *key,
       str = g_string_new("");
       
       if (modifiers & GDK_SHIFT_MASK)
-	g_string_append (str, "Shft+");
+	g_string_append (str, "Shift+");
       if (modifiers & GDK_CONTROL_MASK)
-	g_string_append (str, "Ctl+");
+	g_string_append (str, "Ctrl+");
       if (modifiers & GDK_MOD1_MASK)
 	g_string_append (str, "Alt+");
       
