@@ -38,8 +38,6 @@ enum {
 
 static void gtk_item_class_init (GtkItemClass     *klass);
 static void gtk_item_init       (GtkItem          *item);
-static void gtk_item_map        (GtkWidget        *widget);
-static void gtk_item_unmap      (GtkWidget        *widget);
 static void gtk_item_realize    (GtkWidget        *widget);
 static gint gtk_item_enter      (GtkWidget        *widget,
 				 GdkEventCrossing *event);
@@ -84,6 +82,15 @@ gtk_item_class_init (GtkItemClass *class)
   object_class = (GtkObjectClass*) class;
   widget_class = (GtkWidgetClass*) class;
 
+
+  widget_class->realize = gtk_item_realize;
+  widget_class->enter_notify_event = gtk_item_enter;
+  widget_class->leave_notify_event = gtk_item_leave;
+
+  class->select = NULL;
+  class->deselect = NULL;
+  class->toggle = NULL;
+
   item_signals[SELECT] =
     gtk_signal_new ("select",
                     GTK_RUN_FIRST,
@@ -105,19 +112,7 @@ gtk_item_class_init (GtkItemClass *class)
                     GTK_SIGNAL_OFFSET (GtkItemClass, toggle),
                     gtk_marshal_VOID__VOID,
 		    GTK_TYPE_NONE, 0);
-
-  gtk_object_class_add_signals (object_class, item_signals, LAST_SIGNAL);
-
   widget_class->activate_signal = item_signals[TOGGLE];
-  widget_class->map = gtk_item_map;
-  widget_class->unmap = gtk_item_unmap;
-  widget_class->realize = gtk_item_realize;
-  widget_class->enter_notify_event = gtk_item_enter;
-  widget_class->leave_notify_event = gtk_item_leave;
-
-  class->select = NULL;
-  class->deselect = NULL;
-  class->toggle = NULL;
 }
 
 static void
@@ -145,44 +140,12 @@ gtk_item_toggle (GtkItem *item)
 }
 
 
-static void
-gtk_item_map (GtkWidget *widget)
-{
-  GtkBin *bin;
-
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_ITEM (widget));
-
-  GTK_WIDGET_SET_FLAGS (widget, GTK_MAPPED);
-
-  bin = GTK_BIN (widget);
-
-  if (bin->child &&
-      GTK_WIDGET_VISIBLE (bin->child) &&
-      !GTK_WIDGET_MAPPED (bin->child))
-    gtk_widget_map (bin->child);
-
-  gdk_window_show (widget->window);
-}
-
-static void
-gtk_item_unmap (GtkWidget *widget)
-{
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_ITEM (widget));
-
-  GTK_WIDGET_UNSET_FLAGS (widget, GTK_MAPPED);
-
-  gdk_window_hide (widget->window);
-}
-
 static void
 gtk_item_realize (GtkWidget *widget)
 {
   GdkWindowAttr attributes;
   gint attributes_mask;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_ITEM (widget));
 
   GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
@@ -216,7 +179,6 @@ static gint
 gtk_item_enter (GtkWidget        *widget,
 		GdkEventCrossing *event)
 {
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_ITEM (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -227,7 +189,6 @@ static gint
 gtk_item_leave (GtkWidget        *widget,
 		GdkEventCrossing *event)
 {
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_ITEM (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
