@@ -45,7 +45,6 @@
 #include	"gtk/gtkstock.h"
 #include	"gtk/gtkiconfactory.h"
 #include	<string.h>
-#include	<sys/stat.h>
 #include	<fcntl.h>
 #ifdef HAVE_UNISTD_H
 #include	<unistd.h>
@@ -228,8 +227,8 @@ gtk_item_factory_class_init (GtkItemFactoryClass  *class)
   quark_type_toggle_item	= g_quark_from_static_string ("<ToggleItem>");
   quark_type_image_item         = g_quark_from_static_string ("<ImageItem>");
   quark_type_stock_item         = g_quark_from_static_string ("<StockItem>");
-  quark_type_tearoff_item	= g_quark_from_static_string ("<Tearoff>");
   quark_type_separator_item	= g_quark_from_static_string ("<Separator>");
+  quark_type_tearoff_item	= g_quark_from_static_string ("<Tearoff>");
   quark_type_branch		= g_quark_from_static_string ("<Branch>");
   quark_type_last_branch	= g_quark_from_static_string ("<LastBranch>");
 }
@@ -335,7 +334,7 @@ gtk_item_factory_propagate_accelerator (GtkItemFactoryItem *item,
 	    {
 	      GSList *work;
 	      
-	      work = gtk_accel_group_entries_from_object (GTK_OBJECT (widget));
+	      work = gtk_accel_group_entries_from_object (G_OBJECT (widget));
 	      while (work)
 		{
 		  GtkAccelEntry *ac_entry;
@@ -580,7 +579,6 @@ gtk_item_factory_construct (GtkItemFactory	*ifactory,
 {
   guint len;
 
-  g_return_if_fail (ifactory != NULL);
   g_return_if_fail (GTK_IS_ITEM_FACTORY (ifactory));
   g_return_if_fail (ifactory->accel_group == NULL);
   g_return_if_fail (path != NULL);
@@ -659,7 +657,6 @@ gtk_item_factory_destroy (GtkObject *object)
   GtkItemFactory *ifactory;
   GSList *slist;
 
-  g_return_if_fail (object != NULL);
   g_return_if_fail (GTK_IS_ITEM_FACTORY (object));
 
   ifactory = (GtkItemFactory*) object;
@@ -698,7 +695,6 @@ gtk_item_factory_finalize (GObject *object)
 {
   GtkItemFactory *ifactory;
 
-  g_return_if_fail (object != NULL);
   g_return_if_fail (GTK_IS_ITEM_FACTORY (object));
 
   ifactory = GTK_ITEM_FACTORY (object);
@@ -716,7 +712,6 @@ gtk_item_factory_finalize (GObject *object)
 GtkItemFactory*
 gtk_item_factory_from_widget (GtkWidget	       *widget)
 {
-  g_return_val_if_fail (widget != NULL, NULL);
   g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
 
   return gtk_object_get_data_by_id (GTK_OBJECT (widget), quark_item_factory);
@@ -725,7 +720,6 @@ gtk_item_factory_from_widget (GtkWidget	       *widget)
 gchar*
 gtk_item_factory_path_from_widget (GtkWidget	    *widget)
 {
-  g_return_val_if_fail (widget != NULL, NULL);
   g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
 
   return gtk_object_get_data_by_id (GTK_OBJECT (widget), quark_item_path);
@@ -879,7 +873,6 @@ gtk_item_factory_create_items_ac (GtkItemFactory      *ifactory,
 {
   guint i;
 
-  g_return_if_fail (ifactory != NULL);
   g_return_if_fail (GTK_IS_ITEM_FACTORY (ifactory));
   g_return_if_fail (callback_type >= 1 && callback_type <= 2);
 
@@ -1114,7 +1107,6 @@ gtk_item_factory_create_item (GtkItemFactory	     *ifactory,
   gchar *item_type_path;
   GtkStockItem stock_item;
       
-  g_return_if_fail (ifactory != NULL);
   g_return_if_fail (GTK_IS_ITEM_FACTORY (ifactory));
   g_return_if_fail (entry != NULL);
   g_return_if_fail (entry->path != NULL);
@@ -1228,21 +1220,25 @@ gtk_item_factory_create_item (GtkItemFactory	     *ifactory,
     gtk_radio_menu_item_set_group (GTK_RADIO_MENU_ITEM (widget), radio_group);
   if (GTK_IS_CHECK_MENU_ITEM (widget))
     gtk_check_menu_item_set_show_toggle (GTK_CHECK_MENU_ITEM (widget), TRUE);
-  if (GTK_IS_IMAGE_MENU_ITEM (widget))
+  if (type_id == quark_type_image_item)
     {
       GdkPixbuf *pixbuf = NULL;
       image = NULL;
 
-      pixbuf = gdk_pixbuf_new_from_inline (entry->extra_data,
+      pixbuf = gdk_pixbuf_new_from_stream (-1,
+					   entry->extra_data,
 					   FALSE,
-					   entry->extra_data2,
 					   NULL);
 
       if (pixbuf)
 	image = gtk_image_new_from_pixbuf (pixbuf);
 
       if (image)
-	gtk_image_menu_item_add_image (GTK_IMAGE_MENU_ITEM (widget), image);
+	{
+	  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (widget), image);
+
+	  gtk_widget_show (image);
+	}
 
       if (pixbuf)
 	g_object_unref (G_OBJECT (pixbuf));
@@ -1250,9 +1246,11 @@ gtk_item_factory_create_item (GtkItemFactory	     *ifactory,
   if (type_id == quark_type_stock_item)
     {
       image = gtk_image_new_from_stock (entry->extra_data, GTK_ICON_SIZE_MENU);
-      if (image)
-	gtk_image_menu_item_add_image (GTK_IMAGE_MENU_ITEM (widget), image);
-      
+
+      gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (widget), image);
+
+      gtk_widget_show (image);
+
       if (gtk_stock_lookup (entry->extra_data, &stock_item))
 	{
 	  if (!accelerator)
@@ -1273,7 +1271,7 @@ gtk_item_factory_create_item (GtkItemFactory	     *ifactory,
       label = gtk_widget_new (GTK_TYPE_ACCEL_LABEL,
 			      "GtkWidget::visible", TRUE,
 			      "GtkWidget::parent", widget,
-			      "GtkAccelLabel::accel_widget", widget,
+			      "GtkAccelLabel::accel_object", widget,
 			      "GtkMisc::xalign", 0.0,
 			      NULL);
       
@@ -1290,7 +1288,7 @@ gtk_item_factory_create_item (GtkItemFactory	     *ifactory,
 		   entry->path);
       
       if (type_id == quark_type_last_branch)
-	gtk_menu_item_right_justify (GTK_MENU_ITEM (widget));
+	gtk_menu_item_set_right_justified (GTK_MENU_ITEM (widget), TRUE);
       
       parent = widget;
       widget = gtk_widget_new (GTK_TYPE_MENU,
@@ -1449,7 +1447,6 @@ gtk_item_factory_delete_item (GtkItemFactory         *ifactory,
   GtkItemFactoryClass *class;
   GtkWidget *widget;
 
-  g_return_if_fail (ifactory != NULL);
   g_return_if_fail (GTK_IS_ITEM_FACTORY (ifactory));
   g_return_if_fail (path != NULL);
 
@@ -1470,7 +1467,6 @@ void
 gtk_item_factory_delete_entry (GtkItemFactory         *ifactory,
 			       GtkItemFactoryEntry    *entry)
 {
-  g_return_if_fail (ifactory != NULL);
   g_return_if_fail (GTK_IS_ITEM_FACTORY (ifactory));
   g_return_if_fail (entry != NULL);
 
@@ -1484,7 +1480,6 @@ gtk_item_factory_delete_entries (GtkItemFactory         *ifactory,
 {
   guint i;
 
-  g_return_if_fail (ifactory != NULL);
   g_return_if_fail (GTK_IS_ITEM_FACTORY (ifactory));
   if (n_entries > 0)
     g_return_if_fail (entries != NULL);
@@ -1517,7 +1512,6 @@ gtk_item_factory_popup_data_from_widget (GtkWidget     *widget)
 {
   GtkItemFactory *ifactory;
   
-  g_return_val_if_fail (widget != NULL, NULL);
   g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
 
   ifactory = gtk_item_factory_from_widget (widget);
@@ -1530,7 +1524,6 @@ gtk_item_factory_popup_data_from_widget (GtkWidget     *widget)
 gpointer
 gtk_item_factory_popup_data (GtkItemFactory *ifactory)
 {
-  g_return_val_if_fail (ifactory != NULL, NULL);
   g_return_val_if_fail (GTK_IS_ITEM_FACTORY (ifactory), NULL);
 
   return gtk_object_get_data_by_id (GTK_OBJECT (ifactory), quark_popup_data);
@@ -1567,7 +1560,6 @@ gtk_item_factory_popup_with_data (GtkItemFactory	*ifactory,
 {
   MenuPos *mpos;
 
-  g_return_if_fail (ifactory != NULL);
   g_return_if_fail (GTK_IS_ITEM_FACTORY (ifactory));
   g_return_if_fail (GTK_IS_MENU (ifactory->widget));
   
@@ -1767,7 +1759,7 @@ gtk_item_factory_parse_rc (const gchar	  *file_name)
 
   g_return_if_fail (file_name != NULL);
 
-  if (!S_ISREG (g_scanner_stat_mode (file_name)))
+  if (!g_file_test (file_name, G_FILE_TEST_IS_REGULAR))
     return;
 
   fd = open (file_name, O_RDONLY);
