@@ -30,7 +30,7 @@
 #include <gdk/gdk.h>
 #include <gtk/gtkmisc.h>
 #include <gtk/gtkwindow.h>
-
+#include <gtk/gtkmenu.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -65,6 +65,7 @@ struct _GtkLabel
   
   gchar  *text; 
   PangoAttrList *attrs;
+  PangoAttrList *effective_attrs;
   
   PangoLayout *layout;
 
@@ -77,47 +78,75 @@ struct _GtkLabel
 struct _GtkLabelClass
 {
   GtkMiscClass parent_class;
+
+  void (* move_cursor)     (GtkLabel       *label,
+			    GtkMovementStep step,
+			    gint            count,
+			    gboolean        extend_selection);
+  void (* copy_clipboard)  (GtkLabel       *label);
+  void (* select_all)      (GtkLabel       *label);
+  
+  /* Hook to customize right-click popup for selectable labels */
+  void (* populate_popup)   (GtkLabel       *label,
+                             GtkMenu        *menu);
 };
 
 GtkType               gtk_label_get_type          (void) G_GNUC_CONST;
-GtkWidget*            gtk_label_new               (const char       *str);
-GtkWidget*            gtk_label_new_with_mnemonic (const char       *str);
-void                  gtk_label_set_text          (GtkLabel         *label,
-						   const char       *str);
-G_CONST_RETURN gchar* gtk_label_get_text          (GtkLabel         *label);
-void                  gtk_label_set_attributes    (GtkLabel         *label,
-						   PangoAttrList    *attrs);
-void                  gtk_label_set_markup        (GtkLabel         *label,
-						   const gchar      *str);
+GtkWidget*            gtk_label_new               (const char    *str);
+GtkWidget*            gtk_label_new_with_mnemonic (const char    *str);
+void                  gtk_label_set_text          (GtkLabel      *label,
+						   const char    *str);
+G_CONST_RETURN gchar* gtk_label_get_text          (GtkLabel      *label);
+void                  gtk_label_set_attributes    (GtkLabel      *label,
+						   PangoAttrList *attrs);
+PangoAttrList        *gtk_label_get_attributes    (GtkLabel      *label);
+void                  gtk_label_set_label         (GtkLabel      *label,
+						   const gchar   *str);
+G_CONST_RETURN gchar *gtk_label_get_label         (GtkLabel      *label);
+void                  gtk_label_set_markup        (GtkLabel      *label,
+						   const gchar   *str);
+void                  gtk_label_set_use_markup    (GtkLabel      *label,
+						   gboolean       setting);
+gboolean              gtk_label_get_use_markup    (GtkLabel      *label);
+void                  gtk_label_set_use_underline (GtkLabel      *label,
+						   gboolean       setting);
+gboolean              gtk_label_get_use_underline (GtkLabel      *label);
+
 void     gtk_label_set_markup_with_mnemonic       (GtkLabel         *label,
 						   const gchar      *str);
 guint    gtk_label_get_mnemonic_keyval            (GtkLabel         *label);
 void     gtk_label_set_mnemonic_widget            (GtkLabel         *label,
 						   GtkWidget        *widget);
+GtkWidget *gtk_label_get_mnemonic_widget          (GtkLabel         *label);
 void     gtk_label_set_text_with_mnemonic         (GtkLabel         *label,
 						   const gchar      *str);
 void     gtk_label_set_justify                    (GtkLabel         *label,
 						   GtkJustification  jtype);
+GtkJustification gtk_label_get_justify            (GtkLabel         *label);
 void     gtk_label_set_pattern                    (GtkLabel         *label,
 						   const gchar      *pattern);
 void     gtk_label_set_line_wrap                  (GtkLabel         *label,
 						   gboolean          wrap);
+gboolean gtk_label_get_line_wrap                  (GtkLabel         *label);
 void     gtk_label_set_selectable                 (GtkLabel         *label,
 						   gboolean          setting);
 gboolean gtk_label_get_selectable                 (GtkLabel         *label);
 void     gtk_label_select_region                  (GtkLabel         *label,
 						   gint              start_offset,
 						   gint              end_offset);
-void     gtk_label_get_layout_offsets             (GtkLabel         *label,
-						   gint             *x,
-						   gint             *y);
+gboolean gtk_label_get_selection_bounds           (GtkLabel         *label,
+                                                   gint             *start,
+                                                   gint             *end);
+
+PangoLayout *gtk_label_get_layout         (GtkLabel *label);
+void         gtk_label_get_layout_offsets (GtkLabel *label,
+                                           gint     *x,
+                                           gint     *y);
 
-#ifndef	GTK_DISABLE_COMPAT_H
-#  define gtk_label_set				gtk_label_set_text
-#endif	/* GTK_DISABLE_COMPAT_H */
 
 #ifndef GTK_DISABLE_DEPRECATED
-/* Deprecated */
+
+#define  gtk_label_set           gtk_label_set_text
 void       gtk_label_get           (GtkLabel          *label,
                                     char             **str);
 
