@@ -67,7 +67,7 @@ GtkListStore *gtk_list_store_new             (gint          n_columns,
 					      ...);
 GtkListStore *gtk_list_store_newv            (gint          n_columns,
 					      GType        *types);
-void          gtk_list_store_set_value       (GtkListStore *store,
+void          gtk_list_store_set_value       (GtkListStore *list_store,
 					      GtkTreeIter  *iter,
 					      gint          column,
 					      GValue       *value);
@@ -77,22 +77,22 @@ void          gtk_list_store_set             (GtkListStore *list_store,
 void          gtk_list_store_set_valist      (GtkListStore *list_store,
 					      GtkTreeIter  *iter,
 					      va_list       var_args);
-void          gtk_list_store_remove          (GtkListStore *store,
+void          gtk_list_store_remove          (GtkListStore *list_store,
 					      GtkTreeIter  *iter);
-void          gtk_list_store_insert          (GtkListStore *store,
+void          gtk_list_store_insert          (GtkListStore *list_store,
 					      GtkTreeIter  *iter,
 					      gint          position);
-void          gtk_list_store_insert_before   (GtkListStore *store,
+void          gtk_list_store_insert_before   (GtkListStore *list_store,
 					      GtkTreeIter  *iter,
 					      GtkTreeIter  *sibling);
-void          gtk_list_store_insert_after    (GtkListStore *store,
+void          gtk_list_store_insert_after    (GtkListStore *list_store,
 					      GtkTreeIter  *iter,
 					      GtkTreeIter  *sibling);
-void          gtk_list_store_prepend         (GtkListStore *store,
+void          gtk_list_store_prepend         (GtkListStore *list_store,
 					      GtkTreeIter  *iter);
-void          gtk_list_store_append          (GtkListStore *store,
+void          gtk_list_store_append          (GtkListStore *list_store,
 					      GtkTreeIter  *iter);
-void          gtk_list_store_clear           (GtkListStore *store);
+void          gtk_list_store_clear           (GtkListStore *list_store);
 
 
 #ifdef __cplusplus
