@@ -611,7 +611,46 @@ gtk_marshal_VOID__BOXED_BOXED (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,BOXED,POINTER (gtkmarshal.list:46) */
+/* VOID:BOXED,BOXED,BOXED (gtkmarshal.list:46) */
+void
+gtk_marshal_VOID__BOXED_BOXED_BOXED (GClosure     *closure,
+                                     GValue       *return_value,
+                                     guint         n_param_values,
+                                     const GValue *param_values,
+                                     gpointer      invocation_hint,
+                                     gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__BOXED_BOXED_BOXED) (gpointer     data1,
+                                                        gpointer     arg_1,
+                                                        gpointer     arg_2,
+                                                        gpointer     arg_3,
+                                                        gpointer     data2);
+  register GMarshalFunc_VOID__BOXED_BOXED_BOXED callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 4);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__BOXED_BOXED_BOXED) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_value_get_boxed (param_values + 1),
+            g_value_get_boxed (param_values + 2),
+            g_value_get_boxed (param_values + 3),
+            data2);
+}
+
+/* VOID:BOXED,BOXED,POINTER (gtkmarshal.list:47) */
 void
 gtk_marshal_VOID__BOXED_BOXED_POINTER (GClosure     *closure,
                                        GValue       *return_value,
@@ -650,7 +689,7 @@ gtk_marshal_VOID__BOXED_BOXED_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,POINTER (gtkmarshal.list:47) */
+/* VOID:BOXED,POINTER (gtkmarshal.list:48) */
 void
 gtk_marshal_VOID__BOXED_POINTER (GClosure     *closure,
                                  GValue       *return_value,
@@ -687,7 +726,7 @@ gtk_marshal_VOID__BOXED_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,OBJECT (gtkmarshal.list:48) */
+/* VOID:BOXED,OBJECT (gtkmarshal.list:49) */
 void
 gtk_marshal_VOID__BOXED_OBJECT (GClosure     *closure,
                                 GValue       *return_value,
@@ -724,7 +763,7 @@ gtk_marshal_VOID__BOXED_OBJECT (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,STRING,INT (gtkmarshal.list:49) */
+/* VOID:BOXED,STRING,INT (gtkmarshal.list:50) */
 void
 gtk_marshal_VOID__BOXED_STRING_INT (GClosure     *closure,
                                     GValue       *return_value,
@@ -763,7 +802,44 @@ gtk_marshal_VOID__BOXED_STRING_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,UINT (gtkmarshal.list:50) */
+/* VOID:BOXED,INT (gtkmarshal.list:51) */
+void
+gtk_marshal_VOID__BOXED_INT (GClosure     *closure,
+                             GValue       *return_value,
+                             guint         n_param_values,
+                             const GValue *param_values,
+                             gpointer      invocation_hint,
+                             gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__BOXED_INT) (gpointer     data1,
+                                                gpointer     arg_1,
+                                                gint         arg_2,
+                                                gpointer     data2);
+  register GMarshalFunc_VOID__BOXED_INT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__BOXED_INT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_value_get_boxed (param_values + 1),
+            g_value_get_int (param_values + 2),
+            data2);
+}
+
+/* VOID:BOXED,UINT (gtkmarshal.list:52) */
 void
 gtk_marshal_VOID__BOXED_UINT (GClosure     *closure,
                               GValue       *return_value,
@@ -800,7 +876,7 @@ gtk_marshal_VOID__BOXED_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,UINT,FLAGS (gtkmarshal.list:51) */
+/* VOID:BOXED,UINT,FLAGS (gtkmarshal.list:53) */
 void
 gtk_marshal_VOID__BOXED_UINT_FLAGS (GClosure     *closure,
                                     GValue       *return_value,
@@ -839,7 +915,7 @@ gtk_marshal_VOID__BOXED_UINT_FLAGS (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,UINT,UINT (gtkmarshal.list:52) */
+/* VOID:BOXED,UINT,UINT (gtkmarshal.list:54) */
 void
 gtk_marshal_VOID__BOXED_UINT_UINT (GClosure     *closure,
                                    GValue       *return_value,
@@ -878,9 +954,9 @@ gtk_marshal_VOID__BOXED_UINT_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:ENUM (gtkmarshal.list:53) */
+/* VOID:ENUM (gtkmarshal.list:55) */
 
-/* VOID:ENUM,ENUM (gtkmarshal.list:54) */
+/* VOID:ENUM,ENUM (gtkmarshal.list:56) */
 void
 gtk_marshal_VOID__ENUM_ENUM (GClosure     *closure,
                              GValue       *return_value,
@@ -917,7 +993,7 @@ gtk_marshal_VOID__ENUM_ENUM (GClosure     *closure,
             data2);
 }
 
-/* VOID:ENUM,FLOAT (gtkmarshal.list:55) */
+/* VOID:ENUM,FLOAT (gtkmarshal.list:57) */
 void
 gtk_marshal_VOID__ENUM_FLOAT (GClosure     *closure,
                               GValue       *return_value,
@@ -954,7 +1030,7 @@ gtk_marshal_VOID__ENUM_FLOAT (GClosure     *closure,
             data2);
 }
 
-/* VOID:ENUM,FLOAT,BOOL (gtkmarshal.list:56) */
+/* VOID:ENUM,FLOAT,BOOL (gtkmarshal.list:58) */
 void
 gtk_marshal_VOID__ENUM_FLOAT_BOOLEAN (GClosure     *closure,
                                       GValue       *return_value,
@@ -993,7 +1069,7 @@ gtk_marshal_VOID__ENUM_FLOAT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:ENUM,INT (gtkmarshal.list:57) */
+/* VOID:ENUM,INT (gtkmarshal.list:59) */
 void
 gtk_marshal_VOID__ENUM_INT (GClosure     *closure,
                             GValue       *return_value,
@@ -1030,7 +1106,7 @@ gtk_marshal_VOID__ENUM_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:ENUM,INT,BOOLEAN (gtkmarshal.list:58) */
+/* VOID:ENUM,INT,BOOLEAN (gtkmarshal.list:60) */
 void
 gtk_marshal_VOID__ENUM_INT_BOOLEAN (GClosure     *closure,
                                     GValue       *return_value,
@@ -1069,11 +1145,11 @@ gtk_marshal_VOID__ENUM_INT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:INT (gtkmarshal.list:59) */
+/* VOID:INT (gtkmarshal.list:61) */
 
-/* VOID:INT,INT (gtkmarshal.list:60) */
+/* VOID:INT,INT (gtkmarshal.list:62) */
 
-/* VOID:INT,INT,BOXED (gtkmarshal.list:61) */
+/* VOID:INT,INT,BOXED (gtkmarshal.list:63) */
 void
 gtk_marshal_VOID__INT_INT_BOXED (GClosure     *closure,
                                  GValue       *return_value,
@@ -1112,7 +1188,7 @@ gtk_marshal_VOID__INT_INT_BOXED (GClosure     *closure,
             data2);
 }
 
-/* VOID:INT,INT,INT (gtkmarshal.list:62) */
+/* VOID:INT,INT,INT (gtkmarshal.list:64) */
 void
 gtk_marshal_VOID__INT_INT_INT (GClosure     *closure,
                                GValue       *return_value,
@@ -1151,9 +1227,9 @@ gtk_marshal_VOID__INT_INT_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT (gtkmarshal.list:63) */
+/* VOID:OBJECT (gtkmarshal.list:65) */
 
-/* VOID:OBJECT,BOOLEAN (gtkmarshal.list:64) */
+/* VOID:OBJECT,BOOLEAN (gtkmarshal.list:66) */
 void
 gtk_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                   GValue       *return_value,
@@ -1190,7 +1266,7 @@ gtk_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,BOXED,BOXED (gtkmarshal.list:65) */
+/* VOID:OBJECT,BOXED,BOXED (gtkmarshal.list:67) */
 void
 gtk_marshal_VOID__OBJECT_BOXED_BOXED (GClosure     *closure,
                                       GValue       *return_value,
@@ -1229,7 +1305,7 @@ gtk_marshal_VOID__OBJECT_BOXED_BOXED (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,BOXED,UINT,UINT (gtkmarshal.list:66) */
+/* VOID:OBJECT,BOXED,UINT,UINT (gtkmarshal.list:68) */
 void
 gtk_marshal_VOID__OBJECT_BOXED_UINT_UINT (GClosure     *closure,
                                           GValue       *return_value,
@@ -1270,7 +1346,7 @@ gtk_marshal_VOID__OBJECT_BOXED_UINT_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,INT,INT (gtkmarshal.list:67) */
+/* VOID:OBJECT,INT,INT (gtkmarshal.list:69) */
 void
 gtk_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                   GValue       *return_value,
@@ -1309,7 +1385,7 @@ gtk_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (gtkmarshal.list:68) */
+/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (gtkmarshal.list:70) */
 void
 gtk_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closure,
                                                   GValue       *return_value,
@@ -1354,7 +1430,7 @@ gtk_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,OBJECT (gtkmarshal.list:69) */
+/* VOID:OBJECT,OBJECT (gtkmarshal.list:71) */
 void
 gtk_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                  GValue       *return_value,
@@ -1391,7 +1467,7 @@ gtk_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,STRING,STRING (gtkmarshal.list:70) */
+/* VOID:OBJECT,STRING,STRING (gtkmarshal.list:72) */
 void
 gtk_marshal_VOID__OBJECT_STRING_STRING (GClosure     *closure,
                                         GValue       *return_value,
@@ -1430,7 +1506,7 @@ gtk_marshal_VOID__OBJECT_STRING_STRING (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,UINT (gtkmarshal.list:71) */
+/* VOID:OBJECT,UINT (gtkmarshal.list:73) */
 void
 gtk_marshal_VOID__OBJECT_UINT (GClosure     *closure,
                                GValue       *return_value,
@@ -1467,7 +1543,7 @@ gtk_marshal_VOID__OBJECT_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,UINT,FLAGS (gtkmarshal.list:72) */
+/* VOID:OBJECT,UINT,FLAGS (gtkmarshal.list:74) */
 void
 gtk_marshal_VOID__OBJECT_UINT_FLAGS (GClosure     *closure,
                                      GValue       *return_value,
@@ -1506,9 +1582,9 @@ gtk_marshal_VOID__OBJECT_UINT_FLAGS (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER (gtkmarshal.list:73) */
+/* VOID:POINTER (gtkmarshal.list:75) */
 
-/* VOID:POINTER,INT (gtkmarshal.list:74) */
+/* VOID:POINTER,INT (gtkmarshal.list:76) */
 void
 gtk_marshal_VOID__POINTER_INT (GClosure     *closure,
                                GValue       *return_value,
@@ -1545,7 +1621,7 @@ gtk_marshal_VOID__POINTER_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,POINTER,POINTER (gtkmarshal.list:75) */
+/* VOID:POINTER,POINTER,POINTER (gtkmarshal.list:77) */
 void
 gtk_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
                                            GValue       *return_value,
@@ -1584,7 +1660,7 @@ gtk_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,UINT (gtkmarshal.list:76) */
+/* VOID:POINTER,UINT (gtkmarshal.list:78) */
 void
 gtk_marshal_VOID__POINTER_UINT (GClosure     *closure,
                                 GValue       *return_value,
@@ -1621,11 +1697,48 @@ gtk_marshal_VOID__POINTER_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING (gtkmarshal.list:77) */
+/* VOID:STRING (gtkmarshal.list:79) */
 
-/* VOID:STRING,INT,POINTER (gtkmarshal.list:78) */
+/* VOID:STRING,STRING (gtkmarshal.list:80) */
+void
+gtk_marshal_VOID__STRING_STRING (GClosure     *closure,
+                                 GValue       *return_value,
+                                 guint         n_param_values,
+                                 const GValue *param_values,
+                                 gpointer      invocation_hint,
+                                 gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__STRING_STRING) (gpointer     data1,
+                                                    gpointer     arg_1,
+                                                    gpointer     arg_2,
+                                                    gpointer     data2);
+  register GMarshalFunc_VOID__STRING_STRING callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
 
-/* VOID:UINT,BOXED,UINT,FLAGS,FLAGS (gtkmarshal.list:79) */
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__STRING_STRING) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            (char*) g_value_get_string (param_values + 1),
+            (char*) g_value_get_string (param_values + 2),
+            data2);
+}
+
+/* VOID:STRING,INT,POINTER (gtkmarshal.list:81) */
+
+/* VOID:UINT,BOXED,UINT,FLAGS,FLAGS (gtkmarshal.list:82) */
 void
 gtk_marshal_VOID__UINT_BOXED_UINT_FLAGS_FLAGS (GClosure     *closure,
                                                GValue       *return_value,
@@ -1668,7 +1781,7 @@ gtk_marshal_VOID__UINT_BOXED_UINT_FLAGS_FLAGS (GClosure     *closure,
             data2);
 }
 
-/* VOID:UINT,OBJECT,UINT,FLAGS,FLAGS (gtkmarshal.list:80) */
+/* VOID:UINT,OBJECT,UINT,FLAGS,FLAGS (gtkmarshal.list:83) */
 void
 gtk_marshal_VOID__UINT_OBJECT_UINT_FLAGS_FLAGS (GClosure     *closure,
                                                 GValue       *return_value,
@@ -1711,7 +1824,7 @@ gtk_marshal_VOID__UINT_OBJECT_UINT_FLAGS_FLAGS (GClosure     *closure,
             data2);
 }
 
-/* VOID:UINT,STRING (gtkmarshal.list:81) */
+/* VOID:UINT,STRING (gtkmarshal.list:84) */
 void
 gtk_marshal_VOID__UINT_STRING (GClosure     *closure,
                                GValue       *return_value,
@@ -1748,5 +1861,5 @@ gtk_marshal_VOID__UINT_STRING (GClosure     *closure,
             data2);
 }
 
-/* VOID:VOID (gtkmarshal.list:82) */
+/* VOID:VOID (gtkmarshal.list:85) */
 
