@@ -58,7 +58,6 @@ struct _GtkMenuItem
   guint16 toggle_size;
   guint16 accelerator_width;
   
-  guint show_toggle_indicator : 1;
   guint show_submenu_indicator : 1;
   guint submenu_placement : 1;
   guint submenu_direction : 1;
@@ -90,14 +89,13 @@ struct _GtkMenuItemClass
 GtkType	   gtk_menu_item_get_type	      (void) G_GNUC_CONST;
 GtkWidget* gtk_menu_item_new                  (void);
 GtkWidget* gtk_menu_item_new_with_label       (const gchar         *label);
+GtkWidget* gtk_menu_item_new_with_mnemonic    (const gchar         *label);
 void       gtk_menu_item_set_submenu          (GtkMenuItem         *menu_item,
 					       GtkWidget           *submenu);
+GtkWidget* gtk_menu_item_get_submenu          (GtkMenuItem         *menu_item);
 void       gtk_menu_item_remove_submenu       (GtkMenuItem         *menu_item);
-void       gtk_menu_item_set_placement        (GtkMenuItem         *menu_item,
+void       _gtk_menu_item_set_placement       (GtkMenuItem         *menu_item,
 					       GtkSubmenuPlacement  placement);
-void       gtk_menu_item_configure            (GtkMenuItem         *menu_item,
-					       gboolean		    show_toggle_indicator,
-					       gboolean		    show_submenu_indicator);
 void       gtk_menu_item_select               (GtkMenuItem         *menu_item);
 void       gtk_menu_item_deselect             (GtkMenuItem         *menu_item);
 void       gtk_menu_item_activate             (GtkMenuItem         *menu_item);
@@ -105,9 +103,13 @@ void       gtk_menu_item_toggle_size_request  (GtkMenuItem         *menu_item,
 					       gint                *requisition);
 void       gtk_menu_item_toggle_size_allocate (GtkMenuItem         *menu_item,
 					       gint                 allocation);
-void       gtk_menu_item_right_justify        (GtkMenuItem         *menu_item);
-
+void       gtk_menu_item_set_right_justified  (GtkMenuItem         *menu_item,
+					       gboolean             right_justified);
+gboolean   gtk_menu_item_get_right_justified  (GtkMenuItem         *menu_item);
 
+#ifndef GTK_DISABLE_DEPRECATED
+#define gtk_menu_item_right_justify(menu_item) gtk_menu_item_set_right_justified ((menu_item), TRUE)
+#endif /* GTK_DISABLE_DEPRECATED */
 
 #ifdef __cplusplus
 }
