@@ -67,7 +67,7 @@ static gint gtk_option_menu_button_press    (GtkWidget          *widget,
 					     GdkEventButton     *event);
 static gint gtk_option_menu_key_press	    (GtkWidget          *widget,
 					     GdkEventKey        *event);
-static void gtk_option_menu_deactivate      (GtkMenuShell       *menu_shell,
+static void gtk_option_menu_selection_done  (GtkMenuShell       *menu_shell,
 					     GtkOptionMenu      *option_menu);
 static void gtk_option_menu_update_contents (GtkOptionMenu      *option_menu);
 static void gtk_option_menu_remove_contents (GtkOptionMenu      *option_menu);
@@ -80,6 +80,8 @@ static void gtk_option_menu_position        (GtkMenu            *menu,
 static void gtk_option_menu_show_all        (GtkWidget          *widget);
 static void gtk_option_menu_hide_all        (GtkWidget          *widget);
 static GtkType gtk_option_menu_child_type   (GtkContainer       *container);
+static gint gtk_option_menu_scroll_event    (GtkWidget          *widget,
+					     GdkEventScroll     *event);
 
 enum
 {
@@ -132,13 +134,13 @@ gtk_option_menu_class_init (GtkOptionMenuClass *class)
   parent_class = gtk_type_class (gtk_button_get_type ());
 
   signals[CHANGED] =
-    g_signal_newc ("changed",
-                   G_OBJECT_CLASS_TYPE (class),
-                   G_SIGNAL_RUN_LAST,
-                   G_STRUCT_OFFSET (GtkOptionMenuClass, changed),
-                   NULL, NULL,
-                   gtk_marshal_VOID__VOID,
-                   G_TYPE_NONE, 0);
+    g_signal_new ("changed",
+                  G_OBJECT_CLASS_TYPE (class),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (GtkOptionMenuClass, changed),
+                  NULL, NULL,
+                  gtk_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
   
   object_class->destroy = gtk_option_menu_destroy;
   
@@ -147,6 +149,7 @@ gtk_option_menu_class_init (GtkOptionMenuClass *class)
   widget_class->expose_event = gtk_option_menu_expose;
   widget_class->button_press_event = gtk_option_menu_button_press;
   widget_class->key_press_event = gtk_option_menu_key_press;
+  widget_class->scroll_event = gtk_option_menu_scroll_event;
   widget_class->show_all = gtk_option_menu_show_all;
   widget_class->hide_all = gtk_option_menu_hide_all;
 
@@ -193,7 +196,6 @@ gtk_option_menu_new (void)
 GtkWidget*
 gtk_option_menu_get_menu (GtkOptionMenu *option_menu)
 {
-  g_return_val_if_fail (option_menu != NULL, NULL);
   g_return_val_if_fail (GTK_IS_OPTION_MENU (option_menu), NULL);
 
   return option_menu->menu;
@@ -205,7 +207,6 @@ gtk_option_menu_detacher (GtkWidget     *widget,
 {
   GtkOptionMenu *option_menu;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (widget));
 
   option_menu = GTK_OPTION_MENU (widget);
@@ -222,9 +223,7 @@ void
 gtk_option_menu_set_menu (GtkOptionMenu *option_menu,
 			  GtkWidget     *menu)
 {
-  g_return_if_fail (option_menu != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (option_menu));
-  g_return_if_fail (menu != NULL);
   g_return_if_fail (GTK_IS_MENU (menu));
 
   if (option_menu->menu != menu)
@@ -238,9 +237,9 @@ gtk_option_menu_set_menu (GtkOptionMenu *option_menu,
 
       gtk_option_menu_calc_size (option_menu);
 
-      gtk_signal_connect (GTK_OBJECT (option_menu->menu), "deactivate",
-			  (GtkSignalFunc) gtk_option_menu_deactivate,
-			  option_menu);
+      gtk_signal_connect_after (GTK_OBJECT (option_menu->menu), "selection_done",
+				G_CALLBACK (gtk_option_menu_selection_done),
+				option_menu);
       gtk_signal_connect_object (GTK_OBJECT (option_menu->menu), "size_request",
 				 (GtkSignalFunc) gtk_option_menu_calc_size,
 				 GTK_OBJECT (option_menu));
@@ -255,7 +254,6 @@ gtk_option_menu_set_menu (GtkOptionMenu *option_menu,
 void
 gtk_option_menu_remove_menu (GtkOptionMenu *option_menu)
 {
-  g_return_if_fail (option_menu != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (option_menu));
 
   if (option_menu->menu)
@@ -268,7 +266,6 @@ gtk_option_menu_set_history (GtkOptionMenu *option_menu,
 {
   GtkWidget *menu_item;
 
-  g_return_if_fail (option_menu != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (option_menu));
 
   if (option_menu->menu)
@@ -316,7 +313,6 @@ gtk_option_menu_destroy (GtkObject *object)
 {
   GtkOptionMenu *option_menu;
 
-  g_return_if_fail (object != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (object));
 
   option_menu = GTK_OPTION_MENU (object);
@@ -397,30 +393,26 @@ gtk_option_menu_size_allocate (GtkWidget     *widget,
   GtkWidget *child;
   GtkAllocation child_allocation;
   GtkOptionMenuProps props;
+  gint border_width;
     
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_OPTION_MENU (widget));
-  g_return_if_fail (allocation != NULL);
-
   gtk_option_menu_get_props (GTK_OPTION_MENU (widget), &props);
+  border_width = GTK_CONTAINER (widget)->border_width;
 
   widget->allocation = *allocation;
   if (GTK_WIDGET_REALIZED (widget))
     gdk_window_move_resize (widget->window,
-			    allocation->x, allocation->y,
-			    allocation->width, allocation->height);
+			    allocation->x + border_width, allocation->y + border_width,
+			    allocation->width - border_width * 2, allocation->height - border_width * 2);
 
   child = GTK_BIN (widget)->child;
   if (child && GTK_WIDGET_VISIBLE (child))
     {
-      child_allocation.x = (GTK_CONTAINER (widget)->border_width +
-			    GTK_WIDGET (widget)->style->xthickness) + 1;
-      child_allocation.y = (GTK_CONTAINER (widget)->border_width +
-			    GTK_WIDGET (widget)->style->ythickness) + 1;
-      child_allocation.width = MAX (1, (gint)allocation->width - child_allocation.x * 2 -
+      child_allocation.x = GTK_WIDGET (widget)->style->xthickness + 1;
+      child_allocation.y = GTK_CONTAINER (widget)->border_width + 1;
+      child_allocation.width = MAX (1, (gint)allocation->width - child_allocation.x * 2 - border_width * 2 -
 				    props.indicator_size.width - props.indicator_spacing.left - props.indicator_spacing.right -
 				    CHILD_LEFT_SPACING - CHILD_RIGHT_SPACING - 2);
-      child_allocation.height = MAX (1, (gint)allocation->height - child_allocation.y * 2 -
+      child_allocation.height = MAX (1, (gint)allocation->height - child_allocation.y * 2 - border_width * 2 -
 				     CHILD_TOP_SPACING - CHILD_BOTTOM_SPACING - 2);
       child_allocation.x += CHILD_LEFT_SPACING;
       child_allocation.y += CHILD_TOP_SPACING;
@@ -435,19 +427,18 @@ gtk_option_menu_paint (GtkWidget    *widget,
 {
   GdkRectangle button_area;
   GtkOptionMenuProps props;
+  gint border_width;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (widget));
   g_return_if_fail (area != NULL);
 
   if (GTK_WIDGET_DRAWABLE (widget))
     {
-      gint border_width = GTK_CONTAINER (widget)->border_width;
-	
+      border_width = GTK_CONTAINER (widget)->border_width;
       gtk_option_menu_get_props (GTK_OPTION_MENU (widget), &props);
 
-      button_area.x = border_width;
-      button_area.y = border_width;
+      button_area.x = 0;
+      button_area.y = 0;
       button_area.width = widget->allocation.width - 2 * border_width;
       button_area.height = widget->allocation.height - 2 * border_width;
 
@@ -466,11 +457,11 @@ gtk_option_menu_paint (GtkWidget    *widget,
 	  gdk_window_clear_area (widget->window, area->x, area->y, area->width, area->height);
 	}
 
-      gtk_paint_box(widget->style, widget->window,
-		    GTK_WIDGET_STATE (widget), GTK_SHADOW_OUT,
-		    area, widget, "optionmenu",
-		    button_area.x, button_area.y,
-		    button_area.width, button_area.height);
+      gtk_paint_box (widget->style, widget->window,
+		     GTK_WIDGET_STATE (widget), GTK_SHADOW_OUT,
+		     area, widget, "optionmenu",
+		     button_area.x, button_area.y,
+		     button_area.width, button_area.height);
       
       gtk_paint_tab (widget->style, widget->window,
 		     GTK_WIDGET_STATE (widget), GTK_SHADOW_OUT,
@@ -513,7 +504,6 @@ static gint
 gtk_option_menu_expose (GtkWidget      *widget,
 			GdkEventExpose *event)
 {
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_OPTION_MENU (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -573,7 +563,6 @@ gtk_option_menu_button_press (GtkWidget      *widget,
   GtkOptionMenu *option_menu;
   GtkWidget *menu_item;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_OPTION_MENU (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -602,7 +591,6 @@ gtk_option_menu_key_press (GtkWidget   *widget,
   GtkOptionMenu *option_menu;
   GtkWidget *menu_item;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_OPTION_MENU (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -610,6 +598,7 @@ gtk_option_menu_key_press (GtkWidget   *widget,
 
   switch (event->keyval)
     {
+    case GDK_KP_Space:
     case GDK_space:
       gtk_option_menu_remove_contents (option_menu);
       gtk_menu_popup (GTK_MENU (option_menu->menu), NULL, NULL,
@@ -625,11 +614,10 @@ gtk_option_menu_key_press (GtkWidget   *widget,
 }
 
 static void
-gtk_option_menu_deactivate (GtkMenuShell  *menu_shell,
-			    GtkOptionMenu *option_menu)
+gtk_option_menu_selection_done (GtkMenuShell  *menu_shell,
+				GtkOptionMenu *option_menu)
 {
   g_return_if_fail (menu_shell != NULL);
-  g_return_if_fail (option_menu != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (option_menu));
 
   gtk_option_menu_update_contents (option_menu);
@@ -699,7 +687,6 @@ gtk_option_menu_update_contents (GtkOptionMenu *option_menu)
   GtkWidget *child;
   GtkRequisition child_requisition;
 
-  g_return_if_fail (option_menu != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (option_menu));
 
   if (option_menu->menu)
@@ -743,7 +730,6 @@ gtk_option_menu_remove_contents (GtkOptionMenu *option_menu)
 {
   GtkWidget *child;
   
-  g_return_if_fail (option_menu != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (option_menu));
 
   if (option_menu->menu_item)
@@ -777,7 +763,6 @@ gtk_option_menu_calc_size (GtkOptionMenu *option_menu)
   gint old_width = option_menu->width;
   gint old_height = option_menu->height;
 
-  g_return_if_fail (option_menu != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (option_menu));
 
   option_menu->width = 0;
@@ -822,7 +807,6 @@ gtk_option_menu_position (GtkMenu  *menu,
   gint menu_ypos;
   gint menu_width;
 
-  g_return_if_fail (user_data != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (user_data));
 
   option_menu = GTK_OPTION_MENU (user_data);
@@ -877,7 +861,6 @@ gtk_option_menu_show_all (GtkWidget *widget)
   GtkContainer *container;
   GtkOptionMenu *option_menu;
   
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (widget));
   container = GTK_CONTAINER (widget);
   option_menu = GTK_OPTION_MENU (widget);
@@ -896,7 +879,6 @@ gtk_option_menu_hide_all (GtkWidget *widget)
 {
   GtkContainer *container;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (widget));
   container = GTK_CONTAINER (widget);
 
@@ -904,3 +886,49 @@ gtk_option_menu_hide_all (GtkWidget *widget)
   gtk_container_foreach (container, (GtkCallback) gtk_widget_hide_all, NULL);
 }
 
+static gint
+gtk_option_menu_scroll_event (GtkWidget          *widget,
+			      GdkEventScroll     *event)
+{
+  GtkOptionMenu *option_menu = GTK_OPTION_MENU (widget);
+  gint index;
+  gint n_children;
+  gint index_dir;
+  GList *l;
+  GtkMenuItem *item;
+    
+  index = gtk_option_menu_get_history (option_menu);
+
+  if (index != -1)
+    {
+      n_children = g_list_length (GTK_MENU_SHELL (option_menu->menu)->children);
+      
+      if (event->direction == GDK_SCROLL_UP)
+	index_dir = -1;
+      else
+	index_dir = 1;
+
+
+      while (TRUE)
+	{
+	  index += index_dir;
+
+	  if (index < 0)
+	    break;
+	  if (index >= n_children)
+	    break;
+
+	  l = g_list_nth (GTK_MENU_SHELL (option_menu->menu)->children, index);
+	  item = GTK_MENU_ITEM (l->data);
+	  if (GTK_WIDGET_VISIBLE (item) && GTK_WIDGET_IS_SENSITIVE (item))
+	    {
+	      gtk_option_menu_set_history (option_menu, index);
+	      break;
+	    }
+	      
+	}
+    }
+
+  return TRUE;
+}
+
