@@ -17,11 +17,18 @@
  */
 
 #include "gtksettings.h"
+#include "gtkrc.h"
 #include "gtkintl.h"
+#include "gtkwidget.h"
 
 enum {
   PROP_0,
-  PROP_DOUBLE_CLICK_TIMEOUT
+  PROP_DOUBLE_CLICK_TIME,
+  PROP_CURSOR_BLINK,
+  PROP_CURSOR_BLINK_TIME,
+  PROP_SPLIT_CURSOR,
+  PROP_THEME_NAME,
+  PROP_KEY_THEME_NAME
 };
 
 
@@ -51,7 +58,6 @@ static guint	settings_install_property_parser (GtkSettingsClass      *class,
 static gpointer		 parent_class = NULL;
 static GtkSettings	*the_singleton = NULL;
 static GQuark		 quark_property_parser = 0;
-static GQuark		 quark_property_id = 0;
 static GSList           *object_list = NULL;
 static guint		 class_n_properties = 0;
 
@@ -86,8 +92,8 @@ gtk_settings_get_type (void)
 static void
 gtk_settings_init (GtkSettings *settings)
 {
-  GObjectClass *gobject_class = G_OBJECT_GET_CLASS (settings);
-  guint i;
+  GParamSpec **pspecs, **p;
+  guint i = 0;
   
   g_datalist_init (&settings->queued_settings);
   object_list = g_slist_prepend (object_list, settings);
@@ -96,15 +102,26 @@ gtk_settings_init (GtkSettings *settings)
    * notification for them (at least notification for internal properties
    * will instantly be caught)
    */
-  settings->property_values = g_new0 (GValue, class_n_properties);
-  for (i = 0; i < class_n_properties; i++)
+  pspecs = g_object_class_list_properties (G_OBJECT_GET_CLASS (settings), NULL);
+  for (p = pspecs; *p; p++)
+    if ((*p)->owner_type == G_OBJECT_TYPE (settings))
+      i++;
+  settings->property_values = g_new0 (GValue, i);
+  i = 0;
+  g_object_freeze_notify (G_OBJECT (settings));
+  for (p = pspecs; *p; p++)
     {
-      GParamSpec *pspec = gobject_class->property_specs[i]; // FIXME: g_object_list_properties(this_class_type)
+      GParamSpec *pspec = *p;
 
+      if (pspec->owner_type != G_OBJECT_TYPE (settings))
+	continue;
       g_value_init (settings->property_values + i, G_PARAM_SPEC_VALUE_TYPE (pspec));
       g_param_value_set_default (pspec, settings->property_values + i);
       g_object_notify (G_OBJECT (settings), pspec->name);
+      i++;
     }
+  g_object_thaw_notify (G_OBJECT (settings));
+  g_free (pspecs);
 }
 
 static void
@@ -122,17 +139,55 @@ gtk_settings_class_init (GtkSettingsClass *class)
   gobject_class->notify = gtk_settings_notify;
 
   quark_property_parser = g_quark_from_static_string ("gtk-rc-property-parser");
-  quark_property_id = g_quark_try_string ("GObject-property-id");
-  g_assert (quark_property_id != 0);	/* special quarks from GObjectClass */
 
   result = settings_install_property_parser (class,
-                                             g_param_spec_int ("gtk-double-click-timeout",
-                                                               _("Double Click Timeout"),
-                                                               _("Maximum time allowed between two clicks for them to be considered a double click"),
+                                             g_param_spec_int ("gtk-double-click-time",
+                                                               _("Double Click Time"),
+                                                               _("Maximum time allowed between two clicks for them to be considered a double click (in milliseconds)"),
                                                                0, G_MAXINT, 250,
                                                                G_PARAM_READWRITE),
                                              NULL);
-  g_assert (result == PROP_DOUBLE_CLICK_TIMEOUT);
+  g_assert (result == PROP_DOUBLE_CLICK_TIME);
+  result = settings_install_property_parser (class,
+                                             g_param_spec_boolean ("gtk-cursor-blink",
+								   _("Cursor Blink"),
+								   _("Whether the cursor should blink"),
+								   TRUE,
+								   G_PARAM_READWRITE),
+					     NULL);
+  g_assert (result == PROP_CURSOR_BLINK);
+  result = settings_install_property_parser (class,
+                                             g_param_spec_int ("gtk-cursor-blink-time",
+                                                               _("Cursor Blink Time"),
+                                                               _("Length of the cursor blink cycle, in milleseconds"),
+                                                               100, G_MAXINT, 1200,
+                                                               G_PARAM_READWRITE),
+                                             NULL);
+  g_assert (result == PROP_CURSOR_BLINK_TIME);
+  result = settings_install_property_parser (class,
+                                             g_param_spec_boolean ("gtk-split-cursor",
+								   _("Split Cursor"),
+								   _("Whether two cursors should be displayed for mixed left-to-right and right-to-left text"),
+								   TRUE,
+								   G_PARAM_READWRITE),
+                                             NULL);
+  g_assert (result == PROP_SPLIT_CURSOR);
+  result = settings_install_property_parser (class,
+                                             g_param_spec_string ("gtk-theme-name",
+								   _("Theme Name"),
+								   _("Name of theme RC file to load"),
+								  NULL,
+								  G_PARAM_READWRITE),
+                                             NULL);
+  g_assert (result == PROP_THEME_NAME);
+  result = settings_install_property_parser (class,
+                                             g_param_spec_string ("gtk-key-theme-name",
+								  _("Key Theme Name"),
+								  _("Name of key theme RC file to load"),
+								  NULL,
+								  G_PARAM_READWRITE),
+                                             NULL);
+  g_assert (result == PROP_KEY_THEME_NAME);
 }
 
 static void
@@ -174,7 +229,7 @@ gtk_settings_constructor (GType			 type,
 }
 
 GtkSettings*
-gtk_settings_get_global (void)
+gtk_settings_get_default (void)
 {
   if (!the_singleton)
     g_object_new (GTK_TYPE_SETTINGS, NULL);
@@ -201,15 +256,65 @@ gtk_settings_get_property (GObject     *object,
 {
   GtkSettings *settings = GTK_SETTINGS (object);
 
-  if (!gdk_setting_get (pspec->name, value))
-    g_value_copy (settings->property_values + property_id - 1, value);
+  if (g_value_type_transformable (G_TYPE_INT, G_VALUE_TYPE (value)) ||
+      g_value_type_transformable (G_TYPE_STRING, G_VALUE_TYPE (value)) ||
+      g_value_type_transformable (GDK_TYPE_COLOR, G_VALUE_TYPE (value)))
+    {
+      if (gdk_setting_get (pspec->name, value))
+        g_param_value_validate (pspec, value);
+      else
+        g_value_copy (settings->property_values + property_id - 1, value);
+    }
+  else
+    {
+      GValue val = { 0, };
+
+      /* Try to get xsetting as a string and parse it. */
+      
+      g_value_init (&val, G_TYPE_STRING);
+
+      if (!gdk_setting_get (pspec->name, &val))
+        {
+          g_value_copy (settings->property_values + property_id - 1, value);
+        }
+      else
+        {
+          GValue tmp_value = { 0, };
+          GValue gstring_value = { 0, };
+          GtkRcPropertyParser parser = g_param_spec_get_qdata (pspec,
+                                                               quark_property_parser);
+          
+          g_value_init (&gstring_value, G_TYPE_GSTRING);
+
+          g_value_set_boxed (&gstring_value,
+                             g_string_new (g_value_get_string (&val)));
+
+          g_value_init (&tmp_value, G_PARAM_SPEC_VALUE_TYPE (pspec));
+
+          if (parser && _gtk_settings_parse_convert (parser, &gstring_value,
+                                                     pspec, &tmp_value))
+            {
+              g_value_copy (&tmp_value, value);
+              g_param_value_validate (pspec, value);
+            }
+          else
+            {
+              g_value_copy (settings->property_values + property_id - 1, value);
+            }
+
+          g_value_unset (&gstring_value);
+          g_value_unset (&tmp_value);
+        }
+
+      g_value_unset (&val);
+    }
 }
 
 static void
 gtk_settings_notify (GObject    *object,
 		     GParamSpec *pspec)
 {
-  guint property_id = GPOINTER_TO_UINT (g_param_spec_get_qdata ((pspec), quark_property_id));
+  guint property_id = pspec->param_id;
   gint double_click_time;
   
 #if 1
@@ -221,7 +326,7 @@ gtk_settings_notify (GObject    *object,
 
   switch (property_id)
     {
-    case PROP_DOUBLE_CLICK_TIMEOUT:
+    case PROP_DOUBLE_CLICK_TIME:
       g_object_get (object, pspec->name, &double_click_time, NULL);
       gdk_set_double_click_time (double_click_time);
       break;
@@ -339,9 +444,11 @@ settings_install_property_parser (GtkSettingsClass   *class,
       break;
     default:
       if (!parser)
-	g_warning (G_STRLOC ": parser needs to be specified for property \"%s\" of type `%s'",
-		   pspec->name, g_type_name (G_PARAM_SPEC_VALUE_TYPE (pspec)));
-      return 0;
+        {
+          g_warning (G_STRLOC ": parser needs to be specified for property \"%s\" of type `%s'",
+                     pspec->name, g_type_name (G_PARAM_SPEC_VALUE_TYPE (pspec)));
+          return 0;
+        }
     }
   if (g_object_class_find_property (G_OBJECT_CLASS (class), pspec->name))
     {
@@ -399,29 +506,25 @@ _gtk_rc_property_parser_from_type (GType type)
 }
 
 void
-gtk_settings_install_property (GtkSettings *settings,
-			       GParamSpec  *pspec)
+gtk_settings_install_property (GParamSpec  *pspec)
 {
   GtkRcPropertyParser parser = NULL;
 
-  g_return_if_fail (GTK_IS_SETTINGS (settings));
   g_return_if_fail (G_IS_PARAM_SPEC (pspec));
 
   parser = _gtk_rc_property_parser_from_type (G_PARAM_SPEC_VALUE_TYPE (pspec));
 
-  settings_install_property_parser (GTK_SETTINGS_GET_CLASS (settings), pspec, parser);
+  settings_install_property_parser (gtk_type_class (GTK_TYPE_SETTINGS), pspec, parser);
 }
 
 void
-gtk_settings_install_property_parser (GtkSettings        *settings,
-				      GParamSpec         *pspec,
+gtk_settings_install_property_parser (GParamSpec         *pspec,
 				      GtkRcPropertyParser parser)
 {
-  g_return_if_fail (GTK_IS_SETTINGS (settings));
   g_return_if_fail (G_IS_PARAM_SPEC (pspec));
   g_return_if_fail (parser != NULL);
   
-  settings_install_property_parser (GTK_SETTINGS_GET_CLASS (settings), pspec, parser);
+  settings_install_property_parser (gtk_type_class (GTK_TYPE_SETTINGS), pspec, parser);
 }
 
 static void
@@ -597,7 +700,7 @@ gtk_rc_property_parse_enum (const GParamSpec *pspec,
   if (scanner->token == G_TOKEN_IDENTIFIER)
     {
       GEnumClass *class = G_PARAM_SPEC_ENUM (pspec)->enum_class;
-
+      
       enum_value = g_enum_get_value_by_name (class, scanner->value.v_identifier);
       if (!enum_value)
 	enum_value = g_enum_get_value_by_nick (class, scanner->value.v_identifier);
@@ -799,8 +902,8 @@ gtk_rc_property_parse_border (const GParamSpec *pspec,
 void
 _gtk_settings_handle_event (GdkEventSetting *event)
 {
-  GtkSettings *settings = gtk_settings_get_global ();
-
+  GtkSettings *settings = gtk_settings_get_default ();
+  
   if (g_object_class_find_property (G_OBJECT_GET_CLASS (settings), event->name))
     g_object_notify (G_OBJECT (settings), event->name);
 }
