@@ -37,26 +37,26 @@
 #include "gtkmain.h"
 #include "gtksignal.h"
 #include "gtksettings.h"
-
+#include "gtkintl.h"
 
 #define MIN_SPIN_BUTTON_WIDTH              30
 #define ARROW_SIZE                         11
 #define SPIN_BUTTON_INITIAL_TIMER_DELAY    200
 #define SPIN_BUTTON_TIMER_DELAY            20
-#define MAX_TEXT_LENGTH                    256
 #define MAX_TIMER_CALLS                    5
 #define EPSILON                            1e-5
+#define	MAX_DIGITS			   20
 
 enum {
-  ARG_0,
-  ARG_ADJUSTMENT,
-  ARG_CLIMB_RATE,
-  ARG_DIGITS,
-  ARG_SNAP_TO_TICKS,
-  ARG_NUMERIC,
-  ARG_WRAP,
-  ARG_UPDATE_POLICY,
-  ARG_VALUE
+  PROP_0,
+  PROP_ADJUSTMENT,
+  PROP_CLIMB_RATE,
+  PROP_DIGITS,
+  PROP_SNAP_TO_TICKS,
+  PROP_NUMERIC,
+  PROP_WRAP,
+  PROP_UPDATE_POLICY,
+  PROP_VALUE
 };
 
 /* Signals */
@@ -71,12 +71,14 @@ enum
 static void gtk_spin_button_class_init     (GtkSpinButtonClass *klass);
 static void gtk_spin_button_init           (GtkSpinButton      *spin_button);
 static void gtk_spin_button_finalize       (GObject            *object);
-static void gtk_spin_button_set_arg        (GtkObject          *object,
-					    GtkArg             *arg,
-					    guint               arg_id);
-static void gtk_spin_button_get_arg        (GtkObject          *object,
-					    GtkArg             *arg,
-					    guint               arg_id);
+static void gtk_spin_button_set_property   (GObject         *object,
+					    guint            prop_id,
+					    const GValue    *value,
+					    GParamSpec      *pspec);
+static void gtk_spin_button_get_property   (GObject         *object,
+					    guint            prop_id,
+					    GValue          *value,
+					    GParamSpec      *pspec);
 static void gtk_spin_button_map            (GtkWidget          *widget);
 static void gtk_spin_button_unmap          (GtkWidget          *widget);
 static void gtk_spin_button_realize        (GtkWidget          *widget);
@@ -132,7 +134,7 @@ static guint spinbutton_signals[LAST_SIGNAL] = {0};
 GtkType
 gtk_spin_button_get_type (void)
 {
-  static guint spin_button_type = 0;
+  static GtkType spin_button_type = 0;
 
   if (!spin_button_type)
     {
@@ -169,8 +171,8 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
 
   gobject_class->finalize = gtk_spin_button_finalize;
 
-  object_class->set_arg = gtk_spin_button_set_arg;
-  object_class->get_arg = gtk_spin_button_get_arg;
+  gobject_class->set_property = gtk_spin_button_set_property;
+  gobject_class->get_property = gtk_spin_button_get_property;
 
   widget_class->map = gtk_spin_button_map;
   widget_class->unmap = gtk_spin_button_unmap;
@@ -195,38 +197,76 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
   class->input = NULL;
   class->output = NULL;
 
-  gtk_object_add_arg_type ("GtkSpinButton::adjustment",
-			   GTK_TYPE_ADJUSTMENT,
-			   GTK_ARG_READWRITE,
-			   ARG_ADJUSTMENT);
-  gtk_object_add_arg_type ("GtkSpinButton::climb_rate",
-			   GTK_TYPE_DOUBLE,
-			   GTK_ARG_READWRITE,
-			   ARG_CLIMB_RATE);
-  gtk_object_add_arg_type ("GtkSpinButton::digits",
-			   GTK_TYPE_UINT,
-			   GTK_ARG_READWRITE,
-			   ARG_DIGITS);
-  gtk_object_add_arg_type ("GtkSpinButton::snap_to_ticks",
-			   GTK_TYPE_BOOL,
-			   GTK_ARG_READWRITE,
-			   ARG_SNAP_TO_TICKS);
-  gtk_object_add_arg_type ("GtkSpinButton::numeric",
-			   GTK_TYPE_BOOL,
-			   GTK_ARG_READWRITE,
-			   ARG_NUMERIC);
-  gtk_object_add_arg_type ("GtkSpinButton::wrap",
-			   GTK_TYPE_BOOL,
-			   GTK_ARG_READWRITE,
-			   ARG_WRAP);
-  gtk_object_add_arg_type ("GtkSpinButton::update_policy",
-			   GTK_TYPE_SPIN_BUTTON_UPDATE_POLICY,
-			   GTK_ARG_READWRITE,
-			   ARG_UPDATE_POLICY);
-  gtk_object_add_arg_type ("GtkSpinButton::value",
-			   GTK_TYPE_DOUBLE,
-			   GTK_ARG_READWRITE,
-			   ARG_VALUE);
+  g_object_class_install_property (gobject_class,
+                                   PROP_ADJUSTMENT,
+                                   g_param_spec_object ("adjustment",
+                                                        _("Adjustment"),
+                                                        _("The adjustment that holds the value of the spinbutton"),
+                                                        GTK_TYPE_ADJUSTMENT,
+                                                        G_PARAM_READWRITE));
+  
+  g_object_class_install_property (gobject_class,
+                                   PROP_CLIMB_RATE,
+                                   g_param_spec_double ("climb_rate",
+							_("Climb Rate"),
+							_("The acceleration rate when you hold down a button"),
+							0.0,
+							G_MAXDOUBLE,
+							0.0,
+							G_PARAM_READWRITE));  
+  
+  g_object_class_install_property (gobject_class,
+                                   PROP_DIGITS,
+                                   g_param_spec_uint ("digits",
+						      _("Digits"),
+						      _("The number of decimal places to display"),
+						      0,
+						      MAX_DIGITS,
+						      0,
+						      G_PARAM_READWRITE));
+  
+  g_object_class_install_property (gobject_class,
+                                   PROP_SNAP_TO_TICKS,
+                                   g_param_spec_boolean ("snap_to_ticks",
+							 _("Snap to Ticks"),
+							 _("Whether erroneous values are automatically changed to a spin button's nearest step increment"),
+							 FALSE,
+							 G_PARAM_READWRITE));
+  
+  g_object_class_install_property (gobject_class,
+                                   PROP_NUMERIC,
+                                   g_param_spec_boolean ("numeric",
+							 _("Numeric"),
+							 _("Whether non-numeric characters should be ignored"),
+							 FALSE,
+							 G_PARAM_READWRITE));
+  
+  g_object_class_install_property (gobject_class,
+                                   PROP_WRAP,
+                                   g_param_spec_boolean ("wrap",
+							 _("Wrap"),
+							 _("Whether a spin button should wrap upon reaching its limits"),
+							 FALSE,
+							 G_PARAM_READWRITE));
+  
+  g_object_class_install_property (gobject_class,
+                                   PROP_UPDATE_POLICY,
+                                   g_param_spec_enum ("update_policy",
+						      _("Update Policy"),
+						      _("Whether the spin button should update always, or only when the value is legal"),
+						      GTK_TYPE_SPIN_BUTTON_UPDATE_POLICY,
+						      GTK_UPDATE_ALWAYS,
+						      G_PARAM_READWRITE));
+  
+  g_object_class_install_property (gobject_class,
+                                   PROP_VALUE,
+                                   g_param_spec_double ("value",
+							_("Value"),
+							_("Reads the current value, or sets a new value"),
+							-G_MAXDOUBLE,
+							G_MAXDOUBLE,
+							0.0,
+							G_PARAM_READWRITE));  
   
   gtk_widget_class_install_style_property_parser (widget_class,
 						  g_param_spec_enum ("shadow_type", "Shadow Type", NULL,
@@ -243,13 +283,13 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
 		    GTK_TYPE_INT, 1, GTK_TYPE_POINTER);
 
   spinbutton_signals[OUTPUT] =
-    g_signal_newc ("output",
-		   G_TYPE_FROM_CLASS(object_class),
-		   G_SIGNAL_RUN_LAST,
-		   G_STRUCT_OFFSET(GtkSpinButtonClass, output),
-		   _gtk_boolean_handled_accumulator, NULL,
-		   gtk_marshal_BOOLEAN__VOID,
-		   G_TYPE_BOOLEAN, 0);
+    g_signal_new ("output",
+                  G_TYPE_FROM_CLASS(object_class),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET(GtkSpinButtonClass, output),
+                  _gtk_boolean_handled_accumulator, NULL,
+                  gtk_marshal_BOOLEAN__VOID,
+                  G_TYPE_BOOLEAN, 0);
 
   spinbutton_signals[VALUE_CHANGED] =
     gtk_signal_new ("value_changed",
@@ -261,50 +301,51 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
 }
 
 static void
-gtk_spin_button_set_arg (GtkObject        *object,
-			 GtkArg           *arg,
-			 guint             arg_id)
+gtk_spin_button_set_property (GObject      *object,
+			      guint         prop_id,
+			      const GValue *value,
+			      GParamSpec   *pspec)
 {
   GtkSpinButton *spin_button;
 
   spin_button = GTK_SPIN_BUTTON (object);
   
-  switch (arg_id)
+  switch (prop_id)
     {
       GtkAdjustment *adjustment;
 
-    case ARG_ADJUSTMENT:
-      adjustment = GTK_VALUE_POINTER (*arg);
+    case PROP_ADJUSTMENT:
+      adjustment = GTK_ADJUSTMENT (g_value_get_object (value));
       if (!adjustment)
 	adjustment = (GtkAdjustment*) gtk_adjustment_new (0.0, 0.0, 0.0, 0.0, 0.0, 0.0);
       gtk_spin_button_set_adjustment (spin_button, adjustment);
       break;
-    case ARG_CLIMB_RATE:
+    case PROP_CLIMB_RATE:
       gtk_spin_button_configure (spin_button,
 				 spin_button->adjustment,
-				 GTK_VALUE_DOUBLE (*arg),
+				 g_value_get_double (value),
 				 spin_button->digits);
       break;
-    case ARG_DIGITS:
+    case PROP_DIGITS:
       gtk_spin_button_configure (spin_button,
 				 spin_button->adjustment,
 				 spin_button->climb_rate,
-				 GTK_VALUE_UINT (*arg));
+				 g_value_get_uint (value));
       break;
-    case ARG_SNAP_TO_TICKS:
-      gtk_spin_button_set_snap_to_ticks (spin_button, GTK_VALUE_BOOL (*arg));
+    case PROP_SNAP_TO_TICKS:
+      gtk_spin_button_set_snap_to_ticks (spin_button, g_value_get_boolean (value));
       break;
-    case ARG_NUMERIC:
-      gtk_spin_button_set_numeric (spin_button, GTK_VALUE_BOOL (*arg));
+    case PROP_NUMERIC:
+      gtk_spin_button_set_numeric (spin_button, g_value_get_boolean (value));
       break;
-    case ARG_WRAP:
-      gtk_spin_button_set_wrap (spin_button, GTK_VALUE_BOOL (*arg));
+    case PROP_WRAP:
+      gtk_spin_button_set_wrap (spin_button, g_value_get_boolean (value));
       break;
-    case ARG_UPDATE_POLICY:
-      gtk_spin_button_set_update_policy (spin_button, GTK_VALUE_ENUM (*arg));
+    case PROP_UPDATE_POLICY:
+      gtk_spin_button_set_update_policy (spin_button, g_value_get_enum (value));
       break;
-    case ARG_VALUE:
-      gtk_spin_button_set_value (spin_button, GTK_VALUE_DOUBLE (*arg));
+    case PROP_VALUE:
+      gtk_spin_button_set_value (spin_button, g_value_get_double (value));
       break;
     default:
       break;
@@ -312,42 +353,43 @@ gtk_spin_button_set_arg (GtkObject        *object,
 }
 
 static void
-gtk_spin_button_get_arg (GtkObject        *object,
-			 GtkArg           *arg,
-			 guint             arg_id)
+gtk_spin_button_get_property (GObject      *object,
+			      guint         prop_id,
+			      GValue       *value,
+			      GParamSpec   *pspec)
 {
   GtkSpinButton *spin_button;
 
   spin_button = GTK_SPIN_BUTTON (object);
   
-  switch (arg_id)
+  switch (prop_id)
     {
-    case ARG_ADJUSTMENT:
-      GTK_VALUE_POINTER (*arg) = spin_button->adjustment;
+    case PROP_ADJUSTMENT:
+      g_value_set_object (value, G_OBJECT (spin_button->adjustment));
       break;
-    case ARG_CLIMB_RATE:
-      GTK_VALUE_DOUBLE (*arg) = spin_button->climb_rate;
+    case PROP_CLIMB_RATE:
+      g_value_set_double (value, spin_button->climb_rate);
       break;
-    case ARG_DIGITS:
-      GTK_VALUE_UINT (*arg) = spin_button->digits;
+    case PROP_DIGITS:
+      g_value_set_uint (value, spin_button->digits);
       break;
-    case ARG_SNAP_TO_TICKS:
-      GTK_VALUE_BOOL (*arg) = spin_button->snap_to_ticks;
+    case PROP_SNAP_TO_TICKS:
+      g_value_set_boolean (value, spin_button->snap_to_ticks);
       break;
-    case ARG_NUMERIC:
-      GTK_VALUE_BOOL (*arg) = spin_button->numeric;
+    case PROP_NUMERIC:
+      g_value_set_boolean (value, spin_button->numeric);
       break;
-    case ARG_WRAP:
-      GTK_VALUE_BOOL (*arg) = spin_button->wrap;
+    case PROP_WRAP:
+      g_value_set_boolean (value, spin_button->wrap);
       break;
-    case ARG_UPDATE_POLICY:
-      GTK_VALUE_ENUM (*arg) = spin_button->update_policy;
+    case PROP_UPDATE_POLICY:
+      g_value_set_enum (value, spin_button->update_policy);
       break;
-    case ARG_VALUE:
-      GTK_VALUE_DOUBLE (*arg) = spin_button->adjustment->value;
+     case PROP_VALUE:
+       g_value_set_double (value, spin_button->adjustment->value);
       break;
     default:
-      arg->type = GTK_TYPE_INVALID;
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
     }
 }
@@ -388,7 +430,6 @@ gtk_spin_button_finalize (GObject *object)
 static void
 gtk_spin_button_map (GtkWidget *widget)
 {
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_SPIN_BUTTON (widget));
 
   if (GTK_WIDGET_REALIZED (widget) && !GTK_WIDGET_MAPPED (widget))
@@ -401,7 +442,6 @@ gtk_spin_button_map (GtkWidget *widget)
 static void
 gtk_spin_button_unmap (GtkWidget *widget)
 {
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_SPIN_BUTTON (widget));
 
   if (GTK_WIDGET_MAPPED (widget))
@@ -420,7 +460,6 @@ gtk_spin_button_realize (GtkWidget *widget)
   guint real_width;
   gint return_val;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_SPIN_BUTTON (widget));
   
   spin_button = GTK_SPIN_BUTTON (widget);
@@ -469,7 +508,6 @@ gtk_spin_button_unrealize (GtkWidget *widget)
 {
   GtkSpinButton *spin;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_SPIN_BUTTON (widget));
 
   spin = GTK_SPIN_BUTTON (widget);
@@ -484,6 +522,29 @@ gtk_spin_button_unrealize (GtkWidget *widget)
     }
 }
 
+static int
+compute_double_length (double val, int digits)
+{
+  int a;
+  int extra;
+
+  a = 1;
+  if (fabs (val) > 1.0)
+    a = floor (log10 (fabs (val))) + 1;  
+
+  extra = 0;
+  
+  /* The dot: */
+  if (digits > 0)
+    extra++;
+
+  /* The sign: */
+  if (val < 0)
+    extra++;
+
+  return a + digits + extra;
+}
+
 static void
 gtk_spin_button_size_request (GtkWidget      *widget,
 			      GtkRequisition *requisition)
@@ -491,7 +552,6 @@ gtk_spin_button_size_request (GtkWidget      *widget,
   GtkEntry *entry;
   GtkSpinButton *spin_button;
   
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (requisition != NULL);
   g_return_if_fail (GTK_IS_SPIN_BUTTON (widget));
 
@@ -502,30 +562,34 @@ gtk_spin_button_size_request (GtkWidget      *widget,
 
   if (entry->width_chars < 0)
     {
-      PangoFontMetrics metrics;
-      PangoFont *font;
-      gchar *lang;
-      gchar buf[MAX_TEXT_LENGTH];
+      PangoContext *context;
+      PangoFontMetrics *metrics;
       gint width;
       gint w;
+      gint string_len;
+      gint digit_width;
 
+      context = gtk_widget_get_pango_context (widget);
+      metrics = pango_context_get_metrics (context,
+					   widget->style->font_desc,
+					   pango_context_get_language (context));
 
-      font = pango_context_load_font (gtk_widget_get_pango_context (widget),
-                                      widget->style->font_desc);
-      lang = pango_context_get_lang (gtk_widget_get_pango_context (widget));
-      pango_font_get_metrics (font, lang, &metrics);
-      g_free (lang);
-      g_object_unref (G_OBJECT (font));
+      digit_width = pango_font_metrics_get_approximate_digit_width (metrics);
+      digit_width = PANGO_PIXELS (digit_width);
+
+      pango_font_metrics_unref (metrics);
       
       /* Get max of MIN_SPIN_BUTTON_WIDTH, size of upper, size of lower */
       
       width = MIN_SPIN_BUTTON_WIDTH;
-      
-      sprintf (buf, "%0.*f", spin_button->digits, spin_button->adjustment->upper);
-      w = strlen (buf) * PANGO_PIXELS (metrics.approximate_digit_width);
+
+      string_len = compute_double_length (spin_button->adjustment->upper,
+                                          spin_button->digits);
+      w = MIN (string_len, 10) * digit_width;
       width = MAX (width, w);
-      sprintf (buf, "%0.*f", spin_button->digits, spin_button->adjustment->lower);
-      w = strlen (buf) * PANGO_PIXELS (metrics.approximate_digit_width);
+      string_len = compute_double_length (spin_button->adjustment->lower,
+					  spin_button->digits);
+      w = MIN (string_len, 10) * digit_width;
       width = MAX (width, w);
       
       requisition->width = width + ARROW_SIZE + 2 * widget->style->xthickness;
@@ -540,7 +604,6 @@ gtk_spin_button_size_allocate (GtkWidget     *widget,
 {
   GtkAllocation child_allocation;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_SPIN_BUTTON (widget));
   g_return_if_fail (allocation != NULL);
 
@@ -582,7 +645,6 @@ gtk_spin_button_expose (GtkWidget      *widget,
 {
   GtkSpinButton *spin;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -632,7 +694,6 @@ gtk_spin_button_draw_arrow (GtkSpinButton *spin_button,
   gint x;
   gint y;
 
-  g_return_if_fail (spin_button != NULL);
   g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
   
   widget = GTK_WIDGET (spin_button);
@@ -715,7 +776,6 @@ gtk_spin_button_enter_notify (GtkWidget        *widget,
 {
   GtkSpinButton *spin;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -750,7 +810,6 @@ gtk_spin_button_leave_notify (GtkWidget        *widget,
 {
   GtkSpinButton *spin;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -776,7 +835,6 @@ static gint
 gtk_spin_button_focus_out (GtkWidget     *widget,
 			   GdkEventFocus *event)
 {
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -792,7 +850,6 @@ gtk_spin_button_scroll (GtkWidget      *widget,
 {
   GtkSpinButton *spin;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -822,7 +879,6 @@ gtk_spin_button_button_press (GtkWidget      *widget,
 {
   GtkSpinButton *spin;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -916,7 +972,6 @@ gtk_spin_button_button_release (GtkWidget      *widget,
 {
   GtkSpinButton *spin;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -979,7 +1034,6 @@ gtk_spin_button_motion_notify (GtkWidget      *widget,
 {
   GtkSpinButton *spin;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -1065,7 +1119,6 @@ gtk_spin_button_value_changed (GtkAdjustment *adjustment,
 {
   gint return_val;
 
-  g_return_if_fail (adjustment != NULL);
   g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
 
   return_val = FALSE;
@@ -1079,6 +1132,8 @@ gtk_spin_button_value_changed (GtkAdjustment *adjustment,
 
   gtk_spin_button_draw_arrow (spin_button, GTK_ARROW_UP);
   gtk_spin_button_draw_arrow (spin_button, GTK_ARROW_DOWN);
+  
+  g_object_notify (G_OBJECT (spin_button), "value");
 }
 
 static gint
@@ -1088,8 +1143,8 @@ gtk_spin_button_key_press (GtkWidget     *widget,
   GtkSpinButton *spin;
   gint key;
   gboolean key_repeat = FALSE;
-
-  g_return_val_if_fail (widget != NULL, FALSE);
+  gboolean retval = FALSE;
+  
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
   
@@ -1098,100 +1153,112 @@ gtk_spin_button_key_press (GtkWidget     *widget,
 
   key_repeat = (event->time == spin->ev_time);
 
-  if (GTK_ENTRY (widget)->editable &&
-      (key == GDK_Up || key == GDK_Down || 
-       key == GDK_Page_Up || key == GDK_Page_Down))
-    gtk_spin_button_update (spin);
-
-  switch (key)
+  if (GTK_ENTRY (widget)->editable)
     {
-    case GDK_Up:
+      switch (key)
+        {
+        case GDK_KP_Up:
+        case GDK_Up:
 
-      if (GTK_WIDGET_HAS_FOCUS (widget))
-	{
-	  gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), 
-					"key_press_event");
-	  if (!key_repeat)
-	    spin->timer_step = spin->adjustment->step_increment;
+          if (GTK_WIDGET_HAS_FOCUS (widget))
+            {
+              gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), 
+                                            "key_press_event");
+              if (!key_repeat)
+                spin->timer_step = spin->adjustment->step_increment;
 
-	 gtk_spin_button_real_spin (spin, spin->timer_step);
+              gtk_spin_button_real_spin (spin, spin->timer_step);
 
-	  if (key_repeat)
-	    {
-	      if (spin->climb_rate > 0.0 && spin->timer_step
-		  < spin->adjustment->page_increment)
-		{
-		  if (spin->timer_calls < MAX_TIMER_CALLS)
-		    spin->timer_calls++;
-		  else 
-		    {
-		      spin->timer_calls = 0;
-		      spin->timer_step += spin->climb_rate;
-		    }
-		}
-	    }
-	  return TRUE;
-	}
-      return FALSE;
+              if (key_repeat)
+                {
+                  if (spin->climb_rate > 0.0 && spin->timer_step
+                      < spin->adjustment->page_increment)
+                    {
+                      if (spin->timer_calls < MAX_TIMER_CALLS)
+                        spin->timer_calls++;
+                      else 
+                        {
+                          spin->timer_calls = 0;
+                          spin->timer_step += spin->climb_rate;
+                        }
+                    }
+                }
+              retval = TRUE;
+            }
+          break;
 
-    case GDK_Down:
+        case GDK_KP_Down:
+        case GDK_Down:
 
-      if (GTK_WIDGET_HAS_FOCUS (widget))
-	{
-	  gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), 
-					"key_press_event");
-	  if (!key_repeat)
-	    spin->timer_step = spin->adjustment->step_increment;
+          if (GTK_WIDGET_HAS_FOCUS (widget))
+            {
+              gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), 
+                                            "key_press_event");
+              if (!key_repeat)
+                spin->timer_step = spin->adjustment->step_increment;
 
-	 gtk_spin_button_real_spin (spin, -spin->timer_step);
+              gtk_spin_button_real_spin (spin, -spin->timer_step);
 
-	  if (key_repeat)
-	    {
-	      if (spin->climb_rate > 0.0 && spin->timer_step
-		  < spin->adjustment->page_increment)
-		{
-		  if (spin->timer_calls < MAX_TIMER_CALLS)
-		    spin->timer_calls++;
-		  else 
-		    {
-		      spin->timer_calls = 0;
-		      spin->timer_step += spin->climb_rate;
-		    }
-		}
-	    }
-	  return TRUE;
-	}
-      return FALSE;
+              if (key_repeat)
+                {
+                  if (spin->climb_rate > 0.0 && spin->timer_step
+                      < spin->adjustment->page_increment)
+                    {
+                      if (spin->timer_calls < MAX_TIMER_CALLS)
+                        spin->timer_calls++;
+                      else 
+                        {
+                          spin->timer_calls = 0;
+                          spin->timer_step += spin->climb_rate;
+                        }
+                    }
+                }
+              retval = TRUE;
+            }
+          break;
 
-    case GDK_Page_Up:
+        case GDK_KP_Page_Up:
+        case GDK_Page_Up:
 
-      if (event->state & GDK_CONTROL_MASK)
-	{
-	  gdouble diff = spin->adjustment->upper - spin->adjustment->value;
-	  if (diff > EPSILON)
-	    gtk_spin_button_real_spin (spin, diff);
-	}
-      else
-	gtk_spin_button_real_spin (spin, spin->adjustment->page_increment);
-      return TRUE;
+          if (event->state & GDK_CONTROL_MASK)
+            {
+              gdouble diff = spin->adjustment->upper - spin->adjustment->value;
+              if (diff > EPSILON)
+                gtk_spin_button_real_spin (spin, diff);
+            }
+          else
+            gtk_spin_button_real_spin (spin, spin->adjustment->page_increment);
 
-    case GDK_Page_Down:
+          retval = TRUE;
+          break;
+          
+        case GDK_KP_Page_Down:
+        case GDK_Page_Down:
 
-      if (event->state & GDK_CONTROL_MASK)
-	{
-	  gdouble diff = spin->adjustment->value - spin->adjustment->lower;
-	  if (diff > EPSILON)
-	    gtk_spin_button_real_spin (spin, -diff);
-	}
-      else
-	gtk_spin_button_real_spin (spin, -spin->adjustment->page_increment);
-      return TRUE;
+          if (event->state & GDK_CONTROL_MASK)
+            {
+              gdouble diff = spin->adjustment->value - spin->adjustment->lower;
+              if (diff > EPSILON)
+                gtk_spin_button_real_spin (spin, -diff);
+            }
+          else
+            gtk_spin_button_real_spin (spin, -spin->adjustment->page_increment);
 
-    default:
-      break;
+          retval = TRUE;
+          break;
+
+        default:
+          break;
+        }
     }
 
-  return GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
+  if (retval)
+    {
+      gtk_spin_button_update (spin);
+      return TRUE;
+    }
+  else
+    return GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
 }
 
 static gint
@@ -1200,7 +1267,6 @@ gtk_spin_button_key_release (GtkWidget   *widget,
 {
   GtkSpinButton *spin;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   
   spin = GTK_SPIN_BUTTON (widget);
@@ -1330,7 +1396,6 @@ gtk_spin_button_real_spin (GtkSpinButton *spin_button,
   GtkAdjustment *adj;
   gdouble new_value = 0.0;
 
-  g_return_if_fail (spin_button != NULL);
   g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
   
   adj = spin_button->adjustment;
@@ -1382,11 +1447,11 @@ gtk_spin_button_default_input (GtkSpinButton *spin_button,
 static gint
 gtk_spin_button_default_output (GtkSpinButton *spin_button)
 {
-  gchar buf[MAX_TEXT_LENGTH];
+  gchar *buf = g_strdup_printf ("%0.*f", spin_button->digits, spin_button->adjustment->value);
 
-  sprintf (buf, "%0.*f", spin_button->digits, spin_button->adjustment->value);
   if (strcmp (buf, gtk_entry_get_text (GTK_ENTRY (spin_button))))
     gtk_entry_set_text (GTK_ENTRY (spin_button), buf);
+  g_free (buf);
   return FALSE;
 }
 
@@ -1404,17 +1469,27 @@ gtk_spin_button_configure (GtkSpinButton  *spin_button,
 			   gdouble         climb_rate,
 			   guint           digits)
 {
-  g_return_if_fail (spin_button != NULL);
   g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
-  g_return_if_fail (digits < 6);
 
   if (adjustment)
     gtk_spin_button_set_adjustment (spin_button, adjustment);
   else
     adjustment = spin_button->adjustment;
 
-  spin_button->digits = digits;
-  spin_button->climb_rate = climb_rate;
+  g_object_freeze_notify (G_OBJECT (spin_button));
+  if (spin_button->digits != digits) 
+    {
+      spin_button->digits = digits;
+      g_object_notify (G_OBJECT (spin_button), "digits");
+    }
+
+  if (spin_button->climb_rate != climb_rate)
+    {
+      spin_button->climb_rate = climb_rate;
+      g_object_notify (G_OBJECT (spin_button), "climb_rate");
+    }
+  g_object_thaw_notify (G_OBJECT (spin_button));
+
   gtk_adjustment_value_changed (adjustment);
 }
 
@@ -1427,7 +1502,6 @@ gtk_spin_button_new (GtkAdjustment *adjustment,
 
   if (adjustment)
     g_return_val_if_fail (GTK_IS_ADJUSTMENT (adjustment), NULL);
-  g_return_val_if_fail (digits < 6, NULL);
 
   spin = gtk_type_new (GTK_TYPE_SPIN_BUTTON);
 
@@ -1466,12 +1540,12 @@ gtk_spin_button_new_with_range (gdouble min,
 
   adj = gtk_adjustment_new (min, min, max, step, 10 * step, step);
 
-  if (fabs (step) >= 1.0)
+  if (fabs (step) >= 1.0 || step == 0.0)
     digits = 0;
   else {
     digits = abs ((gint) floor (log10 (fabs (step))));
-    if (digits > 5)
-      digits = 5;
+    if (digits > MAX_DIGITS)
+      digits = MAX_DIGITS;
   }
 
   gtk_spin_button_configure (spin, GTK_ADJUSTMENT (adj), step, digits);
@@ -1530,6 +1604,8 @@ gtk_spin_button_set_adjustment (GtkSpinButton *spin_button,
 
       gtk_widget_queue_resize (GTK_WIDGET (spin_button));
     }
+
+  g_object_notify (G_OBJECT (spin_button), "adjustment");
 }
 
 /**
@@ -1553,7 +1629,7 @@ gtk_spin_button_get_adjustment (GtkSpinButton *spin_button)
  * @spin_button: a #GtkSpinButton
  * @digits: the number of digits to be displayed for the spin button's value
  * 
- * Set the precision to be displayed by @spin_button. Up to 5 digit precision
+ * Set the precision to be displayed by @spin_button. Up to 20 digit precision
  * is allowed.
  **/
 void
@@ -1561,17 +1637,34 @@ gtk_spin_button_set_digits (GtkSpinButton *spin_button,
 			    guint          digits)
 {
   g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
-  g_return_if_fail (digits < 6);
 
   if (spin_button->digits != digits)
     {
       spin_button->digits = digits;
       gtk_spin_button_value_changed (spin_button->adjustment, spin_button);
+      g_object_notify (G_OBJECT (spin_button), "digits");
+      
       /* since lower/upper may have changed */
       gtk_widget_queue_resize (GTK_WIDGET (spin_button));
     }
 }
 
+/**
+ * gtk_spin_button_get_digits:
+ * @spin_button: a #GtkSpinButton
+ *
+ * Fetches the precision of @spin_button. See gtk_spin_button_set_digits().
+ *
+ * Returns: the current precision
+ **/
+guint
+gtk_spin_button_get_digits (GtkSpinButton *spin_button)
+{
+  g_return_val_if_fail (GTK_IS_SPIN_BUTTON (spin_button), 0);
+
+  return spin_button->digits;
+}
+
 /**
  * gtk_spin_button_set_increments:
  * @spin_button: a #GtkSpinButton
@@ -1592,6 +1685,28 @@ gtk_spin_button_set_increments (GtkSpinButton *spin_button,
   spin_button->adjustment->page_increment = page;
 }
 
+/**
+ * gtk_spin_button_get_increments:
+ * @spin_button: a #GtkSpinButton
+ * @step: location to store step increment, or %NULL
+ * @page: location to store page increment, or %NULL
+ *
+ * Gets the current step and page the increments used by @spin_button. See
+ * gtk_spin_button_set_increments().
+ **/
+void
+gtk_spin_button_get_increments (GtkSpinButton *spin_button,
+				gdouble       *step,
+				gdouble       *page)
+{
+  g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
+
+  if (step)
+    *step = spin_button->adjustment->step_increment;
+  if (page)
+    *page = spin_button->adjustment->page_increment;
+}
+
 /**
  * gtk_spin_button_set_range:
  * @spin_button: a #GtkSpinButton
@@ -1605,22 +1720,55 @@ gtk_spin_button_set_range (GtkSpinButton *spin_button,
 			   gdouble        min,
 			   gdouble        max)
 {
+  gdouble value;
+  
   g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
 
   spin_button->adjustment->lower = min;
   spin_button->adjustment->upper = max;
+
+  value = CLAMP (spin_button->adjustment->value,
+                 spin_button->adjustment->lower,
+                 (spin_button->adjustment->upper - spin_button->adjustment->page_size));
+
+  if (value != spin_button->adjustment->value)
+    gtk_spin_button_set_value (spin_button, value);
+
+  gtk_adjustment_changed (spin_button->adjustment);
 }
 
 /**
- * gtk_spin_button_get_value_as_float:
+ * gtk_spin_button_get_range:
+ * @spin_button: a #GtkSpinButton
+ * @min: location to store minimum allowed value, or %NULL
+ * @max: location to store maximum allowed value, or %NULL
+ *
+ * Gets the range allowed for @spin_button. See
+ * gtk_spin_button_set_range().
+ **/
+void
+gtk_spin_button_get_range (GtkSpinButton *spin_button,
+			   gdouble       *min,
+			   gdouble       *max)
+{
+  g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
+
+  if (min)
+    *min = spin_button->adjustment->lower;
+  if (max)
+    *max = spin_button->adjustment->upper;
+}
+
+/**
+ * gtk_spin_button_get_value:
  * @spin_button: a #GtkSpinButton
  * 
- * Get the value @spin_button represented as a floating point number.
+ * Get the value in the @spin_button.
  * 
  * Return value: the value of @spin_button
  **/
 gdouble
-gtk_spin_button_get_value_as_float (GtkSpinButton *spin_button)
+gtk_spin_button_get_value (GtkSpinButton *spin_button)
 {
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (spin_button), 0.0);
 
@@ -1688,7 +1836,28 @@ gtk_spin_button_set_update_policy (GtkSpinButton             *spin_button,
 {
   g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
 
-  spin_button->update_policy = policy;
+  if (spin_button->update_policy != policy)
+    {
+      spin_button->update_policy = policy;
+      g_object_notify (G_OBJECT (spin_button), "update_policy");
+    }
+}
+
+/**
+ * gtk_spin_button_get_update_policy:
+ * @spin_button: a #GtkSpinButton
+ *
+ * Gets the update behavior of a spin button. See
+ * gtk_spin_button_set_update_policy().
+ *
+ * Return value: the current update policy
+ **/
+GtkSpinButtonUpdatePolicy
+gtk_spin_button_get_update_policy (GtkSpinButton *spin_button)
+{
+  g_return_val_if_fail (GTK_IS_SPIN_BUTTON (spin_button), GTK_UPDATE_ALWAYS);
+
+  return spin_button->update_policy;
 }
 
 /**
@@ -1706,6 +1875,25 @@ gtk_spin_button_set_numeric (GtkSpinButton  *spin_button,
   g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
 
   spin_button->numeric = (numeric != 0);
+
+  g_object_notify (G_OBJECT (spin_button), "numeric");
+}
+
+/**
+ * gtk_spin_button_get_numeric:
+ * @spin_button: a #GtkSpinButton
+ *
+ * Returns whether non-numeric text can be typed into the spin button.
+ * See gtk_spin_button_set_numeric().
+ *
+ * Return value: %TRUE if only numeric text can be entered
+ **/
+gboolean
+gtk_spin_button_get_numeric (GtkSpinButton *spin_button)
+{
+  g_return_val_if_fail (GTK_IS_SPIN_BUTTON (spin_button), FALSE);
+
+  return spin_button->numeric;
 }
 
 /**
@@ -1723,6 +1911,26 @@ gtk_spin_button_set_wrap (GtkSpinButton  *spin_button,
   g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
 
   spin_button->wrap = (wrap != 0);
+  
+  g_object_notify (G_OBJECT (spin_button), "wrap");
+}
+
+/**
+ * gtk_spin_button_get_wrap:
+ * @spin_button: a #GtkSpinButton
+ *
+ * Returns whether the spin button's value wraps around to the
+ * opposite limit when the upper or lower limit of the range is
+ * exceeded. See gtk_spin_button_set_wrap().
+ *
+ * Return value: %TRUE if the spin button wraps around
+ **/
+gboolean
+gtk_spin_button_get_wrap (GtkSpinButton *spin_button)
+{
+  g_return_val_if_fail (GTK_IS_SPIN_BUTTON (spin_button), FALSE);
+
+  return spin_button->wrap;
 }
 
 /**
@@ -1767,9 +1975,28 @@ gtk_spin_button_set_snap_to_ticks (GtkSpinButton *spin_button,
       spin_button->snap_to_ticks = new_val;
       if (new_val && GTK_ENTRY (spin_button)->editable)
 	gtk_spin_button_update (spin_button);
+      
+      g_object_notify (G_OBJECT (spin_button), "snap_to_ticks");
     }
 }
 
+/**
+ * gtk_spin_button_get_snap_to_ticks:
+ * @spin_button: a #GtkSpinButton
+ *
+ * Returns whether the values are corrected to the nearest step. See
+ * gtk_spin_button_set_snap_to_ticks().
+ *
+ * Return value: %TRUE if values are snapped to the nearest step.
+ **/
+gboolean
+gtk_spin_button_get_snap_to_ticks (GtkSpinButton *spin_button)
+{
+  g_return_val_if_fail (GTK_IS_SPIN_BUTTON (spin_button), FALSE);
+
+  return spin_button->snap_to_ticks;
+}
+
 /**
  * gtk_spin_button_spin:
  * @spin_button: a #GtkSpinButton 
