@@ -868,6 +868,11 @@ void gtk_paint_string     (GtkStyle        *style,
 			   const gchar     *string);
 #endif /* GTK_DISABLE_DEPRECATED */
 
+void _gtk_draw_insertion_cursor (GdkDrawable      *drawable,
+				 GdkGC            *gc,
+				 GdkRectangle     *location,
+				 GtkTextDirection  dir);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
