@@ -97,13 +97,13 @@ gtk_text_line_segment_split (const GtkTextIter *iter)
   GtkTextLine *line;
   int count;
 
-  line = gtk_text_iter_get_text_line (iter);
-  tree = gtk_text_iter_get_btree (iter);
+  line = _gtk_text_iter_get_text_line (iter);
+  tree = _gtk_text_iter_get_btree (iter);
 
   count = gtk_text_iter_get_line_index (iter);
 
   if (gtk_debug_flags & GTK_DEBUG_TEXT)
-    gtk_text_iter_check (iter);
+    _gtk_text_iter_check (iter);
   
   prev = NULL;
   seg = line->segments;
@@ -121,7 +121,7 @@ gtk_text_line_segment_split (const GtkTextIter *iter)
               g_assert (count != seg->byte_count);
               g_assert (seg->byte_count > 0);
 
-              gtk_text_btree_segments_changed (tree);
+              _gtk_text_btree_segments_changed (tree);
 
               seg = (*seg->type->splitFunc)(seg, count);
 
@@ -392,14 +392,7 @@ char_segment_check_func (segPtr, line)
 {
   char_segment_self_check (segPtr);
 
-  if (segPtr->next == NULL)
-    {
-      if (segPtr->body.chars[segPtr->byte_count-1] != '\n')
-        {
-          g_error ("char_segment_check_func: line doesn't end with newline");
-        }
-    }
-  else
+  if (segPtr->next != NULL)
     {
       if (segPtr->next->type == &gtk_text_char_type)
         {
