@@ -749,6 +749,46 @@ get_property_font_set_mask (guint prop_id)
   return 0;
 }
 
+static PangoFontMask
+set_font_desc_fields (PangoFontDescription *desc,
+		      PangoFontMask         to_set)
+{
+  PangoFontMask changed_mask = 0;
+  
+  if (to_set & PANGO_FONT_MASK_FAMILY)
+    {
+      const char *family = pango_font_description_get_family (desc);
+      if (!family)
+	{
+	  family = "sans";
+	  changed_mask |= PANGO_FONT_MASK_FAMILY;
+	}
+
+      pango_font_description_set_family (desc, family);
+    }
+  if (to_set & PANGO_FONT_MASK_STYLE)
+    pango_font_description_set_style (desc, pango_font_description_get_style (desc));
+  if (to_set & PANGO_FONT_MASK_VARIANT)
+    pango_font_description_set_variant (desc, pango_font_description_get_variant (desc));
+  if (to_set & PANGO_FONT_MASK_WEIGHT)
+    pango_font_description_set_weight (desc, pango_font_description_get_weight (desc));
+  if (to_set & PANGO_FONT_MASK_STRETCH)
+    pango_font_description_set_stretch (desc, pango_font_description_get_stretch (desc));
+  if (to_set & PANGO_FONT_MASK_SIZE)
+    {
+      gint size = pango_font_description_get_size (desc);
+      if (size <= 0)
+	{
+	  size = 10 * PANGO_SCALE;
+	  changed_mask |= PANGO_FONT_MASK_SIZE;
+	}
+      
+      pango_font_description_set_size (desc, size);
+    }
+
+  return changed_mask;
+}
+
 static void
 notify_set_changed (GObject       *object,
 		    PangoFontMask  changed_mask)
@@ -767,6 +807,24 @@ notify_set_changed (GObject       *object,
     g_object_notify (object, "size_set");
 }
 
+static void
+notify_fields_changed (GObject       *object,
+		       PangoFontMask  changed_mask)
+{
+  if (changed_mask & PANGO_FONT_MASK_FAMILY)
+    g_object_notify (object, "family");
+  if (changed_mask & PANGO_FONT_MASK_STYLE)
+    g_object_notify (object, "style");
+  if (changed_mask & PANGO_FONT_MASK_VARIANT)
+    g_object_notify (object, "variant");
+  if (changed_mask & PANGO_FONT_MASK_WEIGHT)
+    g_object_notify (object, "weight");
+  if (changed_mask & PANGO_FONT_MASK_STRETCH)
+    g_object_notify (object, "stretch");
+  if (changed_mask & PANGO_FONT_MASK_SIZE)
+    g_object_notify (object, "size");
+}
+
 static void
 set_font_description (GtkTextTag           *text_tag,
                       PangoFontDescription *font_desc)
@@ -986,7 +1044,7 @@ gtk_text_tag_set_property (GObject      *object,
 	    break;
 	  case PROP_WEIGHT:
 	    pango_font_description_set_weight (text_tag->values->font,
-					       g_value_get_enum (value));
+					       g_value_get_int (value));
 	    break;
 	  case PROP_STRETCH:
 	    pango_font_description_set_stretch (text_tag->values->font,
@@ -1174,9 +1232,21 @@ gtk_text_tag_set_property (GObject      *object,
     case PROP_WEIGHT_SET:
     case PROP_STRETCH_SET:
     case PROP_SIZE_SET:
-      if (!g_value_get_boolean (value) && text_tag->values->font)
-	pango_font_description_unset_fields (text_tag->values->font,
-					     get_property_font_set_mask (prop_id));
+      if (!g_value_get_boolean (value))
+	{
+	  if (text_tag->values->font)
+	    pango_font_description_unset_fields (text_tag->values->font,
+						 get_property_font_set_mask (prop_id));
+	}
+      else
+	{
+	  PangoFontMask changed_mask;
+	  
+	  gtk_text_tag_ensure_font (text_tag);
+	  changed_mask = set_font_desc_fields (text_tag->values->font,
+					       get_property_font_set_mask (prop_id));
+	  notify_fields_changed (G_OBJECT (text_tag), changed_mask);
+	}
       break;
 
     case PROP_SCALE_SET:
