@@ -15,36 +15,45 @@ enum {
   LAST_ARG
 };
 
-static void gtk_text_tag_table_init (GtkTextTagTable *table);
-static void gtk_text_tag_table_class_init (GtkTextTagTableClass *klass);
-static void gtk_text_tag_table_destroy (GtkObject *object);
-static void gtk_text_tag_table_finalize (GObject *object);
-static void gtk_text_tag_table_set_arg (GtkObject *object, GtkArg *arg, guint arg_id);
-static void gtk_text_tag_table_get_arg (GtkObject *object, GtkArg *arg, guint arg_id);
+static void gtk_text_tag_table_init         (GtkTextTagTable      *table);
+static void gtk_text_tag_table_class_init   (GtkTextTagTableClass *klass);
+static void gtk_text_tag_table_finalize     (GObject              *object);
+static void gtk_text_tag_table_set_property (GObject              *object,
+                                             guint                 prop_id,
+                                             const GValue         *value,
+                                             GParamSpec           *pspec);
+static void gtk_text_tag_table_get_property (GObject              *object,
+                                             guint                 prop_id,
+                                             GValue               *value,
+                                             GParamSpec           *pspec);
 
-static GtkObjectClass *parent_class = NULL;
+static GObjectClass *parent_class = NULL;
 static guint signals[LAST_SIGNAL] = { 0 };
 
-GtkType
+GType
 gtk_text_tag_table_get_type (void)
 {
-  static GtkType our_type = 0;
+  static GType our_type = 0;
 
   if (our_type == 0)
     {
-      static const GtkTypeInfo our_info =
+      static const GTypeInfo our_info =
       {
-        "GtkTextTagTable",
-        sizeof (GtkTextTagTable),
         sizeof (GtkTextTagTableClass),
-        (GtkClassInitFunc) gtk_text_tag_table_class_init,
-        (GtkObjectInitFunc) gtk_text_tag_table_init,
-        /* reserved_1 */ NULL,
-        /* reserved_2 */ NULL,
-        (GtkClassInitFunc) NULL
+        (GBaseInitFunc) NULL,
+        (GBaseFinalizeFunc) NULL,
+        (GClassInitFunc) gtk_text_tag_table_class_init,
+        NULL,           /* class_finalize */
+        NULL,           /* class_data */
+        sizeof (GtkTextTagTable),
+        0,              /* n_preallocs */
+        (GInstanceInitFunc) gtk_text_tag_table_init
       };
 
-      our_type = gtk_type_unique (GTK_TYPE_OBJECT, &our_info);
+      our_type = g_type_register_static (G_TYPE_OBJECT,
+                                         "GtkTextTagTable",
+                                         &our_info,
+                                         0);
     }
 
   return our_type;
@@ -53,50 +62,48 @@ gtk_text_tag_table_get_type (void)
 static void
 gtk_text_tag_table_class_init (GtkTextTagTableClass *klass)
 {
-  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-  GtkObjectClass *object_class = GTK_OBJECT_CLASS (klass);
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-  parent_class = gtk_type_class (GTK_TYPE_OBJECT);
+  parent_class = g_type_class_peek_parent (klass);
 
+  object_class->set_property = gtk_text_tag_table_set_property;
+  object_class->get_property = gtk_text_tag_table_get_property;
+  
+  object_class->finalize = gtk_text_tag_table_finalize;
+  
   signals[TAG_CHANGED] =
-    gtk_signal_new ("tag_changed",
-                    GTK_RUN_LAST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkTextTagTableClass, tag_changed),
-                    gtk_marshal_VOID__OBJECT_BOOLEAN,
-                    GTK_TYPE_NONE,
-                    2,
-                    G_TYPE_OBJECT,
-                    GTK_TYPE_BOOL);
+    g_signal_new ("tag_changed",
+                  G_OBJECT_CLASS_TYPE (object_class),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (GtkTextTagTableClass, tag_changed),
+                  NULL, NULL,
+                  gtk_marshal_VOID__OBJECT_BOOLEAN,
+                  G_TYPE_NONE,
+                  2,
+                  G_TYPE_OBJECT,
+                  G_TYPE_BOOLEAN);  
 
   signals[TAG_ADDED] =
-    gtk_signal_new ("tag_added",
-                    GTK_RUN_LAST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkTextTagTableClass, tag_added),
-                    gtk_marshal_VOID__OBJECT,
-                    GTK_TYPE_NONE,
-                    1,
-                    G_TYPE_OBJECT);
+    g_signal_new ("tag_added",
+                  GTK_CLASS_TYPE (object_class),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (GtkTextTagTableClass, tag_added),
+                  NULL, NULL,
+                  gtk_marshal_VOID__OBJECT,
+                  GTK_TYPE_NONE,
+                  1,
+                  G_TYPE_OBJECT);
 
   signals[TAG_REMOVED] =
-    gtk_signal_new ("tag_removed",
-                    GTK_RUN_LAST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkTextTagTableClass, tag_removed),
-                    gtk_marshal_VOID__OBJECT,
-                    GTK_TYPE_NONE,
-                    1,
-                    G_TYPE_OBJECT);
-
-
-  gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
-
-  object_class->set_arg = gtk_text_tag_table_set_arg;
-  object_class->get_arg = gtk_text_tag_table_get_arg;
-
-  object_class->destroy = gtk_text_tag_table_destroy;
-  gobject_class->finalize = gtk_text_tag_table_finalize;
+    g_signal_new ("tag_removed",                   
+                  GTK_CLASS_TYPE (object_class),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (GtkTextTagTableClass, tag_removed),
+                  NULL, NULL,
+                  gtk_marshal_VOID__OBJECT,
+                  GTK_TYPE_NONE,
+                  1,
+                  G_TYPE_OBJECT);
 }
 
 void
@@ -118,21 +125,11 @@ gtk_text_tag_table_new (void)
 {
   GtkTextTagTable *table;
 
-  table = GTK_TEXT_TAG_TABLE (gtk_type_new (gtk_text_tag_table_get_type ()));
+  table = GTK_TEXT_TAG_TABLE (g_object_new (gtk_text_tag_table_get_type (), NULL));
 
   return table;
 }
 
-static void
-gtk_text_tag_table_destroy (GtkObject *object)
-{
-  GtkTextTagTable *table;
-
-  table = GTK_TEXT_TAG_TABLE (object);
-
-  (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
-}
-
 static void
 foreach_unref (GtkTextTag *tag, gpointer data)
 {
@@ -152,41 +149,47 @@ gtk_text_tag_table_finalize (GObject *object)
   table = GTK_TEXT_TAG_TABLE (object);
 
   gtk_text_tag_table_foreach (table, foreach_unref, NULL);
-
+  
   g_hash_table_destroy (table->hash);
   g_slist_free (table->anonymous);
 
   (* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
-
 static void
-gtk_text_tag_table_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+gtk_text_tag_table_set_property (GObject      *object,
+                                 guint         prop_id,
+                                 const GValue *value,
+                                 GParamSpec   *pspec)
 {
   GtkTextTagTable *table;
 
   table = GTK_TEXT_TAG_TABLE (object);
 
-  switch (arg_id)
+  switch (prop_id)
     {
 
     default:
-      g_assert_not_reached ();
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
     }
 }
 
+
 static void
-gtk_text_tag_table_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+gtk_text_tag_table_get_property (GObject      *object,
+                                 guint         prop_id,
+                                 GValue       *value,
+                                 GParamSpec   *pspec)
 {
   GtkTextTagTable *table;
 
   table = GTK_TEXT_TAG_TABLE (object);
 
-  switch (arg_id)
+  switch (prop_id)
     {
 
     default:
-      arg->type = GTK_TYPE_INVALID;
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
     }
 }
@@ -198,7 +201,9 @@ gtk_text_tag_table_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
  *
  * Add a tag to the table. The tag is assigned the highest priority
  * in the table.
- * 
+ *
+ * @tag must not be in a tag table already, and may not have
+ * the same name as an already-added tag.
  **/
 void
 gtk_text_tag_table_add (GtkTextTagTable *table,
@@ -208,12 +213,16 @@ gtk_text_tag_table_add (GtkTextTagTable *table,
 
   g_return_if_fail (GTK_IS_TEXT_TAG_TABLE (table));
   g_return_if_fail (GTK_IS_TEXT_TAG (tag));
-  g_return_if_fail (tag->name == NULL ||
-                    g_hash_table_lookup (table->hash, tag->name) == NULL);
   g_return_if_fail (tag->table == NULL);
 
-  gtk_object_ref (GTK_OBJECT (tag));
-  gtk_object_sink (GTK_OBJECT (tag));
+  if (tag->name && g_hash_table_lookup (table->hash, tag->name))
+    {
+      g_warning ("A tag named '%s' is already in the tag table.",
+                 tag->name);
+      return;
+    }
+  
+  g_object_ref (G_OBJECT (tag));
 
   if (tag->name)
     g_hash_table_insert (table->hash, tag->name, tag);
@@ -228,11 +237,11 @@ gtk_text_tag_table_add (GtkTextTagTable *table,
   /* We get the highest tag priority, as the most-recently-added
      tag. Note that we do NOT use gtk_text_tag_set_priority,
      as it assumes the tag is already in the table. */
-  size = gtk_text_tag_table_size (table);
+  size = gtk_text_tag_table_get_size (table);
   g_assert (size > 0);
   tag->priority = size - 1;
 
-  gtk_signal_emit (GTK_OBJECT (table), signals[TAG_ADDED], tag);
+  g_signal_emit (G_OBJECT (table), signals[TAG_ADDED], 0, tag);
 }
 
 /**
@@ -274,7 +283,7 @@ gtk_text_tag_table_remove (GtkTextTagTable *table,
   /* Set ourselves to the highest priority; this means
      when we're removed, there won't be any gaps in the
      priorities of the tags in the table. */
-  gtk_text_tag_set_priority (tag, gtk_text_tag_table_size (table) - 1);
+  gtk_text_tag_set_priority (tag, gtk_text_tag_table_get_size (table) - 1);
 
   tag->table = NULL;
 
@@ -286,9 +295,9 @@ gtk_text_tag_table_remove (GtkTextTagTable *table,
       table->anon_count -= 1;
     }
 
-  gtk_signal_emit (GTK_OBJECT (table), signals[TAG_REMOVED], tag);
+  g_signal_emit (G_OBJECT (table), signals[TAG_REMOVED], 0, tag);
 
-  gtk_object_unref (GTK_OBJECT (tag));
+  g_object_unref (G_OBJECT (tag));
 }
 
 struct ForeachData
@@ -344,7 +353,7 @@ gtk_text_tag_table_foreach (GtkTextTagTable       *table,
 }
 
 /**
- * gtk_text_tag_table_size:
+ * gtk_text_tag_table_get_size:
  * @table: a #GtkTextTagTable
  * 
  * Returns the size of the table (number of tags)
@@ -352,7 +361,7 @@ gtk_text_tag_table_foreach (GtkTextTagTable       *table,
  * Return value: number of tags in @table
  **/
 gint
-gtk_text_tag_table_size (GtkTextTagTable *table)
+gtk_text_tag_table_get_size (GtkTextTagTable *table)
 {
   g_return_val_if_fail (GTK_IS_TEXT_TAG_TABLE (table), 0);
 
