@@ -142,6 +142,7 @@ struct _GtkTextView
   GSList *children;
 
   GtkTextPendingScroll *pending_scroll;
+  GdkGC *cursor_gc;
 };
 
 struct _GtkTextViewClass
@@ -248,9 +249,8 @@ GtkTextWindowType gtk_text_view_get_window_type (GtkTextView       *text_view,
 void gtk_text_view_set_border_window_size (GtkTextView       *text_view,
                                            GtkTextWindowType  type,
                                            gint               size);
-void gtk_text_view_set_text_window_size   (GtkTextView       *text_view,
-                                           gint               width,
-                                           gint               height);
+gint gtk_text_view_get_border_window_size (GtkTextView       *text_view,
+					   GtkTextWindowType  type);
 
 gboolean gtk_text_view_forward_display_line           (GtkTextView       *text_view,
                                                        GtkTextIter       *iter);
