@@ -191,9 +191,7 @@ void
 gtk_tree_append (GtkTree   *tree,
 		 GtkWidget *tree_item)
 {
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
-  g_return_if_fail (tree_item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (tree_item));
   
   gtk_tree_insert (tree, tree_item, -1);
@@ -203,9 +201,7 @@ void
 gtk_tree_prepend (GtkTree   *tree,
 		  GtkWidget *tree_item)
 {
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
-  g_return_if_fail (tree_item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (tree_item));
   
   gtk_tree_insert (tree, tree_item, 0);
@@ -218,9 +214,7 @@ gtk_tree_insert (GtkTree   *tree,
 {
   gint nchildren;
   
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
-  g_return_if_fail (tree_item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (tree_item));
   
   nchildren = g_list_length (tree->children);
@@ -234,17 +228,6 @@ gtk_tree_insert (GtkTree   *tree,
     tree->children = g_list_insert (tree->children, tree_item, position);
   
   gtk_widget_set_parent (tree_item, GTK_WIDGET (tree));
-  
-  if (GTK_WIDGET_REALIZED (tree_item->parent))
-    gtk_widget_realize (tree_item);
-
-  if (GTK_WIDGET_VISIBLE (tree_item->parent) && GTK_WIDGET_VISIBLE (tree_item))
-    {
-      if (GTK_WIDGET_MAPPED (tree_item->parent))
-	gtk_widget_map (tree_item);
-
-      gtk_widget_queue_resize (tree_item);
-    }
 }
 
 static void
@@ -253,7 +236,6 @@ gtk_tree_add (GtkContainer *container,
 {
   GtkTree *tree;
   
-  g_return_if_fail (container != NULL);
   g_return_if_fail (GTK_IS_TREE (container));
   g_return_if_fail (GTK_IS_TREE_ITEM (child));
   
@@ -263,17 +245,6 @@ gtk_tree_add (GtkContainer *container,
   
   gtk_widget_set_parent (child, GTK_WIDGET (container));
   
-  if (GTK_WIDGET_REALIZED (child->parent))
-    gtk_widget_realize (child);
-
-  if (GTK_WIDGET_VISIBLE (child->parent) && GTK_WIDGET_VISIBLE (child))
-    {
-      if (GTK_WIDGET_MAPPED (child->parent))
-	gtk_widget_map (child);
-
-      gtk_widget_queue_resize (child);
-    }
-  
   if (!tree->selection && (tree->selection_mode == GTK_SELECTION_BROWSE))
     gtk_tree_select_child (tree, child);
 }
@@ -285,7 +256,6 @@ gtk_tree_button_press (GtkWidget      *widget,
   GtkTree *tree;
   GtkWidget *item;
   
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_TREE (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
   
@@ -321,7 +291,6 @@ gtk_tree_button_release (GtkWidget      *widget,
   GtkTree *tree;
   GtkWidget *item;
   
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_TREE (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
   
@@ -339,7 +308,6 @@ gtk_tree_child_position (GtkTree   *tree,
   gint pos;
   
   
-  g_return_val_if_fail (tree != NULL, -1);
   g_return_val_if_fail (GTK_IS_TREE (tree), -1);
   g_return_val_if_fail (child != NULL, -1);
   
@@ -370,7 +338,6 @@ gtk_tree_clear_items (GtkTree *tree,
   guint nchildren;
   guint index;
   
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
   
   nchildren = g_list_length (tree->children);
@@ -406,7 +373,6 @@ gtk_tree_destroy (GtkObject *object)
   GtkWidget *child;
   GList *children;
   
-  g_return_if_fail (object != NULL);
   g_return_if_fail (GTK_IS_TREE (object));
   
   tree = GTK_TREE (object);
@@ -450,7 +416,6 @@ gtk_tree_forall (GtkContainer *container,
   GList *children;
   
   
-  g_return_if_fail (container != NULL);
   g_return_if_fail (GTK_IS_TREE (container));
   g_return_if_fail (callback != NULL);
   
@@ -569,7 +534,6 @@ static gint
 gtk_tree_motion_notify (GtkWidget      *widget,
 			GdkEventMotion *event)
 {
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_TREE (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
   
@@ -587,7 +551,6 @@ gtk_tree_realize (GtkWidget *widget)
   gint attributes_mask;
   
   
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_TREE (widget));
   
   GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
@@ -618,7 +581,6 @@ gtk_tree_remove_item (GtkTree      *container,
 {
   GList *item_list;
   
-  g_return_if_fail (container != NULL);
   g_return_if_fail (GTK_IS_TREE (container));
   g_return_if_fail (widget != NULL);
   g_return_if_fail (container == GTK_TREE (widget->parent));
@@ -655,7 +617,6 @@ gtk_tree_remove_items (GtkTree *tree,
   GtkTree *real_tree;
   GtkTree *root_tree;
   
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
   
 #ifdef TREE_DEBUG
@@ -845,9 +806,7 @@ void
 gtk_tree_select_child (GtkTree   *tree,
 		       GtkWidget *tree_item)
 {
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
-  g_return_if_fail (tree_item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (tree_item));
   
   gtk_signal_emit (GTK_OBJECT (tree), tree_signals[SELECT_CHILD], tree_item);
@@ -859,7 +818,6 @@ gtk_tree_select_item (GtkTree   *tree,
 {
   GList *tmp_list;
   
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
   
   tmp_list = g_list_nth (tree->children, item);
@@ -878,7 +836,6 @@ gtk_tree_size_allocate (GtkWidget     *widget,
   GList *children;
   
   
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_TREE (widget));
   g_return_if_fail (allocation != NULL);
   
@@ -937,7 +894,6 @@ gtk_tree_size_request (GtkWidget      *widget,
   GtkRequisition child_requisition;
   
   
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_TREE (widget));
   g_return_if_fail (requisition != NULL);
   
@@ -983,7 +939,6 @@ static void
 gtk_tree_unmap (GtkWidget *widget)
 {
   
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_TREE (widget));
   
   GTK_WIDGET_UNSET_FLAGS (widget, GTK_MAPPED);
@@ -995,9 +950,7 @@ void
 gtk_tree_unselect_child (GtkTree   *tree,
 			 GtkWidget *tree_item)
 {
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
-  g_return_if_fail (tree_item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (tree_item));
   
   gtk_signal_emit (GTK_OBJECT (tree), tree_signals[UNSELECT_CHILD], tree_item);
@@ -1009,7 +962,6 @@ gtk_tree_unselect_item (GtkTree *tree,
 {
   GList *tmp_list;
   
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
   
   tmp_list = g_list_nth (tree->children, item);
@@ -1026,9 +978,7 @@ gtk_real_tree_select_child (GtkTree   *tree,
   GList *tmp_list;
   GtkWidget *tmp_item;
   
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
-  g_return_if_fail (child != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (child));
 
   root_selection = tree->root_tree->selection;
@@ -1117,27 +1067,6 @@ gtk_real_tree_select_child (GtkTree   *tree,
       break;
       
     case GTK_SELECTION_MULTIPLE:
-      if (child->state == GTK_STATE_NORMAL)
-	{
-	  gtk_tree_item_select (GTK_TREE_ITEM (child));
-	  root_selection = g_list_prepend (root_selection, child);
-	  gtk_widget_ref (child);
-	  tree->root_tree->selection = root_selection;
-	  gtk_signal_emit (GTK_OBJECT (tree->root_tree), 
-			   tree_signals[SELECTION_CHANGED]);
-	}
-      else if (child->state == GTK_STATE_SELECTED)
-	{
-	  gtk_tree_item_deselect (GTK_TREE_ITEM (child));
-	  root_selection = g_list_remove (root_selection, child);
-	  gtk_widget_unref (child);
-	  tree->root_tree->selection = root_selection;
-	  gtk_signal_emit (GTK_OBJECT (tree->root_tree), 
-			   tree_signals[SELECTION_CHANGED]);
-	}
-      break;
-      
-    case GTK_SELECTION_EXTENDED:
       break;
     }
 }
@@ -1146,15 +1075,12 @@ static void
 gtk_real_tree_unselect_child (GtkTree   *tree,
 			      GtkWidget *child)
 {
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
-  g_return_if_fail (child != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (child));
   
   switch (tree->selection_mode)
     {
     case GTK_SELECTION_SINGLE:
-    case GTK_SELECTION_MULTIPLE:
     case GTK_SELECTION_BROWSE:
       if (child->state == GTK_STATE_SELECTED)
 	{
@@ -1167,7 +1093,7 @@ gtk_real_tree_unselect_child (GtkTree   *tree,
 	}
       break;
       
-    case GTK_SELECTION_EXTENDED:
+    case GTK_SELECTION_MULTIPLE:
       break;
     }
 }
@@ -1176,7 +1102,6 @@ void
 gtk_tree_set_selection_mode (GtkTree       *tree,
 			     GtkSelectionMode mode) 
 {
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
   
   tree->selection_mode = mode;
@@ -1186,7 +1111,6 @@ void
 gtk_tree_set_view_mode (GtkTree       *tree,
 			GtkTreeViewMode mode) 
 {
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
   
   tree->view_mode = mode;
@@ -1196,7 +1120,6 @@ void
 gtk_tree_set_view_lines (GtkTree       *tree,
 			 gboolean	flag) 
 {
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
   
   tree->view_line = flag;
