@@ -36,11 +36,13 @@ typedef struct _GtkTreePath         GtkTreePath;
 typedef struct _GtkTreeRowReference GtkTreeRowReference;
 typedef struct _GtkTreeModel        GtkTreeModel; /* Dummy typedef */
 typedef struct _GtkTreeModelIface   GtkTreeModelIface;
+typedef gboolean (* GtkTreeModelForeachFunc) (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data);
 
 
 typedef enum
 {
-  GTK_TREE_MODEL_ITERS_PERSIST = 1 << 0
+  GTK_TREE_MODEL_ITERS_PERSIST = 1 << 0,
+  GTK_TREE_MODEL_LIST_ONLY = 1 << 1,
 } GtkTreeModelFlags;
 
 struct _GtkTreeIter
@@ -56,23 +58,21 @@ struct _GtkTreeModelIface
   GTypeInterface g_iface;
 
   /* Signals */
-  void         (* range_changed)     (GtkTreeModel *tree_model,
-				      GtkTreePath  *start_path,
-				      GtkTreeIter  *start_iter,
-				      GtkTreePath  *end_path,
-				      GtkTreeIter  *end_iter);
-  void         (* inserted)          (GtkTreeModel *tree_model,
-				      GtkTreePath  *path,
-				      GtkTreeIter  *iter);
-  void         (* has_child_toggled) (GtkTreeModel *tree_model,
-				      GtkTreePath  *path,
-				      GtkTreeIter  *iter);
-  void         (* deleted)           (GtkTreeModel *tree_model,
-				      GtkTreePath  *path);
-  void         (* reordered)         (GtkTreeModel *tree_model,
-				      GtkTreePath  *path,
-				      GtkTreeIter  *iter,
-				      gint         *new_order);
+  void         (* row_changed)           (GtkTreeModel *tree_model,
+					  GtkTreePath  *path,
+					  GtkTreeIter  *iter);
+  void         (* row_inserted)          (GtkTreeModel *tree_model,
+					  GtkTreePath  *path,
+					  GtkTreeIter  *iter);
+  void         (* row_has_child_toggled) (GtkTreeModel *tree_model,
+					  GtkTreePath  *path,
+					  GtkTreeIter  *iter);
+  void         (* row_deleted)           (GtkTreeModel *tree_model,
+					  GtkTreePath  *path);
+  void         (* rows_reordered)        (GtkTreeModel *tree_model,
+					  GtkTreePath  *path,
+					  GtkTreeIter  *iter,
+					  gint         *new_order);
 
   /* Virtual Table */
   GtkTreeModelFlags (* get_flags)  (GtkTreeModel *tree_model);   
@@ -114,7 +114,7 @@ struct _GtkTreeModelIface
 
 /* GtkTreePath operations */
 GtkTreePath *gtk_tree_path_new              (void);
-GtkTreePath *gtk_tree_path_new_from_string  (gchar             *path);
+GtkTreePath *gtk_tree_path_new_from_string  (const gchar       *path);
 gchar       *gtk_tree_path_to_string        (GtkTreePath       *path);
 GtkTreePath *gtk_tree_path_new_root         (void);
 void         gtk_tree_path_append_index     (GtkTreePath       *path,
@@ -148,8 +148,8 @@ GtkTreeRowReference *gtk_tree_row_reference_new_proxy (GObject             *prox
 						       GtkTreeModel        *model,
 						       GtkTreePath         *path);
 GtkTreePath         *gtk_tree_row_reference_get_path  (GtkTreeRowReference *reference);
+gboolean             gtk_tree_row_reference_valid     (GtkTreeRowReference *reference);
 void                 gtk_tree_row_reference_free      (GtkTreeRowReference *reference);
-
 /* These two functions are only needed if you created the row reference with a
  * proxy object */
 void                 gtk_tree_row_reference_inserted  (GObject     *proxy,
@@ -176,7 +176,7 @@ GType             gtk_tree_model_get_column_type (GtkTreeModel *tree_model,
 gboolean          gtk_tree_model_get_iter        (GtkTreeModel *tree_model,
 						  GtkTreeIter  *iter,
 						  GtkTreePath  *path);
-gboolean          gtk_tree_model_get_first       (GtkTreeModel *tree_model,
+gboolean          gtk_tree_model_get_iter_root   (GtkTreeModel *tree_model,
 						  GtkTreeIter  *iter);
 GtkTreePath *     gtk_tree_model_get_path        (GtkTreeModel *tree_model,
 						  GtkTreeIter  *iter);
@@ -212,24 +212,28 @@ void              gtk_tree_model_get_valist      (GtkTreeModel *tree_model,
 						  va_list       var_args);
 
 
+void              gtk_tree_model_foreach         (GtkTreeModel            *model,
+						  GtkTreeModelForeachFunc  func,
+						  gpointer                 user_data);
+
+
 /* Signals */
-void gtk_tree_model_range_changed     (GtkTreeModel *tree_model,
-				       GtkTreePath  *start_path,
-				       GtkTreeIter  *start_iter,
-				       GtkTreePath  *end_path,
-				       GtkTreeIter  *end_iter);
-void gtk_tree_model_inserted          (GtkTreeModel *tree_model,
-				       GtkTreePath  *path,
-				       GtkTreeIter  *iter);
-void gtk_tree_model_has_child_toggled (GtkTreeModel *tree_model,
-				       GtkTreePath  *path,
-				       GtkTreeIter  *iter);
-void gtk_tree_model_deleted           (GtkTreeModel *tree_model,
-				       GtkTreePath  *path);
-void gtk_tree_model_reordered         (GtkTreeModel *tree_model,
-				       GtkTreePath  *path,
-				       GtkTreeIter  *iter,
-				       gint         *new_order);
+void gtk_tree_model_row_changed           (GtkTreeModel *tree_model,
+					   GtkTreePath  *start_path,
+					   GtkTreeIter  *start_iter);
+void gtk_tree_model_row_inserted          (GtkTreeModel *tree_model,
+					   GtkTreePath  *path,
+					   GtkTreeIter  *iter);
+void gtk_tree_model_row_has_child_toggled (GtkTreeModel *tree_model,
+					   GtkTreePath  *path,
+					   GtkTreeIter  *iter);
+void gtk_tree_model_row_deleted           (GtkTreeModel *tree_model,
+					   GtkTreePath  *path);
+void gtk_tree_model_rows_reordered        (GtkTreeModel *tree_model,
+					   GtkTreePath  *path,
+					   GtkTreeIter  *iter,
+					   gint         *new_order);
+
 
 
 
