@@ -21,6 +21,7 @@
 #define __GTK_TREE_MODEL_SORT_H__
 
 #include <gtk/gtktreemodel.h>
+#include <gtk/gtktreesortable.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -31,13 +32,11 @@ extern "C" {
 #define GTK_TREE_MODEL_SORT_CLASS(klass)		(GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_TREE_MODEL_SORT, GtkTreeModelSortClass))
 #define GTK_IS_TREE_MODEL_SORT(obj)			(GTK_CHECK_TYPE ((obj), GTK_TYPE_TREE_MODEL_SORT))
 #define GTK_IS_TREE_MODEL_SORT_CLASS(klass)		(GTK_CHECK_CLASS_TYPE ((obj), GTK_TYPE_TREE_MODEL_SORT))
+#define GTK_TREE_MODEL_SORT_GET_CLASS(obj)              (GTK_CHECK_GET_CLASS ((obj), GTK_TYPE_TREE_MODEL_SORT, GtkTreeModelSortClass))
 
 typedef struct _GtkTreeModelSort       GtkTreeModelSort;
 typedef struct _GtkTreeModelSortClass  GtkTreeModelSortClass;
 
-typedef gint (* GValueCompareFunc) (const GValue *a,
-				    const GValue *b);
-
 struct _GtkTreeModelSort
 {
   GObject parent;
@@ -47,13 +46,19 @@ struct _GtkTreeModelSort
   gint stamp;
   guint flags;
   GtkTreeModel *child_model;
-  gint sort_col;
-  GValueCompareFunc func;
+  gboolean cache_child_iters;
 
+  /* sort information */
+  GList *sort_list;
+  gint sort_column_id;
+  GtkSortType order;
+
+  /* signal ids */
   guint changed_id;
   guint inserted_id;
   guint has_child_toggled_id;
   guint deleted_id;
+  guint reordered_id;
 };
 
 struct _GtkTreeModelSortClass
@@ -64,16 +69,12 @@ struct _GtkTreeModelSortClass
 
 GtkType       gtk_tree_model_sort_get_type       (void);
 GtkTreeModel *gtk_tree_model_sort_new            (void);
-GtkTreeModel *gtk_tree_model_sort_new_with_model (GtkTreeModel      *child_model,
-						  GValueCompareFunc  func,
-						  gint               sort_col);
+GtkTreeModel *gtk_tree_model_sort_new_with_model (GtkTreeModel      *child_model);
 void          gtk_tree_model_sort_set_model      (GtkTreeModelSort  *tree_model_sort,
 						  GtkTreeModel      *child_model);
 GtkTreeModel *gtk_tree_model_sort_get_model      (GtkTreeModelSort  *tree_model);
 GtkTreePath  *gtk_tree_model_sort_convert_path   (GtkTreeModelSort  *tree_model_sort,
 						  GtkTreePath       *child_path);
-
-/* not implemented */
 void          gtk_tree_model_sort_convert_iter    (GtkTreeModelSort  *tree_model_sort,
                                                    GtkTreeIter       *sort_iter,
                                                    GtkTreeIter       *child_iter);
