@@ -48,8 +48,11 @@ struct _GtkTreeStore
   gint n_columns;
   gint sort_column_id;
   GList *sort_list;
-  GtkTreeSortOrder order;
+  GtkSortType order;
   GType *column_headers;
+  GtkTreeIterCompareFunc default_sort_func;
+  gpointer default_sort_data;
+  GtkDestroyNotify default_sort_destroy;
 };
 
 struct _GtkTreeStoreClass
@@ -59,15 +62,10 @@ struct _GtkTreeStoreClass
 
 
 GtkType       gtk_tree_store_get_type        (void);
-GtkTreeStore *gtk_tree_store_new             (void);
-
-GtkTreeStore *gtk_tree_store_new_with_types  (gint          n_columns,
+GtkTreeStore *gtk_tree_store_new             (gint          n_columns,
 					      ...);
-void          gtk_tree_store_set_n_columns   (GtkTreeStore *tree_store,
-					      gint          n_columns);
-void          gtk_tree_store_set_column_type (GtkTreeStore *tree_store,
-					      gint          column,
-					      GType         type);
+GtkTreeStore *gtk_tree_store_newv            (gint          n_columns,
+					      GType        *types);
 void          gtk_tree_store_set_value       (GtkTreeStore *tree_store,
 					      GtkTreeIter  *iter,
 					      gint          column,
@@ -103,7 +101,7 @@ gboolean      gtk_tree_store_is_ancestor     (GtkTreeStore *tree_store,
 					      GtkTreeIter  *descendant);
 gint          gtk_tree_store_iter_depth      (GtkTreeStore *tree_store,
 					      GtkTreeIter  *iter);
-
+void          gtk_tree_store_clear           (GtkTreeStore *tree_store);
 
 #ifdef __cplusplus
 }
