@@ -19,6 +19,7 @@
 #ifndef __GTK_TREE_VIEW_H__
 #define __GTK_TREE_VIEW_H__
 
+#include <gtk/gtkwidget.h>
 #include <gtk/gtkcontainer.h>
 #include <gtk/gtktreemodel.h>
 #include <gtk/gtktreeviewcolumn.h>
@@ -45,6 +46,7 @@ typedef enum
 #define GTK_TREE_VIEW_CLASS(klass)	(GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_TREE_VIEW, GtkTreeViewClass))
 #define GTK_IS_TREE_VIEW(obj)		(GTK_CHECK_TYPE ((obj), GTK_TYPE_TREE_VIEW))
 #define GTK_IS_TREE_VIEW_CLASS(klass)	(GTK_CHECK_CLASS_TYPE ((obj), GTK_TYPE_TREE_VIEW))
+#define GTK_TREE_VIEW_GET_CLASS(obj)    (GTK_CHECK_GET_CLASS ((obj), GTK_TYPE_TREE_VIEW, GtkTreeViewClass))
 
 typedef struct _GtkTreeView           GtkTreeView;
 typedef struct _GtkTreeViewClass      GtkTreeViewClass;
@@ -63,18 +65,38 @@ struct _GtkTreeViewClass
 {
   GtkContainerClass parent_class;
 
-  void     (* set_scroll_adjustments) (GtkTreeView       *tree_view,
-				       GtkAdjustment     *hadjustment,
-				       GtkAdjustment     *vadjustment);
-  void     (* row_activated)          (GtkTreeView       *tree_view,
-				       GtkTreePath       *path,
-				       GtkTreeViewColumn *column);
-  gboolean (* expand_row)             (GtkTreeView       *tree_view,
-				       GtkTreeIter       *iter,
-				       GtkTreePath       *path);
-  gboolean (* collapse_row)           (GtkTreeView       *tree_view,
-				       GtkTreeIter       *iter,
-				       GtkTreePath       *path);
+  void     (* set_scroll_adjustments)     (GtkTreeView       *tree_view,
+				           GtkAdjustment     *hadjustment,
+				           GtkAdjustment     *vadjustment);
+  void     (* row_activated)              (GtkTreeView       *tree_view,
+				           GtkTreePath       *path,
+					   GtkTreeViewColumn *column);
+  gboolean (* test_expand_row)            (GtkTreeView       *tree_view,
+				           GtkTreeIter       *iter,
+				           GtkTreePath       *path);
+  gboolean (* test_collapse_row)          (GtkTreeView       *tree_view,
+				           GtkTreeIter       *iter,
+				           GtkTreePath       *path);
+  void     (* row_expanded)               (GtkTreeView       *tree_view,
+				           GtkTreeIter       *iter,
+				           GtkTreePath       *path);
+  void     (* row_collapsed)              (GtkTreeView       *tree_view,
+				           GtkTreeIter       *iter,
+				           GtkTreePath       *path);
+  void     (* columns_changed)            (GtkTreeView       *tree_view);
+
+  /* Key Binding signals */
+  void     (* move_cursor)                (GtkTreeView       *tree_view,
+				           GtkMovementStep    step,
+				           gint               count);
+  void     (* select_cursor_row)          (GtkTreeView       *tree_view);
+  void     (* toggle_cursor_row)          (GtkTreeView       *tree_view);
+  void     (* expand_collapse_cursor_row) (GtkTreeView       *tree_view,
+					   gboolean           logical,
+					   gboolean           expand,
+					   gboolean           open_all);
+  void     (* select_cursor_parent)       (GtkTreeView       *tree_view);
+  void     (* start_interactive_search)   (GtkTreeView       *tree_view); 
 };
 
 
@@ -95,6 +117,11 @@ typedef gboolean (* GtkTreeViewDroppableFunc)  (GtkTreeView             *tree_vi
 						GtkTreePath             *path,
 						GtkTreeViewDropPosition *pos,
 						gpointer                 user_data);
+typedef gboolean (*GtkTreeViewSearchEqualFunc) (GtkTreeModel            *model,
+						gint                     column,
+						gchar                   *key,
+						GtkTreeIter             *iter,
+						gpointer                 search_data);
 
 
 /* Creators */
@@ -136,6 +163,13 @@ gint                   gtk_tree_view_insert_column_with_attributes (GtkTreeView
 								    gchar                     *title,
 								    GtkCellRenderer           *cell,
 								    ...);
+gint                   gtk_tree_view_insert_column_with_data_func  (GtkTreeView               *tree_view,
+								    gint                       position,
+								    gchar                     *title,
+								    GtkCellRenderer           *cell,
+                                                                    GtkTreeCellDataFunc        func,
+                                                                    gpointer                   data,
+                                                                    GDestroyNotify             dnotify);
 GtkTreeViewColumn     *gtk_tree_view_get_column                    (GtkTreeView               *tree_view,
 								    gint                       n);
 GList                 *gtk_tree_view_get_columns                   (GtkTreeView               *tree_view);
@@ -143,8 +177,12 @@ void                   gtk_tree_view_move_column_after             (GtkTreeView
 								    GtkTreeViewColumn         *column,
 								    GtkTreeViewColumn         *base_column);
 void                   gtk_tree_view_set_expander_column           (GtkTreeView               *tree_view,
-								    gint                       col);
-gint                   gtk_tree_view_get_expander_column           (GtkTreeView               *tree_view);
+								    GtkTreeViewColumn         *column);
+GtkTreeViewColumn     *gtk_tree_view_get_expander_column           (GtkTreeView               *tree_view);
+void                   gtk_tree_view_set_column_drag_function      (GtkTreeView               *tree_view,
+								    GtkTreeViewColumnDropFunc  func,
+								    gpointer                   user_data,
+								    GtkDestroyNotify           destroy);
 
 /* Actions */
 void                   gtk_tree_view_scroll_to_point               (GtkTreeView               *tree_view,
@@ -153,6 +191,7 @@ void                   gtk_tree_view_scroll_to_point               (GtkTreeView
 void                   gtk_tree_view_scroll_to_cell                (GtkTreeView               *tree_view,
 								    GtkTreePath               *path,
 								    GtkTreeViewColumn         *column,
+								    gboolean                   use_align,
 								    gfloat                     row_align,
 								    gfloat                     col_align);
 void                   gtk_tree_view_row_activated                 (GtkTreeView               *tree_view,
@@ -168,6 +207,19 @@ gboolean               gtk_tree_view_collapse_row                  (GtkTreeView
 void                   gtk_tree_view_map_expanded_rows             (GtkTreeView               *tree_view,
 								    GtkTreeViewMappingFunc     func,
 								    gpointer                   data);
+gboolean               gtk_tree_view_row_expanded                  (GtkTreeView               *tree_view,
+								    GtkTreePath               *path);
+void                   gtk_tree_view_set_reorderable               (GtkTreeView               *tree_view,
+								    gboolean                   reorderable);
+gboolean               gtk_tree_view_get_reorderable               (GtkTreeView               *tree_view);
+void                   gtk_tree_view_set_cursor                    (GtkTreeView               *tree_view,
+								    GtkTreePath               *path,
+								    GtkTreeViewColumn         *focus_column,
+								    gboolean                   start_editing);
+void                   gtk_tree_view_get_cursor                    (GtkTreeView               *tree_view,
+								    GtkTreePath              **path,
+								    GtkTreeViewColumn        **focus_column);
+
 
 /* Layout information */
 gboolean               gtk_tree_view_get_path_at_pos               (GtkTreeView               *tree_view,
@@ -216,6 +268,7 @@ void                   gtk_tree_view_set_rows_drag_dest            (GtkTreeView
 void                   gtk_tree_view_unset_rows_drag_source        (GtkTreeView               *tree_view);
 void                   gtk_tree_view_unset_rows_drag_dest          (GtkTreeView               *tree_view);
 
+
 /* These are useful to implement your own custom stuff. */
 void                   gtk_tree_view_set_drag_dest_row             (GtkTreeView               *tree_view,
 								    GtkTreePath               *path,
@@ -231,6 +284,33 @@ gboolean               gtk_tree_view_get_dest_row_at_pos           (GtkTreeView
 GdkPixmap             *gtk_tree_view_create_row_drag_icon          (GtkTreeView               *tree_view,
 								    GtkTreePath               *path);
 
+/* Interactive search */
+void                       gtk_tree_view_set_enable_search     (GtkTreeView                *tree_view,
+								gboolean                    use_search);
+gboolean                   gtk_tree_view_get_enable_search     (GtkTreeView                *tree_view);
+gint                       gtk_tree_view_get_search_column     (GtkTreeView                *tree_view);
+void                       gtk_tree_view_set_search_column     (GtkTreeView                *tree_view,
+								gint                        column);
+GtkTreeViewSearchEqualFunc gtk_tree_view_get_search_equal_func (GtkTreeView                *tree_view);
+void                       gtk_tree_view_set_search_equal_func (GtkTreeView                *tree_view,
+								GtkTreeViewSearchEqualFunc  search_compare_func,
+								gpointer                    search_data,
+								GtkDestroyNotify            search_destroy);
+
+/* This function should really never be used.  It is just for use by ATK.
+ */
+typedef void (* GtkTreeDestroyCountFunc)  (GtkTreeView             *tree_view,
+					   GtkTreePath             *path,
+					   gint                     children,
+					   gpointer                 user_data);
+void gtk_tree_view_set_destroy_count_func (GtkTreeView             *tree_view,
+					   GtkTreeDestroyCountFunc  func,
+					   gpointer                 data,
+					   GtkDestroyNotify         destroy);
+
+
+
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
