@@ -84,11 +84,16 @@ static const GtkEnumValue _gtk_cell_renderer_state_values[] = {
   { GTK_CELL_RENDERER_SORTED, "GTK_CELL_RENDERER_SORTED", "sorted" },
   { 0, NULL, NULL }
 };
+static const GtkEnumValue _gtk_cell_renderer_mode_values[] = {
+  { GTK_CELL_RENDERER_MODE_INERT, "GTK_CELL_RENDERER_MODE_INERT", "inert" },
+  { GTK_CELL_RENDERER_MODE_ACTIVATABLE, "GTK_CELL_RENDERER_MODE_ACTIVATABLE", "activatable" },
+  { GTK_CELL_RENDERER_MODE_EDITABLE, "GTK_CELL_RENDERER_MODE_EDITABLE", "editable" },
+  { 0, NULL, NULL }
+};
 
 /* enumerations from "gtkdebug.h" */
 static const GtkEnumValue _gtk_debug_flag_values[] = {
   { GTK_DEBUG_MISC, "GTK_DEBUG_MISC", "misc" },
-  { GTK_DEBUG_DND, "GTK_DEBUG_DND", "dnd" },
   { GTK_DEBUG_PLUGSOCKET, "GTK_DEBUG_PLUGSOCKET", "plugsocket" },
   { GTK_DEBUG_TEXT, "GTK_DEBUG_TEXT", "text" },
   { GTK_DEBUG_TREE, "GTK_DEBUG_TREE", "tree" },
@@ -200,12 +205,20 @@ static const GtkEnumValue _gtk_direction_type_values[] = {
   { GTK_DIR_RIGHT, "GTK_DIR_RIGHT", "right" },
   { 0, NULL, NULL }
 };
+static const GtkEnumValue _gtk_expander_style_values[] = {
+  { GTK_EXPANDER_COLLAPSED, "GTK_EXPANDER_COLLAPSED", "collapsed" },
+  { GTK_EXPANDER_SEMI_COLLAPSED, "GTK_EXPANDER_SEMI_COLLAPSED", "semi-collapsed" },
+  { GTK_EXPANDER_SEMI_EXPANDED, "GTK_EXPANDER_SEMI_EXPANDED", "semi-expanded" },
+  { GTK_EXPANDER_EXPANDED, "GTK_EXPANDER_EXPANDED", "expanded" },
+  { 0, NULL, NULL }
+};
 static const GtkEnumValue _gtk_icon_size_values[] = {
   { GTK_ICON_SIZE_INVALID, "GTK_ICON_SIZE_INVALID", "invalid" },
   { GTK_ICON_SIZE_MENU, "GTK_ICON_SIZE_MENU", "menu" },
   { GTK_ICON_SIZE_SMALL_TOOLBAR, "GTK_ICON_SIZE_SMALL_TOOLBAR", "small-toolbar" },
   { GTK_ICON_SIZE_LARGE_TOOLBAR, "GTK_ICON_SIZE_LARGE_TOOLBAR", "large-toolbar" },
   { GTK_ICON_SIZE_BUTTON, "GTK_ICON_SIZE_BUTTON", "button" },
+  { GTK_ICON_SIZE_DND, "GTK_ICON_SIZE_DND", "dnd" },
   { GTK_ICON_SIZE_DIALOG, "GTK_ICON_SIZE_DIALOG", "dialog" },
   { 0, NULL, NULL }
 };
@@ -284,6 +297,7 @@ static const GtkEnumValue _gtk_path_priority_type_values[] = {
   { GTK_PATH_PRIO_LOWEST, "GTK_PATH_PRIO_LOWEST", "lowest" },
   { GTK_PATH_PRIO_GTK, "GTK_PATH_PRIO_GTK", "gtk" },
   { GTK_PATH_PRIO_APPLICATION, "GTK_PATH_PRIO_APPLICATION", "application" },
+  { GTK_PATH_PRIO_THEME, "GTK_PATH_PRIO_THEME", "theme" },
   { GTK_PATH_PRIO_RC, "GTK_PATH_PRIO_RC", "rc" },
   { GTK_PATH_PRIO_HIGHEST, "GTK_PATH_PRIO_HIGHEST", "highest" },
   { GTK_PATH_PRIO_MASK, "GTK_PATH_PRIO_MASK", "mask" },
@@ -336,11 +350,11 @@ static const GtkEnumValue _gtk_signal_run_type_values[] = {
 };
 static const GtkEnumValue _gtk_scroll_type_values[] = {
   { GTK_SCROLL_NONE, "GTK_SCROLL_NONE", "none" },
+  { GTK_SCROLL_JUMP, "GTK_SCROLL_JUMP", "jump" },
   { GTK_SCROLL_STEP_BACKWARD, "GTK_SCROLL_STEP_BACKWARD", "step-backward" },
   { GTK_SCROLL_STEP_FORWARD, "GTK_SCROLL_STEP_FORWARD", "step-forward" },
   { GTK_SCROLL_PAGE_BACKWARD, "GTK_SCROLL_PAGE_BACKWARD", "page-backward" },
   { GTK_SCROLL_PAGE_FORWARD, "GTK_SCROLL_PAGE_FORWARD", "page-forward" },
-  { GTK_SCROLL_JUMP, "GTK_SCROLL_JUMP", "jump" },
   { GTK_SCROLL_STEP_UP, "GTK_SCROLL_STEP_UP", "step-up" },
   { GTK_SCROLL_STEP_DOWN, "GTK_SCROLL_STEP_DOWN", "step-down" },
   { GTK_SCROLL_PAGE_UP, "GTK_SCROLL_PAGE_UP", "page-up" },
@@ -349,6 +363,8 @@ static const GtkEnumValue _gtk_scroll_type_values[] = {
   { GTK_SCROLL_STEP_RIGHT, "GTK_SCROLL_STEP_RIGHT", "step-right" },
   { GTK_SCROLL_PAGE_LEFT, "GTK_SCROLL_PAGE_LEFT", "page-left" },
   { GTK_SCROLL_PAGE_RIGHT, "GTK_SCROLL_PAGE_RIGHT", "page-right" },
+  { GTK_SCROLL_START, "GTK_SCROLL_START", "start" },
+  { GTK_SCROLL_END, "GTK_SCROLL_END", "end" },
   { 0, NULL, NULL }
 };
 static const GtkEnumValue _gtk_selection_mode_values[] = {
@@ -391,13 +407,6 @@ static const GtkEnumValue _gtk_toolbar_style_values[] = {
   { GTK_TOOLBAR_BOTH_HORIZ, "GTK_TOOLBAR_BOTH_HORIZ", "both-horiz" },
   { 0, NULL, NULL }
 };
-static const GtkEnumValue _gtk_trough_type_values[] = {
-  { GTK_TROUGH_NONE, "GTK_TROUGH_NONE", "none" },
-  { GTK_TROUGH_START, "GTK_TROUGH_START", "start" },
-  { GTK_TROUGH_END, "GTK_TROUGH_END", "end" },
-  { GTK_TROUGH_JUMP, "GTK_TROUGH_JUMP", "jump" },
-  { 0, NULL, NULL }
-};
 static const GtkEnumValue _gtk_update_type_values[] = {
   { GTK_UPDATE_CONTINUOUS, "GTK_UPDATE_CONTINUOUS", "continuous" },
   { GTK_UPDATE_DISCONTINUOUS, "GTK_UPDATE_DISCONTINUOUS", "discontinuous" },
@@ -443,6 +452,7 @@ static const GtkEnumValue _gtk_image_type_values[] = {
   { GTK_IMAGE_PIXBUF, "GTK_IMAGE_PIXBUF", "pixbuf" },
   { GTK_IMAGE_STOCK, "GTK_IMAGE_STOCK", "stock" },
   { GTK_IMAGE_ICON_SET, "GTK_IMAGE_ICON_SET", "icon-set" },
+  { GTK_IMAGE_ANIMATION, "GTK_IMAGE_ANIMATION", "animation" },
   { 0, NULL, NULL }
 };
 
@@ -488,14 +498,6 @@ static const GtkEnumValue _gtk_arg_flags_values[] = {
   { 0, NULL, NULL }
 };
 
-/* enumerations from "gtkpacker.h" */
-static const GtkEnumValue _gtk_packer_options_values[] = {
-  { GTK_PACK_EXPAND, "GTK_PACK_EXPAND", "expand" },
-  { GTK_FILL_X, "GTK_FILL_X", "x" },
-  { GTK_FILL_Y, "GTK_FILL_Y", "y" },
-  { 0, NULL, NULL }
-};
-
 /* enumerations from "gtkprivate.h" */
 static const GtkEnumValue _gtk_private_flags_values[] = {
   { PRIVATE_GTK_USER_STYLE, "PRIVATE_GTK_USER_STYLE", "user-style" },
@@ -507,6 +509,7 @@ static const GtkEnumValue _gtk_private_flags_values[] = {
   { PRIVATE_GTK_DIRECTION_SET, "PRIVATE_GTK_DIRECTION_SET", "direction-set" },
   { PRIVATE_GTK_DIRECTION_LTR, "PRIVATE_GTK_DIRECTION_LTR", "direction-ltr" },
   { PRIVATE_GTK_ANCHORED, "PRIVATE_GTK_ANCHORED", "anchored" },
+  { PRIVATE_GTK_CHILD_VISIBLE, "PRIVATE_GTK_CHILD_VISIBLE", "child-visible" },
   { 0, NULL, NULL }
 };
 
@@ -560,6 +563,7 @@ static const GtkEnumValue _gtk_rc_token_type_values[] = {
   { GTK_RC_TOKEN_LOWEST, "GTK_RC_TOKEN_LOWEST", "lowest" },
   { GTK_RC_TOKEN_GTK, "GTK_RC_TOKEN_GTK", "gtk" },
   { GTK_RC_TOKEN_APPLICATION, "GTK_RC_TOKEN_APPLICATION", "application" },
+  { GTK_RC_TOKEN_THEME, "GTK_RC_TOKEN_THEME", "theme" },
   { GTK_RC_TOKEN_RC, "GTK_RC_TOKEN_RC", "rc" },
   { GTK_RC_TOKEN_HIGHEST, "GTK_RC_TOKEN_HIGHEST", "highest" },
   { GTK_RC_TOKEN_ENGINE, "GTK_RC_TOKEN_ENGINE", "engine" },
@@ -573,6 +577,15 @@ static const GtkEnumValue _gtk_rc_token_type_values[] = {
   { 0, NULL, NULL }
 };
 
+/* enumerations from "gtksizegroup.h" */
+static const GtkEnumValue _gtk_size_group_mode_values[] = {
+  { GTK_SIZE_GROUP_NONE, "GTK_SIZE_GROUP_NONE", "none" },
+  { GTK_SIZE_GROUP_HORIZONTAL, "GTK_SIZE_GROUP_HORIZONTAL", "horizontal" },
+  { GTK_SIZE_GROUP_VERTICAL, "GTK_SIZE_GROUP_VERTICAL", "vertical" },
+  { GTK_SIZE_GROUP_BOTH, "GTK_SIZE_GROUP_BOTH", "both" },
+  { 0, NULL, NULL }
+};
+
 /* enumerations from "gtkspinbutton.h" */
 static const GtkEnumValue _gtk_spin_button_update_policy_values[] = {
   { GTK_UPDATE_ALWAYS, "GTK_UPDATE_ALWAYS", "always" },
@@ -626,21 +639,8 @@ static const GtkEnumValue _gtk_tree_view_mode_values[] = {
 
 /* enumerations from "gtktreemodel.h" */
 static const GtkEnumValue _gtk_tree_model_flags_values[] = {
-  { GTK_TREE_MODEL_ITERS_PERSIST, "GTK_TREE_MODEL_ITERS_PERSIST", "persist" },
-  { 0, NULL, NULL }
-};
-
-/* enumerations from "gtktreeselection.h" */
-static const GtkEnumValue _gtk_tree_selection_mode_values[] = {
-  { GTK_TREE_SELECTION_SINGLE, "GTK_TREE_SELECTION_SINGLE", "single" },
-  { GTK_TREE_SELECTION_MULTI, "GTK_TREE_SELECTION_MULTI", "multi" },
-  { 0, NULL, NULL }
-};
-
-/* enumerations from "gtktreesortable.h" */
-static const GtkEnumValue _gtk_tree_sort_order_values[] = {
-  { GTK_TREE_SORT_ASCENDING, "GTK_TREE_SORT_ASCENDING", "ascending" },
-  { GTK_TREE_SORT_DESCENDING, "GTK_TREE_SORT_DESCENDING", "descending" },
+  { GTK_TREE_MODEL_ITERS_PERSIST, "GTK_TREE_MODEL_ITERS_PERSIST", "iters-persist" },
+  { GTK_TREE_MODEL_LIST_ONLY, "GTK_TREE_MODEL_LIST_ONLY", "list-only" },
   { 0, NULL, NULL }
 };
 
@@ -655,7 +655,8 @@ static const GtkEnumValue _gtk_tree_view_drop_position_values[] = {
 
 /* enumerations from "gtktreeviewcolumn.h" */
 static const GtkEnumValue _gtk_tree_view_column_sizing_values[] = {
-  { GTK_TREE_VIEW_COLUMN_RESIZEABLE, "GTK_TREE_VIEW_COLUMN_RESIZEABLE", "resizeable" },
+  { GTK_TREE_VIEW_COLUMN_GROW_ONLY, "GTK_TREE_VIEW_COLUMN_GROW_ONLY", "grow-only" },
+  { GTK_TREE_VIEW_COLUMN_RESIZABLE, "GTK_TREE_VIEW_COLUMN_RESIZABLE", "resizable" },
   { GTK_TREE_VIEW_COLUMN_AUTOSIZE, "GTK_TREE_VIEW_COLUMN_AUTOSIZE", "autosize" },
   { GTK_TREE_VIEW_COLUMN_FIXED, "GTK_TREE_VIEW_COLUMN_FIXED", "fixed" },
   { 0, NULL, NULL }
@@ -683,6 +684,11 @@ static const GtkEnumValue _gtk_widget_flags_values[] = {
   { GTK_DOUBLE_BUFFERED, "GTK_DOUBLE_BUFFERED", "double-buffered" },
   { 0, NULL, NULL }
 };
+static const GtkEnumValue _gtk_widget_help_type_values[] = {
+  { GTK_WIDGET_HELP_TOOLTIP, "GTK_WIDGET_HELP_TOOLTIP", "tooltip" },
+  { GTK_WIDGET_HELP_WHATS_THIS, "GTK_WIDGET_HELP_WHATS_THIS", "whats-this" },
+  { 0, NULL, NULL }
+};
 
 /* Generated data ends here */
 
