@@ -24,6 +24,8 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <string.h> /* strcmp */
+
 #include "gtktypeutils.h"
 #include "gtkobject.h"
 
@@ -60,7 +62,7 @@ gtk_type_class (GtkType type)
   gpointer class;
 
   if (!G_TYPE_IS_ENUM (type) && !G_TYPE_IS_FLAGS (type))
-    g_return_val_if_fail (GTK_TYPE_IS_OBJECT (type), NULL);
+    g_return_val_if_fail (G_TYPE_IS_OBJECT (type), NULL);
 
   /* ok, this is a bit ugly, GLib reference counts classes,
    * and gtk_type_class() used to always return static classes.
@@ -94,7 +96,7 @@ gtk_type_new (GtkType type)
 
   g_return_val_if_fail (GTK_TYPE_IS_OBJECT (type), NULL);
 
-  object = g_type_create_instance (type);
+  object = g_object_new (type, NULL);
 
   return object;
 }
@@ -102,6 +104,7 @@ gtk_type_new (GtkType type)
 /* includes for various places
  * with enum definitions
  */
+#define GTK_ENABLE_BROKEN
 #include "makeenums.h"
 /* type variable declarations
  */
@@ -116,24 +119,17 @@ GType GTK_TYPE_IDENTIFIER = 0;
 #else
 #define IMPORT
 #endif
-extern IMPORT gboolean glib_debug_objects;
 
 #include <gtk.h>	/* for gtktypebuiltins_ids.c */
 #include <gdk.h>	/* gtktypebuiltins_ids.c */
 
 void
-gtk_type_init (void)
+gtk_type_init (GTypeDebugFlags debug_flags)
 {
   static gboolean initialized = FALSE;
   
   if (!initialized)
     {
-      static const struct {
-	GtkType type_id;
-	gchar *name;
-      } fundamental_info[] = {
-	{ GTK_TYPE_SIGNAL,	"GtkSignal" },
-      };
       static struct {
 	gchar              *type_name;
 	GtkType            *type_id;
@@ -144,33 +140,19 @@ gtk_type_init (void)
 #include "gtktypebuiltins_ids.c"	/* type entries */
 	{ NULL }
       };
-      GTypeFundamentalInfo finfo = { 0, };
       GTypeInfo tinfo = { 0, };
-      GtkType type_id;
       guint i;
 
       initialized = TRUE;
 
-      glib_debug_objects = (gtk_debug_flags & GTK_DEBUG_OBJECTS) != 0;
-      
       /* initialize GLib type system
        */
-      g_type_init ();
+      g_type_init_with_debug_flags (debug_flags);
       
       /* GTK_TYPE_OBJECT
        */
       gtk_object_get_type ();
 
-      /* compatibility fundamentals
-       */
-      for (i = 0; i < sizeof (fundamental_info) / sizeof (fundamental_info[0]); i++)
-	{
-	  type_id = g_type_register_fundamental (fundamental_info[i].type_id,
-						 fundamental_info[i].name,
-						 &tinfo, &finfo, 0);
-	  g_assert (type_id == fundamental_info[i].type_id);
-	}
-
       /* GTK_TYPE_IDENTIFIER
        */
       GTK_TYPE_IDENTIFIER = g_type_register_static (G_TYPE_STRING, "GtkIdentifier", &tinfo, 0);
