@@ -50,8 +50,6 @@ static void gtk_viewport_get_property             (GObject         *object,
 static void gtk_viewport_set_scroll_adjustments	  (GtkViewport	    *viewport,
 						   GtkAdjustment    *hadjustment,
 						   GtkAdjustment    *vadjustment);
-static void gtk_viewport_map                      (GtkWidget        *widget);
-static void gtk_viewport_unmap                    (GtkWidget        *widget);
 static void gtk_viewport_realize                  (GtkWidget        *widget);
 static void gtk_viewport_unrealize                (GtkWidget        *widget);
 static void gtk_viewport_paint                    (GtkWidget        *widget,
@@ -116,8 +114,6 @@ gtk_viewport_class_init (GtkViewportClass *class)
   gobject_class->get_property = gtk_viewport_get_property;
   object_class->destroy = gtk_viewport_destroy;
   
-  widget_class->map = gtk_viewport_map;
-  widget_class->unmap = gtk_viewport_unmap;
   widget_class->realize = gtk_viewport_realize;
   widget_class->unrealize = gtk_viewport_unrealize;
   widget_class->expose_event = gtk_viewport_expose;
@@ -269,7 +265,6 @@ gtk_viewport_destroy (GtkObject *object)
 GtkAdjustment*
 gtk_viewport_get_hadjustment (GtkViewport *viewport)
 {
-  g_return_val_if_fail (viewport != NULL, NULL);
   g_return_val_if_fail (GTK_IS_VIEWPORT (viewport), NULL);
 
   if (!viewport->hadjustment)
@@ -281,7 +276,6 @@ gtk_viewport_get_hadjustment (GtkViewport *viewport)
 GtkAdjustment*
 gtk_viewport_get_vadjustment (GtkViewport *viewport)
 {
-  g_return_val_if_fail (viewport != NULL, NULL);
   g_return_val_if_fail (GTK_IS_VIEWPORT (viewport), NULL);
 
   if (!viewport->vadjustment)
@@ -294,7 +288,6 @@ void
 gtk_viewport_set_hadjustment (GtkViewport   *viewport,
 			      GtkAdjustment *adjustment)
 {
-  g_return_if_fail (viewport != NULL);
   g_return_if_fail (GTK_IS_VIEWPORT (viewport));
   if (adjustment)
     g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
@@ -333,7 +326,6 @@ void
 gtk_viewport_set_vadjustment (GtkViewport   *viewport,
 			      GtkAdjustment *adjustment)
 {
-  g_return_if_fail (viewport != NULL);
   g_return_if_fail (GTK_IS_VIEWPORT (viewport));
   if (adjustment)
     g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
@@ -383,7 +375,6 @@ void
 gtk_viewport_set_shadow_type (GtkViewport   *viewport,
 			      GtkShadowType  type)
 {
-  g_return_if_fail (viewport != NULL);
   g_return_if_fail (GTK_IS_VIEWPORT (viewport));
 
   if ((GtkShadowType) viewport->shadow_type != type)
@@ -400,35 +391,21 @@ gtk_viewport_set_shadow_type (GtkViewport   *viewport,
   g_object_notify (G_OBJECT (viewport), "shadow_type");
 }
 
-
-static void
-gtk_viewport_map (GtkWidget *widget)
+/**
+ * gtk_viewport_get_shadow_type:
+ * @viewport: a #GtkViewport
+ *
+ * Gets the shadow type of the #GtkViewport. See
+ * gtk_viewport_set_shadow_type().
+ 
+ * Return value: the shadow type 
+ **/
+GtkShadowType
+gtk_viewport_get_shadow_type (GtkViewport *viewport)
 {
-  GtkBin *bin;
+  g_return_val_if_fail (GTK_IS_VIEWPORT (viewport), GTK_SHADOW_NONE);
 
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_VIEWPORT (widget));
-
-  GTK_WIDGET_SET_FLAGS (widget, GTK_MAPPED);
-  bin = GTK_BIN (widget);
-
-  if (bin->child &&
-      GTK_WIDGET_VISIBLE (bin->child) &&
-      !GTK_WIDGET_MAPPED (bin->child))
-    gtk_widget_map (bin->child);
-
-  gdk_window_show (widget->window);
-}
-
-static void
-gtk_viewport_unmap (GtkWidget *widget)
-{
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_VIEWPORT (widget));
-
-  GTK_WIDGET_UNSET_FLAGS (widget, GTK_MAPPED);
-  
-  gdk_window_hide (widget->window);
+  return viewport->shadow_type;
 }
 
 static void
@@ -441,7 +418,6 @@ gtk_viewport_realize (GtkWidget *widget)
   gint event_mask;
   gint border_width;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_VIEWPORT (widget));
 
   border_width = GTK_CONTAINER (widget)->border_width;
@@ -523,7 +499,6 @@ gtk_viewport_unrealize (GtkWidget *widget)
 {
   GtkViewport *viewport;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_VIEWPORT (widget));
 
   viewport = GTK_VIEWPORT (widget);
@@ -546,7 +521,6 @@ gtk_viewport_paint (GtkWidget    *widget,
 {
   GtkViewport *viewport;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_VIEWPORT (widget));
   g_return_if_fail (area != NULL);
 
@@ -568,7 +542,6 @@ gtk_viewport_expose (GtkWidget      *widget,
   GtkBin *bin;
   GdkEventExpose child_event;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_VIEWPORT (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -603,7 +576,6 @@ gtk_viewport_add (GtkContainer *container,
 {
   GtkBin *bin;
 
-  g_return_if_fail (container != NULL);
   g_return_if_fail (GTK_IS_VIEWPORT (container));
   g_return_if_fail (child != NULL);
 
@@ -623,7 +595,6 @@ gtk_viewport_size_request (GtkWidget      *widget,
   GtkBin *bin;
   GtkRequisition child_requisition;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_VIEWPORT (widget));
   g_return_if_fail (requisition != NULL);
 
@@ -654,7 +625,6 @@ gtk_viewport_size_allocate (GtkWidget     *widget,
   gint hval, vval;
   gint border_width;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_VIEWPORT (widget));
   g_return_if_fail (allocation != NULL);
 
@@ -769,7 +739,6 @@ gtk_viewport_adjustment_changed (GtkAdjustment *adjustment,
   GtkViewport *viewport;
 
   g_return_if_fail (adjustment != NULL);
-  g_return_if_fail (data != NULL);
   g_return_if_fail (GTK_IS_VIEWPORT (data));
 
   viewport = GTK_VIEWPORT (data);
@@ -784,7 +753,6 @@ gtk_viewport_adjustment_value_changed (GtkAdjustment *adjustment,
   GtkAllocation child_allocation;
 
   g_return_if_fail (adjustment != NULL);
-  g_return_if_fail (data != NULL);
   g_return_if_fail (GTK_IS_VIEWPORT (data));
 
   viewport = GTK_VIEWPORT (data);
